<?PHP
/*
 * VipsPlugin.class.php - Vips plugin class for Stud.IP
 * Copyright (c) 2007  Elmar Ludwig
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

// set include path
$inc_path = ini_get('include_path');
$inc_path .= PATH_SEPARATOR . dirname(__FILE__);
if (version_compare(PHP_VERSION, '5', '>='))
    $inc_path .= PATH_SEPARATOR . dirname(__FILE__) . '/php5';
else
    $inc_path .= PATH_SEPARATOR . dirname(__FILE__) . '/php4';
ini_set('include_path', $inc_path);

require_once 'lib/classes/Config.class.php';

// require domxml
if (version_compare(PHP_VERSION, '5', '>='))
    require_once 'vendor/php4-to-php5/domxml-php4-to-php5.php';

// require flexi templates
require_once 'vendor/flexi/flexi.php';

// require Creole
require_once 'creole/Creole.php';

require_once 'vips.config.php';
require_once 'vips_common.inc.php';

/**
 * Vips plugin class for Stud.IP
 */
class VipsPlugin extends AbstractStudIPStandardPlugin
{
    var $active_tests;
    var $active_exams;

    function VipsPlugin ()
    {
        global $g_outlet, $vips_db;

        parent::AbstractStudIPStandardPlugin();

        if (version_compare(PHP_VERSION, '5', '>='))
            $options = 0;
        else
            $options = Creole::NO_ASSOC_LOWER();

        if (class_exists('DBManager'))
            $driver = 'mysql-pdo';
        else
            $driver = 'mysql';

        $g_outlet = enter();

        Creole::registerDriver('mysql-pdo', 'creole.drivers.mysql-pdo.MySQLConnection');
        $vips_db = Creole::getConnection(array(
                                'phptype'  => $driver,
                                'hostspec' => $g_outlet->db_host,
                                'username' => $g_outlet->db_user,
                                'password' => $g_outlet->db_password,
                                'database' => $g_outlet->db_database
                        ), $options);

        // this plugin wants its own icon
        $this->setPluginiconname("images/vips_1.gif");
        $this->setChangeIndicatorIconName('images/vipsr_1.gif');

        // navigation
        $navigation =& new PluginNavigation();
        $navigation->setDisplayname(_('bungsblatt'));
        $navigation->addLinkParam('action', 'sheets');

        $admin_nav =& new PluginNavigation();
        $admin_nav->setDisplayname(_('Klausur'));
        $admin_nav->addLinkParam('action', 'exams');
        $navigation->addSubmenu($admin_nav);

        $admin_nav =& new PluginNavigation();
        $admin_nav->setDisplayname(_('Gruppenverwaltung'));
        $admin_nav->addLinkParam('action', 'groups');
        $navigation->addSubmenu($admin_nav);

        $admin_nav =& new PluginNavigation();
        $admin_nav->setDisplayname(_('Ergebnisse'));
        $admin_nav->addLinkParam('action', 'solutions');
        $navigation->addSubmenu($admin_nav);

        if ($g_outlet->status != 'student' && $g_outlet->status != 'nobody')
        {
            $admin_nav =& new PluginNavigation();
            $admin_nav->setDisplayname(_('Einstellungen'));
            $admin_nav->addLinkParam('action', 'admin');
            $navigation->addSubmenu($admin_nav);

            $admin_nav =& new PluginNavigation();
            $admin_nav->setDisplayname(_('Import'));
            $admin_nav->addLinkParam('action', 'import');
            $navigation->addSubmenu($admin_nav);
        }

        $admin_nav =& new PluginNavigation();
        $admin_nav->setDisplayname(_('Druckansicht'));
        $admin_nav->addLinkParam('action', 'export');
        $navigation->addSubmenu($admin_nav);

        $this->setNavigation($navigation);
        $this->setShownInOverview(TRUE);
    }

    function initialize ()
    {
        global $vipsPlugin, $vipsTemplateFactory;
        global $_include_additional_header;
        global $HELP_KEYWORD;

        $vipsPlugin = $this;
        $vipsTemplateFactory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates/');

        $css = PluginEngine::getLink($this, array(), 'css');
        $_include_additional_header =
            '<link rel="stylesheet" href="'.$css.'" type="text/css">';

        $HELP_TOPICS = array(
            'sheets'    => 'Vips.bungklausur',
            'exams'     => 'Vips.bungklausur',
            'groups'    => 'Vips.bungsgruppen',
            'solutions' => 'Vips.Lsung',
            'import'    => 'Vips.Aufgaben',
            'export'    => 'Vips.Drucken',
            'admin'     => 'Vips.Vipad');

        $action = vips_get_option_param('action');
        $HELP_KEYWORD = $HELP_TOPICS[$action];

        if (!isset($HELP_KEYWORD))
            $HELP_KEYWORD = 'Vips';
    }

    /**
     * This is required for Stud.IP <= 1.7.
     */
    function getPluginURL ()
    {
        // ABSOLUTE_URI_STUDIP is not available in 1.2-os1
        if (isset($GLOBALS['ABSOLUTE_URI_STUDIP'])) {
            $ABSOLUTE_URI_STUDIP = $GLOBALS['ABSOLUTE_URI_STUDIP'];
        } else {
            $ABSOLUTE_URI_STUDIP = 'http://'.$GLOBALS['EXTERN_SERVER_NAME'];
        }

        return $ABSOLUTE_URI_STUDIP.$this->getPluginPath();
    }

    function getPluginname ()
    {
        return 'Vips';
    }

    function getDisplaytitle ()
    {
        return $this->getPluginname();
    }

    function actionShow ()
    {
        global $g_outlet, $SMILE_SHORT, $SYMBOL_SHORT;

        if ($this->getId() == '' || $g_outlet->status == 'nobody')
        {
            StudIPTemplateEngine::showErrorMessage(_('Sie haben keine Veranstaltung gewhlt oder keine Rechte, diesen Bereich zu betreten.'));
            return;
        }

        vips_unregister_globals();

        $error_level = error_reporting(E_ALL & ~E_NOTICE);
        $display_errors = ini_set('display_errors', true);
        $lc_numeric = setlocale(LC_NUMERIC, 'C');

        $smile_short = $SMILE_SHORT;
        $symbol_short = $SYMBOL_SHORT;
        $SMILE_SHORT = array();
        $SYMBOL_SHORT = array();

        // action: sheets, exams, groups, solution, admin, import or export
        $action = vips_get_option_param('action');
        // subaction: e.g. store_exercise, delete_exam and the like
        $subaction = vips_get_option_param('subaction');

        switch ($action)
        {
            case 'sheets':
                require_once 'vips_assignments.inc.php';
                vips_assignments($action, $subaction);
                break;

            case 'exams':
                require_once 'vips_assignments.inc.php';
                vips_assignments($action, $subaction);
                break;

            case 'groups':
                require_once 'vips_groups.inc.php';
                vips_groups($subaction);
                break;

            case 'solutions':
                require_once 'vips_solutions.inc.php';
                vips_solutions($subaction);
                break;

            case 'import':
                require_once 'vips_import.inc.php';
                vips_import($subaction);
                break;

            case 'export':
                require_once 'vips_export.inc.php';
                vips_export($subaction);
                break;

            case 'admin':
                require_once 'vips_admin.inc.php';
                vips_admin($subaction);
                break;

            default:
                require_once 'vips_assignments.inc.php';
                vips_assignments('sheets', 'list_assignments');
        }

        $SMILE_SHORT = $smile_short;
        $SYMBOL_SHORT = $symbol_short;

        setlocale(LC_NUMERIC, $lc_numeric);
        error_reporting($error_level);
        ini_set('display_errors', $display_errors);
    }

    function actionShowConfigurationPage ()
    {
        global $vipsTemplateFactory;

        $user = $this->getUser();
        $permission = $user->getPermission();

        if (!$permission->hasRootPermission())
        {
            StudIPTemplateEngine::showErrorMessage(_('Sie besitzen keine Berechtigung, um dieses Plugin zu konfigurieren.'));
            return;
        }

        $config = new Config();
        $stud_id = vips_get_string_param('stud_id', true);

        if ($stud_id == '')
        {
            $stud_id = $config->getValue('VIPS_STUDENT_ID_DATAFIELD');
        }
        else
        {
            $config->setValue($stud_id, 'VIPS_STUDENT_ID_DATAFIELD');
        }

        $template = $vipsTemplateFactory->open('config_main');
        $template->set_attribute('stud_id', $stud_id);

        echo $template->render();
    }

    function actionCss ()
    {
        global $g_outlet, $vipsTemplateFactory;

        $template = $vipsTemplateFactory->open('css/vips_style.css.php');
        $template->set_attribute('picturepath', $g_outlet->picturepath);
        $template->set_attribute('use_ajax', vips_use_ajax());

        // this hack is necessary to disable the standard Stud.IP layout
        ob_end_clean();
        header('Content-Type: text/css');
        // Expires header uses RFC1123 date format
        $expires = time() + 60 * 60 * 24;
        header('Expires: '.gmdate('D, d M Y H:i:s O', $expires));
        header('Cache-Control: private');
        header('Pragma: private');

        echo $template->render();
        ob_start('discard_buffer');
    }

    function actionWiki()
    {
        global $SMILE_SHORT, $SYMBOL_SHORT;

        vips_unregister_globals();

        $error_level = error_reporting(E_ALL & ~E_NOTICE);
        $display_errors = ini_set('display_errors', true);
        $lc_numeric = setlocale(LC_NUMERIC, 'C');

        $smile_short = $SMILE_SHORT;
        $symbol_short = $SYMBOL_SHORT;
        $SMILE_SHORT = array();
        $SYMBOL_SHORT = array();

        // language selected in the wiki
        $language = vips_get_string_param('language');

        if (isset($language)) {
            // note: setlocale(LC_MESSAGES, $language) doesn't work here
            // because Stud.IP has already set LC_ALL in the environment
            setLocaleEnv($language);
            bindtextdomain('vips', dirname(__FILE__).'/locale');
        }

        // subaction: e.g. store_exercise, delete_exam and the like
        $subaction = vips_get_option_param('subaction');

        // this hack is necessary to disable the standard Stud.IP layout
        ob_end_clean();
        header('Content-Type: text/html; charset=ISO-8859-1');

        require_once 'vips_wiki.inc.php';
        vips_wiki($subaction);

        ob_start('discard_buffer');

        $SMILE_SHORT = $smile_short;
        $SYMBOL_SHORT = $symbol_short;

        setlocale(LC_NUMERIC, $lc_numeric);
        error_reporting($error_level);
        ini_set('display_errors', $display_errors);
    }

    function getPluginExistingItems ($range_id)
    {
        return 0;
    }

    function getOverviewMessage ($has_changed = false)
    {
        global $perm;

        $seminar = $this->getId();

        if ($this->active_tests + $this->active_exams == 0)
        {
            return $this->getPluginname();
        }

        if ($perm->have_studip_perm('tutor', $seminar))
        {
            return sprintf(ngettext('%d unkorrigierte Lsung', '%d unkorrigierte Lsungen',
                                    $this->active_tests + $this->active_exams),
                                    $this->active_tests + $this->active_exams);
        }

        if ($this->active_tests > 0)
        {
            $active_items[] = $this->active_tests.' '.
                                ngettext('neue bung', 'neue bungen', $this->active_tests);
        }
        if ($this->active_exams > 0)
        {
            $active_items[] = $this->active_exams.' '.
                                ngettext('neue Klausur', 'neue Klausuren', $this->active_exams);
        }

        return join(', ', $active_items);
    }

    function hasChanged ($lastlogin)
    {
        global $perm, $vips_db;

        $seminar = $this->getId();
        $user = $this->getUser();
        $user_id = $user->getUserid();
        $date = date("Y-m-d H:i:s");

        // NOTE: Permission::hasTutorPermissionInPOI() needs to be fixed
        if ($perm->have_studip_perm('tutor', $seminar))
        {
            // find all uncorrected exercises in finished tests in this course
            // Note: SELECT DISTINCT is required here because for group solutions
            // vips_T_Loesung_Punkte contains one row for each member of the group.
            // Added JOIN with seminar_user to filter out lecturer/tutor solutions.
            $query_tests = 'SELECT DISTINCT vips_T_Loesung.Loesungsid
                            FROM vips_uebungsblatt, seminar_user, vips_T_Loesung
                            LEFT JOIN vips_T_Loesung_Punkte
                            ON vips_T_Loesung.Loesungsid = vips_T_Loesung_Punkte.idLoesung
                            WHERE vips_T_Loesung.Assignmentid = vips_uebungsblatt.ID AND
                                vips_uebungsblatt.Kurs = \''.$seminar.'\' AND
                                vips_uebungsblatt.Ende <= \''.$date.'\' AND
                                seminar_user.Seminar_id = \''.$seminar.'\' AND
                                seminar_user.status = \'autor\' AND
                                vips_T_Loesung.Userid = seminar_user.user_id AND
                                vips_T_Loesung.isLastSolution = \'t\' AND
                                vips_T_Loesung.isAlreadyCorrected = 0 AND
                                (vips_T_Loesung_Punkte.Safe IS NULL OR
                                 vips_T_Loesung_Punkte.Safe = \'f\')';

            // find all uncorrected exercises in finished exams in this course
            // Note: SELECT DISTINCT is required here (see above)
            $query_exams = 'SELECT DISTINCT vips_K_Loesung.Loesungsid
                            FROM vips_klausur, seminar_user, vips_K_Loesung
                            LEFT JOIN vips_K_Loesung_Punkte
                            ON vips_K_Loesung.Loesungsid = vips_K_Loesung_Punkte.idLoesung
                            WHERE vips_K_Loesung.Assignmentid = vips_klausur.ID AND
                                vips_klausur.Kurs = \''.$seminar.'\' AND
                                vips_klausur.Ende <= \''.$date.'\' AND
                                seminar_user.Seminar_id = \''.$seminar.'\' AND
                                seminar_user.status = \'autor\' AND
                                vips_K_Loesung.Userid = seminar_user.user_id AND
                                vips_K_Loesung.isLastSolution = \'t\' AND
                                vips_K_Loesung.isAlreadyCorrected = 0 AND
                                (vips_K_Loesung_Punkte.Safe IS NULL OR
                                 vips_K_Loesung_Punkte.Safe = \'f\')';
        }
        else
        {
            // find all active tests not yet seen by the student
            $query_tests = 'SELECT vips_uebungsblatt.ID FROM vips_uebungsblatt
                            LEFT JOIN vips_aufgaben_zeit
                            ON vips_uebungsblatt.ID = vips_aufgaben_zeit.vips_uebungsblatt AND
                                vips_aufgaben_zeit.user_id = \''.$user_id.'\'
                            WHERE vips_uebungsblatt.Kurs = \''.$seminar.'\' AND
                                vips_uebungsblatt.Beginn <= \''.$date.'\' AND
                                vips_uebungsblatt.Ende > \''.$date.'\' AND
                                vips_aufgaben_zeit.user_id IS NULL';

            // find all active exams not yet seen by the student
            $query_exams = 'SELECT vips_klausur.ID FROM vips_klausur
                            LEFT JOIN vips_aufgaben_zeit
                            ON vips_klausur.ID = vips_aufgaben_zeit.vips_klausur AND
                                vips_aufgaben_zeit.user_id = \''.$user_id.'\'
                            WHERE vips_klausur.Kurs = \''.$seminar.'\' AND
                                vips_klausur.Beginn <= \''.$date.'\' AND
                                vips_klausur.Ende > \''.$date.'\' AND
                                vips_aufgaben_zeit.user_id IS NULL';
        }

        $result = $vips_db->executeQuery($query_tests);
        $this->active_tests = $result->getRecordCount();

        $result = $vips_db->executeQuery($query_exams);
        $this->active_exams = $result->getRecordCount();

        return $this->active_tests + $this->active_exams > 0;
    }
}

?>
