<?php
/*
 * mc_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * XXX detailed description
 *
 * @author    XXX
 * @copyright XXX
 * @abstract
 */
class mc_exercise extends Exercise
{
    var $labelYes,
        $labelNo,
        $evaluationMode;

    // constructor
    function mc_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type     = 'mc_exercise';
        $this->labelYes = _('Ja');
        $this->labelNo  = _('Nein');

        if (!isset($xml)) {  // construct exercise using request parameters
            $this->answerArray    = $this->answerArrayFromInput();
            $this->correctArray   = $this->correctArrayFromInput();
            $this->numberOfAnswers = count($this->answerArray);
            $this->evaluationMode = vips_get_int_param('mc_auswertung');
        }
        else {  // construct exercise using XML
            $dom_xml = $this->getDomXML($xml);
            $answers = $dom_xml->get_elements_by_tagname('Answer');

            foreach ($answers as $node)
            {
                $this->answerArray[] = utf8_decode($node->get_content());
                $this->correctArray[] = $node->get_attribute('Correct');
            }

            $this->numberOfAnswers = count($this->answerArray);
            $this->evaluationMode  = reset($this->getTagsFromXML($xml, 'EvalMode'));
        }
    }



    /**
     * Counts the exercise's number of answers.
     *
     * @returns The number of answers in the exercise
     */
    function getNumberOfAnswers()
    {
        return $this->numberOfAnswers;
    }



    /**
     * Returns all variables which are peculiar for this exercise type.
     *
     * @returns An array with the keys 'label_yes', 'label_no',
     *          'evaluation_mode' and 'change_input_field_size'
     */
    function getQuirks()
    {
        for ($i = 0; $i < count($this->answerArray); $i++) {
            if (vips_get_submit_param('increase_input_field_size_'.$i)) {
                $change_input_field_size = array(
                    'alternative' => $i,
                    'size'        => 'large');
            }
            else if (vips_get_submit_param('decrease_input_field_size_'.$i)) {
                $change_input_field_size = array(
                    'alternative' => $i,
                    'size'        => 'small');
            }
        }

        return array(
            'label_yes'               => $this->labelYes,
            'label_no'                => $this->labelNo,
            'evaluation_mode'         => $this->evaluationMode,
            'change_input_field_size' => $change_input_field_size);
    }



    function setDefaultValues()
    {
        $this->numberOfAnswers = 5;
    }



    /**************************************************************************/
    /*
    /* Evaluates the solution
    /*
    /**************************************************************************/

    /**
     *    Evaluates a solution.
     *
     *    @abstract
     *    @access public
     *    @returns int (from 0 to 100 ) as an % value
     */

    function evaluate ($exercise,$solution) {
        $dom_xml = $this->getDomXML($exercise);
        $answers = $dom_xml->get_elements_by_tagname("Answer");
        $mc_auswertung = $this->getEvaluationMode();
        $punkte = 0;

        $answer_array = array();
        $correct_array = array();

        foreach ($answers as $node) {
            $answer_array[]         = html_ready(utf8_decode($node->get_content()));
            $correct_array[]        = (int) $node->get_attribute("Correct");
        }

        $student_array = $this->getTagsFromXML($solution, "answer");

        for ($i = 0; $i < count($correct_array); $i++) {
            if ($student_array[$i] == $correct_array[$i])
                ++$punkte;
            else if ($mc_auswertung == 1 || $mc_auswertung == 2)
                --$punkte;      // abzug bei falscher antwort
        }

        if ($punkte < 0 && $mc_auswertung != 2)
            $punkte = 0;

        $m_prozent = $punkte / count($correct_array) * 100;
        $safe = 1;

        $m_return[1] = $m_prozent;
        $m_return[2] = $safe;
        $m_return[5] = $punkte;
        $m_return['percent'] = $m_prozent;
        $m_return['safe']    = $safe;
        return $m_return;
    }



    /**
     * Generates an XML-String containing the solution of the student.
     *
     * Gets its values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @param int $exercise_id the id of the exercise the solution related to.
     * @param int $p_number_of_answers (deprecated) the number of answers possible (This is a class variable!)
     * @return XML the XML-String containing the solutions of the student.
     *
     * @todo names of the exml entities are not well choosen. A change will have
     * impact on other parts of the code, of course.
     */

    function genSolution($exercise_id, $p_number_of_answers = null)
    {
        $noa = isset($p_number_of_answers) ? $p_number_of_answers : $this->numberOfAnswers;
        $xml = "<solution id=\"".$exercise_id."\">";

        for ($i = 0; $i < $noa; $i++) {
            // the chosen answers are in the get or post environment: 1. Otherwise:0.
            if (vips_get_int_param("answer_$i")) {
                $xml .= "<answer id=\"".$i."\">";
                $xml .= "<body>1</body>";
                $xml .= "</answer>";
            }
            else {
                $xml .= "<answer id=\"".$i."\">";
                $xml .= "<body>0</body>";
                $xml .= "</answer>";
            } // end if
        } // end for

        $xml .= "</solution>";
        return vips_filter_xml_string($xml);
    }



    /**
     * Generate the description af an exercise in an XML-String.
     *
     * Gets ist values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @return XML the XML-String describing the exercise in terms of its
     * name, questions, wrong and correct solutions etc.
     */

    function getExerciseConstruct()
    {
        $xmlop  = "<TestItem ID=\"" . $this->type . "\" Type=\"MultipleChoice\" Language=\"" . $this->language . "\">";
        $xmlop .= "<Name>".htmlspecialchars($this->name)."</Name>";
        $xmlop .= "<Hint>".htmlspecialchars($this->hint)."</Hint>";
        $xmlop .= "<CommentField Visible=\"".$this->commentAllowed."\"></CommentField>";
        $xmlop .= "<Question>";
        $xmlop .= "<Paragraph>".htmlspecialchars($this->question)."</Paragraph>";
        $xmlop .= "</Question>";
        $xmlop .= "<EvalMode>".htmlspecialchars($this->evaluationMode)."</EvalMode>";

        for ($i = 0; $i < count($this->answerArray); $i++) {
            $xmlop .= '<Answer Correct="'.$this->correctArray[$i].'">'.htmlspecialchars($this->answerArray[$i]).'</Answer>';
        }

        $xmlop .= "</TestItem>";

        return vips_filter_xml_string($xmlop);
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ()
    {
        $result = parent::exportText('MC-Frage');

        for ($i = 0; $i < count($this->answerArray); ++$i)
        {
            if ($this->correctArray[$i] == '1')
                $result .= '+';
            else if ($this->correctArray[$i] == '0.5')
                $result .= '~';

            $result .= 'Antwort: '.$this->answerArray[$i]."\n";
        }

        return $result;
    }



    /**
     * Creates a template for editing a mc_exercise.
     *
     * @returns The template for editing this exercise
     */
    function getEditTemplate()
    {
        global $vipsTemplateFactory, $g_outlet, $vipsPlugin;

        if (vips_get_submit_param('add_answer')) {  // add a new answer field
            $number_of_answers = vips_get_int_param('number_of_answers');
            $number_of_answers += 1;
        } else {
            $number_of_answers = $this->numberOfAnswers;
        }

        $template = $vipsTemplateFactory->open('exercises/edit_mc_exercise');
        $template->set_attributes(array(
            'exercise_id'             => vips_get_int_param('exercise_id'),
            'answer_array'            => $this->getAnswerArray(),
            'correct_array'           => $this->getCorrectArray(),
            'number_of_answers'       => $number_of_answers,
            'label_yes'               => $this->labelYes,
            'label_no'                => $this->labelNo,
            'evaluation_mode'         => $this->evaluationMode,
            'change_input_field_size' => $this->getQuirk('change_input_field_size'),
            'picturepath'             => $g_outlet->picturepath,
            'vips_imagepath'          => $vipsPlugin->getPluginURL().'/images/'));

        return $template;
    }



    /**
     * Creates a template for solving an mc_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/solve_mc_exercise');
        $template->set_attributes(array(
            'exercise_question' => $this->question,
            'exercise_hint'     => $this->hint,
            'answer_array'      => $this->answerArray,
            'student_array'     => $student_array,
            'evaluation_mode'   => $this->evaluationMode));

        return $template;
    }



    /**
     * Fills a WIKI template with all necessary values. If, additionally, a
     * solution is processed to it, it will also be set in.
     *
     * @param $solution (optional) the solution XML
     * @return the filled WIKI template object
     */
    function getWikiTemplate($solutionXML = null)
    {
        global $vipsTemplateFactory;

        // handle solution
        if (isset($solutionXML)) {
            $studentArray = $this->getTagsFromXML($solutionXML, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/wiki_mc_exercise');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'hint'          => $this->hint,
            'answer_array'  => $this->answerArray,
            'student_array' => $studentArray
            ));

        return $template;
    }



    /**
     * Builds a template showing a corrected solution.
     *
     * @param $solution_xml The XML string of the student solution
     * @returns The template object
     */
    function getCorrectedWikiTemplate($solution_xml)
    {
        global $vipsTemplateFactory;

        $student_array = $this->getTagsFromXML($solution_xml, 'answer');

        $template = $vipsTemplateFactory->open('exercises/wiki_mc_exercise_corrected');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'hint'          => $this->hint,
            'answer_array'  => $this->answerArray,
            'correct_array' => $this->correctArray,
            'student_array' => $student_array
            ));

        return $template;
    }



    /**
     * Constructs the <tt>answerArray</tt> from the REQUEST environment.
     *
     * @returns the answer array
     */
    function answerArrayFromInput()
    {
        $count = vips_get_int_param('number_of_answers');
        $answer_array = array();

        for ($i = 0; $i < $count; ++$i) {
            $answer = vips_get_string_param('answer_'.$i, true);

            if ($answer != '') {
                $answer_array[] = $answer;
            }
        }

        return $answer_array;
    }



    /**
     * Constructs the <tt>correctArray</tt> from the REQUEST environment.
     *
     * @returns the correct array
     */
    function correctArrayFromInput()
    {
        $count = vips_get_int_param('number_of_answers');
        $correct_array = array();

        for ($i = 0; $i < $count; ++$i) {
            $answer = vips_get_string_param('answer_'.$i, true);

            if ($answer != '') {
                $correct_array[] = vips_get_int_param('correct_'.$i);
            }
        }

        return $correct_array;
    }

}  // end class

?>
