<?php
/*
 * pl_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * XXX detailed description
 *
 * @author    Philipp Hgelmeyer
 * @copyright XXX
 * @abstract
 */
class pl_exercise extends Exercise
{
    var $answerDefault,
        $queryDefault;

    function pl_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type = 'pl_exercise';

        if (!isset($xml)) {  // construct exercise using request parameters
            $this->answerArray   = array(vips_get_string_param('answer_0', true));
            $this->answerDefault = vips_get_string_param('answer_default', true);
            $this->queryDefault  = vips_get_string_param('query_default', true);
        }
        else {  // construct exercise using XML
            $this->answerArray     = $this->getTagsFromXML($xml, 'Answer');
            $this->numberOfAnswers = count($this->answerArray);
            $this->answerDefault   = reset($this->getTagsFromXML($xml, 'AnswerDefault'));
            $this->queryDefault    = reset($this->getTagsFromXML($xml, 'QueryDefault'));
        }
    }



    /**
     * Returns all variables which are peculiar for this exercise type.
     *
     * @returns An array with the keys 'answer_default' and 'query_default'
     *          (both needed in edit and show template) and 'pl_temp_query',
     *          'pl_score', 'pl_validity', 'pl_count' and 'pl_out' (the latter
     *          ones exclusively needed in show template)
     */
    function getQuirks()
    {
        $pl_temp_query = vips_get_unquoted_string_param('pl_query', true);
        $pl_score      = vips_get_float_param('pl_score');
        $pl_validity   = vips_get_float_param('pl_validity');
        $pl_count      = max(1, vips_get_int_param('pl_count'));
        $pl_out        = vips_get_string_param('pl_out');  // internal communication

        return array(
            'answer_default' => $this->answerDefault,
            'query_default'  => $this->queryDefault,
            'pl_temp_query'  => $pl_temp_query,
            'pl_score'       => $pl_score,
            'pl_validity'    => $pl_validity,
            'pl_count'       => $pl_count,
            'pl_out'         => $pl_out);
    }



    /**************************************************************************/
    /*
    /* Evaluates the solution
    /*
    /**************************************************************************/

    /**
     *    Evaluates a solution.
     *
     *    @abstract
     *    @access public
     *    @returns int (from 0 to 100 ) as an % value
     */

    function evaluate($exercise, $solution, $p_user_id, $p_gesamtpunktzahl, $solutionFromTextarea = NULL) {  //exception for pl exercises

        $username = get_solver_name($p_user_id);

        $punkte = 0;

        $dom_xml = $this->getDomXML($exercise);
        $answers = $dom_xml->get_elements_by_tagname("Answer");
        $node =  $answers[0];
        $musterloesung  = utf8_decode($node->get_content());

        $answer_default         = $this->getTagsFromXML($exercise, "AnswerDefault");
        $answerDefault          = $answer_default[0];

        $testcase = '';
        if (isset($solutionFromTextarea)) {
            $testcase = $solutionFromTextarea;
        }
        else {
            $student_array = $this->getTagsFromXML($solution, "answer");
            $testcase =  $student_array[0];
        }

        $course = $g_outlet->courseID;

        $shell_output = "";
        $fp = fsockopen("mvc.ikw.uni-osnabrueck.de", 2213); //, $errno, $errstr, 30);
        if (!$fp) {
            echo "ERROR fsockopen: PROLOG compiler couldn't be reached'."; //"$errstr ($errno)<br />\n";
        }
        else
        {
            $queryXML = "<prolog>\n<init>\n$answerDefault\n</init>\n<user>\n$username\n</user>\n<course>\n$course\n</course>\n<magic>\n!ViPS!\n</magic>\n<number>\n1\n</number>\n<type>\neval\n</type>\n<exercise>\n".$testcase."\n</exercise>\n<testcase>\n".$musterloesung."\n</testcase>\n</prolog>\n";
            //echo "<br>Query<pre>: $queryXML </pre>";
            fputs($fp, $queryXML);
            while (!feof($fp))      {       $shell_output .= fgets($fp);    }
            fclose($fp);
        }

        list($score1,$rest) = split("<Score>",$shell_output);
        list($score, $rest) = split("</Score>",$rest);

        list($output1,$rest) = split("<Output>",$shell_output);
        list($output,$rest) = split("</Output>",$rest);

        list($validity1,$rest) = split("<Validity>",$shell_output);
        list($validity,$rest) = split("</Validity>",$rest);

        if ($validity == 1) {
            $safe = 1;
        }

        $m_return = array();

        $prozent = $score * 100;
        $punkte = $score * $p_gesamtpunktzahl;  // 2,3824 von 5
        $punkte = floor($punkte * 10) / 10;     // 23,824 -> 2,3

        $m_return[1] = $prozent;  //prozent
        $m_return[2] = $safe;
        $m_return[5] = $punkte;  //punktzahl
        $m_return[6] = $output;
        $m_return['percent'] = $prozent;
        $m_return['safe']    = $safe;

        return $m_return;
    }



    function download($file , $name) {
        $size = filesize($file);
        header("Content-type: application/octet-stream");
        header("Content-disposition: attachment; filename=".$name);
        header("Content-Length: ".$size);
        header("Pragma: no-cache");
        header("Expires: 0");
        readfile($file);
    }



    /**
     * Prepares the currently displayed solution for downloading.
     *
     * @param $exercise_name The name of the current exercise
     * @returns Nothing. The code can directly be downloaded or viewed.
     */
    function downloadSolution($exercise_name)
    {
        $solution_view = vips_get_option_param('solution_view');
        $answer_param = $solution_view == 'solution_edit'
            ? 'commented_solution'
            : 'answer_0';
        $solution = vips_get_unquoted_string_param($answer_param, true);

        // this hack is necessary to disable the standard Stud.IP layout
        ob_end_clean();

        header('Content-Type: text/plain; charset=ISO-8859-1');
        header('Content-Disposition: attachment; filename="'.$exercise_name.'.txt"');
        header('Pragma: public');

        echo $solution;

        ob_start('discard_buffer');
    }



    /**
     * Uploads a file as solution.
     *
     * @returns The file content
     */
    function uploadSolution()
    {
        $solution = null;

        if (!empty($_FILES['pl_userfile'])) {
            if (is_uploaded_file($_FILES['pl_userfile']['tmp_name'])) {
                $solution = file_get_contents($_FILES['pl_userfile']['tmp_name']);
            }
        }
        else {
            echo _('Fehler beim Hochladen!');
        }

        return $solution;
    }



    /****************************************************************************************/
    /* Queries the query inserted by the student with the corresponding code of the exercise.
    /* @param int exercise_id the id of the exercise that sould be queried.
    /* @param string exerciseXML the exercise code containing the info about the exercise
    /****************************************************************************************/

    function evaluateQuery($p_user_id, $p_course, $p_number = 1)
    {
        $username = get_solver_name($p_user_id);

        $solution_view = vips_get_option_param('solution_view');
        $answer_param = $solution_view == 'solution_edit'
            ? 'commented_solution'
            : 'answer_0';
        $answer = vips_get_unquoted_string_param($answer_param, true);
        $query  = vips_get_unquoted_string_param('pl_query', true);
        $shell_output = "";
        $output_array = array();

        $fp = fsockopen("mvc.ikw.uni-osnabrueck.de", 2213); //, $errno, $errstr, 30);
        if (!$fp) {
            echo "ERROR fsockopen: PROLOG compiler couldn't be reached'."; //"$errstr ($errno)<br />\n";
        } else {
            $queryXML = "<prolog>\n<user>\n$username\n</user>\n<course>\n$p_course\n</course>\n<magic>\n!ViPS!\n</magic>\n<number>\n".$p_number."\n</number>\n<type>\nquery\n</type>\n<exercise>\n".$answer."\n</exercise>\n<testcase>\n".$query."\n</testcase>\n</prolog>\n";
            fputs($fp, $queryXML);
            while (!feof($fp)) {
                $output_array[] = fgets($fp);
            }
            //echo "<br>shell_output: $shell_output ";
            fclose($fp);
        }


        foreach ($output_array as $line) {
            if ($line != "\n") {
                $shell_output .= $line;
            }
        }

        return $shell_output;
    }



    /**************************************************************************/
    /*
    /* XML-generating routines
    /*
    /**************************************************************************/



    /**
     * Generates generates an XML-String containing the solution of the
     * student.
     *
     * Gets ist values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @param int $exercise_id the id of the exercise the solution related to.
     * @param int $p_number_of_answers the number of answers possible (??? not a class variable, ES)
     * @return XML the XML-String containing the solutions of the student.
     *
     * @todo names of the exml entities are not well choosen. A change will have
     * impact on other parts of the code, of course.
     */

    function genSolution($p_exercise_id, $p_number_of_answers = null) {

        $i    = 0;
        $msg  = "<solution id=\"".$p_exercise_id."\">";
        $msg .= "<answer id=\"0\">";
        $msg .= "<body>".htmlspecialchars(vips_get_string_param("answer_0", true))."</body>";
        $msg .= "</answer>";

        $msg .= "</solution>";
        return vips_filter_xml_string($msg);
    }



    /**
     * Generate the description af an exercise in an XML-String.
     *
     * Gets ist values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @return XML the SML-String describing the exercise in terms of its
     * name, questions, worng and correct solutions etc.
     */

    function getExerciseConstruct()
    {
        $xmlop  = "<TestItem ID=\"" . $this->type . "\" Type=\"Prolog\" Language=\"" . $this->language . "\">";
        $xmlop .= "<Name>".htmlspecialchars($this->name)."</Name>";
        // In this special case the hint is displayed inside a textbox to provide comments
        // and code fragments to the student.
        $xmlop .= "<Hint>".htmlspecialchars($this->hint)."</Hint>";
        $xmlop .= "<CommentField Visible=\"".$this->commentAllowed."\"></CommentField>";
        $xmlop .= "<Question>";
        $xmlop .= "<Paragraph>".htmlspecialchars($this->question)."</Paragraph>";
        $xmlop .= "</Question>";
        $xmlop .= "<Answer>".htmlspecialchars($this->answerArray[0])."</Answer>";
        $xmlop .= "<AnswerDefault>".htmlspecialchars($this->answerDefault)."</AnswerDefault>";
        $xmlop .= "<QueryDefault>".htmlspecialchars($this->queryDefault)."</QueryDefault>";

        $xmlop .= "</TestItem>";

        return vips_filter_xml_string($xmlop);
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ()
    {
        $result = parent::exportText('PL-Frage');

        $answer_text = $this->answerArray[0];

        if ($this->answerDefault != '')
        {
            $result .= "Vorgabe:\n";
            $result .= $this->answerDefault."\n";
            $result .= "\\Vorgabe\n";
        }

        if ($answer_text != '')
        {
            $result .= "Antwort:\n";
            $result .= $answer_text."\n";
            $result .= "\\Antwort\n";
        }

        return $result;
    }



    /**
     * Creates a template for editing a pl_exercise.
     *
     * @returns The template for editing this exercise
     */
    function getEditTemplate()
    {
        global $vipsTemplateFactory;

        $template = $vipsTemplateFactory->open('exercises/edit_pl_exercise');
        $template->set_attributes(array(
            'answer_array'   => $this->getAnswerArray(),
            'answer_default' => $this->answerDefault,
            'query_default'  => $this->queryDefault));

        return $template;
    }



    /**
     * Creates a template for solving a pl_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global //$g_outlet,
               $vipsTemplateFactory;

//        $solver_user_id    = vips_get_string_param('solver_user_id');
//        $user_id           = isset($solver_user_id) ? $solver_user_id : $g_outlet->userID;
//        $exercise_position = vips_get_int_param('exercise_position');
//        $assignment_id     = vips_get_int_param('assignment_id');
//        $assignment_type   = vips_get_string_param('action') == 'sheets' ? 'vips_uebungsblatt' : 'vips_klausur';
//        $solution_xml      = getUserSolution($user_id, $exercise_position, $assignment_id, $assignment_type);

        $student_answer = vips_get_unquoted_string_param('answer_0', true);

        if (vips_get_submit_param('pl_exercise_mode_upload')) {
            // if there is an uploaded solution, show that one
            $student_array = array($this->uploadSolution());
        } else if (isset($student_answer)) {
            // if there is an edited solution, show that one
            $student_array = array($student_answer);
        } else if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/solve_pl_exercise');
        $template->set_attributes(array(
            'exercise_question' => $this->question,
            'exercise_hint'     => $this->hint,
            'student_array'     => $student_array,
            'answer_default'    => $this->answerDefault,
            'pl_temp_query'     => vips_get_unquoted_string_param('pl_query', true),
            'query_default'     => $this->queryDefault,
            'pl_count'          => max(1, vips_get_int_param('pl_count')),
            'pl_out'            => vips_get_string_param('pl_out')));

        return $template;
    }

}
?>
