<?php
/*
 * vips_admin.inc.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2007  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/**
 * Created on 08.03.2006 by Christa Deiwiks <cdeiwiks@uos.de>
 * 13:49:06
 *
 * Manages Vips-related info concerning administration:
 *
 * - access parameters
 * - sheet weightings
 * - grade distribution
 *
 * @author: Christa Deiwiks <cdeiwiks@uos.de>
 */

function vips_admin ($subaction)
{
    global $g_outlet;

    if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
        if ($subaction == 'store_distribution') {
            store_distribution();
        } else if ($subaction == 'store_parameters') {
            store_parameters();
        } else if ($subaction == 'delete_group') {
            // Klausur-Gruppierung aufloesen
            delete_group();
        } else {
            main();
        }
    } else {
        StudIPTemplateEngine::showErrorMessage(_('Sie haben keine Rechte, diesen Bereich zu betreten.'));
    }
}

/**
 * Undoes a grouping of two exams
 */

function delete_group() {
        global $vips_db, $g_outlet;

        $group_id = vips_get_int_param('group_id');
        $vips_db->executeUpdate("DELETE FROM vips_klausur_gruppierung
                                                  WHERE id = '$group_id'");
        $vips_db->executeUpdate("DELETE FROM vips_gewichtung
                                                WHERE Item_id = '$group_id'
                                                AND Item_type = 'groups'");
        main();

}



/**
 *
 * Stores the weights and exam grouping of blocks,sheets and exams
 *
 */

function store_parameters() {
        global $vips_db, $g_outlet;

        $number_of_blocks = vips_get_int_param("number_of_blocks");
        $number_of_exams  = vips_get_int_param("number_of_exams");
        $number_of_sheets = vips_get_int_param("number_of_sheets");
        $number_of_groups = vips_get_int_param("number_of_groups");

        $gruppierung = vips_get_int_array_param('gruppierung');
        $anzahl_gruppierte_klausuren = count($gruppierung);
        $weight_gruppierte_klausuren = array();

        $prozente       = 0;
        $message = array();

        $bestehen = 0;

        for($i=0; $i<$number_of_blocks; $i++) {
                $prozente += vips_get_float_param("block_weight_$i");
        }
        for($i=0; $i<$number_of_exams; $i++) {
                $prozente += vips_get_float_param("exams_weight_$i");
                if (($bestehen_temp = vips_get_int_param("exams_bestehen_$i")) > $bestehen)
                        $bestehen = $bestehen_temp;
        }
        for($i=0; $i<$number_of_sheets; $i++) {
                $prozente += vips_get_float_param("sheets_weight_$i");
        }
        for($i=0; $i<$number_of_groups; $i++) {
                $prozente += vips_get_float_param("groups_weight_$i");
        }

        foreach($gruppierung as $counter=>$klausur_id) {
                $weight_gruppierte_klausuren[] = vips_get_float_param("exams_weight_$counter");
        }

        // NOTE floating point additions may be inaccurate, so round before comparison
        if (round($prozente, 1) != 100 && $prozente) {
                $message['mode']        = 'error';
                $message['message'] = "Die Gewichtungen mssen in der Summe 100 oder 0 (keine Gewichtung) ergeben. Die Daten wurden nicht bernommen!";
        }
        elseif ($bestehen > 100) {
                $message['mode']        = 'error';
                $message['message'] = "Die Prozentwerte zum Bestehen mssen kleiner oder gleich 100 sein. Die Daten wurden nicht bernommen!";
        }
        elseif ($anzahl_gruppierte_klausuren && $anzahl_gruppierte_klausuren !=2) {
                $message['mode']        = 'error';
                $message['message'] = "Zum Gruppieren bitte genau 2 Klausuren markieren!";
        }
        else {
                for($i=0; $i<$number_of_blocks; $i++) {
                        $id             = vips_get_int_param("block_id_$i");
                        $weight = vips_get_float_param("block_weight_$i");
                        $sql = "REPLACE vips_gewichtung
                                        SET Gewichtung = '$weight',
                                        Item_id = '$id',
                                        Item_type = 'blocks'";
                        $vips_db->executeUpdate($sql);
                }
                for($i=0; $i<$number_of_exams; $i++) {
                        $id             = vips_get_int_param("exams_id_$i");
                        $weight = vips_get_float_param("exams_weight_$i");
                        $sql = "REPLACE vips_gewichtung
                                        SET Gewichtung = '$weight',
                                        Item_id = '$id',
                                        Item_type = 'exams'";
                        $vips_db->executeUpdate($sql);

                        $bestehen = vips_get_int_param("exams_bestehen_$i");
                        $sql = "UPDATE vips_klausur
                                        SET Bestehen = '$bestehen'
                                        WHERE ID = '$id'";
                        $vips_db->executeUpdate($sql);

                }
                for($i=0; $i<$number_of_sheets; $i++) {
                        $id             = vips_get_int_param("sheets_id_$i");
                        $weight = vips_get_float_param("sheets_weight_$i");
                        $sql = "REPLACE vips_gewichtung
                                        SET Gewichtung = '$weight',
                                        Item_id = '$id',
                                        Item_type = 'sheets'";
                        $vips_db->executeUpdate($sql);
                }

                for($i=0; $i<$number_of_groups; $i++) {
                        $id             = vips_get_int_param("groups_id_$i");
                        $weight = vips_get_float_param("groups_weight_$i");
                        $sql = "REPLACE vips_gewichtung
                                        SET Gewichtung = '$weight',
                                        Item_id = '$id',
                                        Item_type = 'groups'";
                        $vips_db->executeUpdate($sql);

                        $fail_k1                = vips_get_option_param("fail_k1_$i");
                        $pass_k1                = vips_get_option_param("pass_k1_$i");

                        $vips_db->executeUpdate("UPDATE vips_klausur_gruppierung
                                        SET fail_k1 = '$fail_k1',
                                                pass_k1 = '$pass_k1'
                                        WHERE id = '$id'
                                        ");

                }

                if ($anzahl_gruppierte_klausuren == 2) {   //if two exams are to be grouped
                        $kurs_id = $g_outlet->courseID;
                        $k1 = $gruppierung[0];
                        $k2 = $gruppierung[1];
                        $weight_k1 = $weight_gruppierte_klausuren[0];
                        $sql = "INSERT INTO vips_klausur_gruppierung
                                        SET k1 = '$k1',
                                            k2 = '$k2',
                                            kurs = '$kurs_id'";
                        $vips_db->executeUpdate($sql);
                        $vips_idgen = $vips_db->getIdGenerator();
                        $weight_id = $vips_idgen->getId();
                        $sql = "INSERT INTO vips_gewichtung
                                        SET Gewichtung = '$weight_k1',
                                            Item_id = $weight_id,
                                            Item_type = 'groups'";
                        $vips_db->executeUpdate($sql);

                        $message['mode']        = 'msg';
                        $message['message'] = "Die Klausuren wurden gruppiert! <font style=color:red>Bitte berarbeiten Sie jetzt ggf. die Gewichtungen!</font>";
                }
        }

        $_POST['message']       = $message;
        main();
}

/**
 *
 * Stores the distribution of grades
 *
 */
function store_distribution() {
    global $vips_db, $g_outlet;

    $message     = array();
    $error       = false;

    $grades      = array('0,7', '1,0', '1,3', '1,7', '2,0', '2,3', '2,7', '3,0',
                         '3,3', '3,7', '4,0');
    $percentages = array();
    $comments    = array();

    for ($i = 0; $i < count($grades); $i++)  {
        $percentages[] = vips_get_int_param("percentage_".$i);
        $comments[] = vips_get_string_param("comment_".$i, true);
    }

    ######################
    #  check percentage  #
    ######################

    $percentage_last = UNUSED_GRADE;

    for ($i = 0; $i < count($grades); $i++)
    {
        $pi = $percentages[$i];

        if ($pi == '-' || $pi == '--' || $pi == '---')  // means: don't use this grade
        {
            $percentages[$i] = UNUSED_GRADE;
        }
        else  // check...
        {
            if ($pi < 0 || $pi > 100) {
                $message['mode']    = 'error';
                $message['message'] = "Die Notenwerte mssen zwischen 0 und 100 liegen!";
                $error = true;
            }
            elseif ($percentage_last <= $pi) {
                $message['mode']    = 'error';
                $message['message'] = "Die Notenwerte mssen monoton absteigen ( $percentages[$i] >= $percentage_last )!";
                $error = true;
            }

            $percentage_last = $pi;
        }
    }

    ###########################
    #  insert/update database #
    ###########################

    if (!$error) {
        for ($i = 0; $i < count($grades); $i++)  {
            $percentage = $percentages[$i];
            $comment = $comments[$i];
            $grade  = $grades[$i];

            $sql = "SELECT *
                FROM vips_noten
                WHERE Note = '$grade'
                AND Kurs = '$g_outlet->courseID'";
            $result = $vips_db->executeQuery($sql);

            if ($result->getRecordCount()) {
                $vips_db->executeUpdate("UPDATE vips_noten
                    SET
                        Prozent = '$percentage',
                        comment = '$comment'
                    WHERE
                        Note    = '$grade'
                    AND
                        Kurs    = '$g_outlet->courseID'
                    ");
            }
            else
            {
                $vips_db->executeUpdate("INSERT INTO vips_noten
                    SET
                        Prozent = '$percentage',
                        Note    = '$grade',
                        Kurs    = '$g_outlet->courseID',
                        comment = '$comment'
                    ");
            }
        }
        $message['mode']    = 'msg';
        $message['message'] = "Die Notenwerte wurden eingetragen!";
    }

    $_POST['message'] = $message;
    main();
}

/**
 *
 * Displays the main page with
 * -access info
 * -evaluation parameters
 * -grade distribution
 * to edit
 *
 */
function main() {
        global $vips_db, $g_outlet, $vipsTemplateFactory;

        $t_main = $vipsTemplateFactory->open('admin_main');

        $t_main->set_attributes(array(
                'picturepath' => $g_outlet->picturepath,
                'status'          => $g_outlet->status));

        ###########################################
        ############# PARAMETERS ##################
        ###########################################

        $message           = vips_get_string_array_param('message');
        $dialog            = vips_get_string_param('dialog');
        $delete_block_name = vips_get_string_param('delete_block_name');

        #####################################
        # get used blocks and their weights #
        #####################################

        $blocks = array();

        $result = $vips_db->executeQuery("SELECT DISTINCT * FROM vips_block
                                                  WHERE Kurs = '$g_outlet->courseID'");
        while ($result->next()) {
                $entry = $result->getRow();
                $block_id = $entry['id'];
                $result2 = $vips_db->executeQuery(
                                "SELECT *
                                FROM vips_gewichtung
                                WHERE Item_id = '$block_id'
                                AND Item_type = 'blocks'");
                if ($result2->next()) {
                        $entry['weight'] = $result2->getFloat('Gewichtung');
                }
                $result2 = $vips_db->executeQuery("SELECT * FROM vips_inBlock WHERE block_id = '$block_id'");
                if ($result2->getRecordCount())
                        $blocks[] = $entry;
        }

        #############################################################
        # ... and remaining assignments and groups and their data   #
        #############################################################

        $sheets = array();
        $query = "SELECT * FROM vips_uebungsblatt
                        WHERE Kurs='$g_outlet->courseID'
                        ORDER BY Beginn, Position";
        $result = $vips_db->executeQuery($query);
        while ($result->next()) {
                $entry = $result->getRow();
                $assignment_id = $entry['ID'];
                $result2 = $vips_db->executeQuery(
                                "SELECT *
                                FROM vips_gewichtung
                                WHERE Item_id = '$assignment_id'
                                AND Item_type = 'sheets'");
                if ($result2->next()) {
                        $entry['weight'] = $result2->getFloat('Gewichtung');
                }
                $result2 = $vips_db->executeQuery("SELECT * FROM vips_inBlock WHERE assignment_id = '$assignment_id'");
                if ($result2->getRecordCount()==0)
                        $sheets[] = $entry;
        }


        $groups = array();
        $query = "SELECT * FROM vips_klausur_gruppierung
                        WHERE kurs='$g_outlet->courseID'
                        ";
        $result = $vips_db->executeQuery($query);
        while ($result->next()) {
                $entry = $result->getRow();
                $group_id = $entry['id'];
                $k1 = $entry['k1'];
                $k2 = $entry['k2'];
                $result2 = $vips_db->executeQuery("SELECT * FROM vips_klausur WHERE ID = '$k1'");
                $result2->next();
                $entry2 = $result2->getRow();
                $entry['k1_name'] = $entry2['Name'];
                $result2 = $vips_db->executeQuery("SELECT Name FROM vips_klausur WHERE ID = '$k2'");
                $result2->next();
                $entry2 = $result2->getRow();
                $entry['k2_name'] = $entry2['Name'];
                $result2 = $vips_db->executeQuery(
                                "SELECT *
                                FROM vips_gewichtung
                                WHERE Item_id = '$group_id'
                                AND Item_type = 'groups'");
                if ($result2->next()) {
                        $entry['weight'] = $result2->getFloat('Gewichtung');
                }
                $groups[] = $entry;
        }

        $exams = array();
        $query = "SELECT * FROM vips_klausur
                        WHERE Kurs='$g_outlet->courseID'
                        ORDER BY Beginn, Position";
        $result = $vips_db->executeQuery($query);
        while ($result->next()) {
                $entry = $result->getRow();
                $assignment_id = $entry['ID'];
                $result2 = $vips_db->executeQuery(
                                "SELECT *
                                FROM vips_gewichtung
                                WHERE Item_id = '$assignment_id'
                                AND Item_type = 'exams'");
                if ($result2->next()) {
                        $entry['weight'] = $result2->getFloat('Gewichtung');
                }
                $result2 = $vips_db->executeQuery("SELECT * FROM vips_klausur_gruppierung WHERE k1 = '$assignment_id' OR k2 = '$assignment_id'");
                if ($result2->getRecordCount()==0)
                        $exams[] = $entry;
        }

        $t_main->set_attribute('blocks', $blocks);
        $t_main->set_attribute('exams',  $exams);
        $t_main->set_attribute('sheets', $sheets);
        $t_main->set_attribute('groups', $groups);



    ###########################################
    ############# DISTRIBUTION ################
    ###########################################

    $grades = array('0,7', '1,0', '1,3', '1,7', '2,0', '2,3', '2,7', '3,0',
                    '3,3', '3,7', '4,0');
    $percentages = array_fill(0, count($grades), '');
    $comments = array_fill(0, count($grades), '');

    $result = $vips_db->executeQuery(
        "SELECT Note, Prozent, comment
         FROM vips_noten
         WHERE Kurs = '$g_outlet->courseID'
         ORDER BY Note");
    while ($result->next()) {
        $percentage = $result->getRow();
        $index = array_search($percentage['Note'], $grades);
        $percentages[$index] =
            $percentage['Prozent'] == UNUSED_GRADE ?
                '--' : $percentage['Prozent'];
        $comments[$index] = $percentage['comment'];
    }

    $t_main->set_attributes(array(
        'message'           => $message,
        'dialog'            => $dialog,
        'delete_block_name' => $delete_block_name,
        'grades'            => $grades,
        'percentages'       => $percentages,
        'comments'          => $comments));

    setlocale(LC_NUMERIC, NULL);
    echo $t_main->render();
    setlocale(LC_NUMERIC, 'C');
}
?>
