<?php
/*
 * vips_assignments.inc.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/**
 *
 * Core file of Vips (I guess...). Handles sheets and exams (that depends on $p_action),
 * everything from creating exercises to assigning assignments to blocks
 *
 * @author: Christa Deiwiks <cdeiwiks@uos.de>
 */

function vips_assignments ($action, $subaction)
{
    global $g_outlet,
           $g_assignment_table,
           $g_inAssignment_table,
           $g_solution_table,
           $g_solution_points_table,
           $g_title_s,
           $g_title_p;

    if ($action == 'sheets') {
        $g_assignment_table      = 'vips_uebungsblatt'; //assign necessary variables for exercise sheets
        $g_title_s               = _('bungsblatt');
        $g_title_p               = _('bungsbltter');
        $g_inAssignment_table    = 'vips_inUebungsblatt';
        $g_solution_table        = 'vips_T_Loesung';
        $g_solution_points_table = 'vips_T_Loesung_Punkte';
    }
    else {  // $action == 'exams'
        $g_assignment_table      = 'vips_klausur'; //assign necessary variables for exams
        $g_title_s               = _('Klausur');
        $g_title_p               = _('Klausuren');
        $g_inAssignment_table    = 'vips_inKlausur';
        $g_solution_table        = 'vips_K_Loesung';
        $g_solution_points_table = 'vips_K_Loesung_Punkte';
    }

    /****************************/
    /* LECTURER/TUTOR FUNCTIONS */
    /****************************/
    if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
        if ($subaction == 'list_assignments') {
            list_assignments($action);
        }
        else if ($subaction == 'list_assignments_stud') {
            list_assignments_stud($action);
        }
        else if ($subaction == 'new_assignment') {
            new_assignment($action);
        }
        else if ($subaction == 'copy_assignment') {
            copy_assignment($action);
        }
        else if ($subaction == 'store_assignment') {
            store_assignment($action);
        }
        else if ($subaction == 'store_exercise') {
            store_exercise($action);
        }
        else if ($subaction == 'edit_assignment') {
            edit_assignment($action);
        }
        else if ($subaction == 'edit_exercise') {
            edit_exercise($action);
        }
        else if ($subaction == 'delete_assignment') {
            delete_assignment($action);
        }
        else if ($subaction == 'delete_exercise') {
            delete_exercise($action);
        }
        else if ($subaction == 'delete_exercise_ajax') {
            delete_exercise_ajax($action);
        }
        else if ($subaction == 'move_exercise') {
            move_exercise($action);
        }
        else if ($subaction == 'move_exercise_ajax') {
            move_exercise_ajax($action);
        }
        else if ($subaction == 'pick_semester_ajax') {
            pick_semester_ajax($action);
        }
        else if ($subaction == 'pick_test_ajax') {
            pick_test_ajax($action);
        }
        else if ($subaction == 'preview_exercise') {
            preview_exercise($action);
        }
        else if ($subaction == 'copy_exercise_ajax') {
            copy_exercise($action, true);
        }
        else if ($subaction == 'move_assignment') {    //???
            move_assignment($action);
        }
        else if ($subaction == 'stopgo_assignment') {
            stopgo_assignment($action);
        }
        else if ($subaction == 'start_assignment') {
            start_assignment($action);
        }
        else if ($subaction == 'show_exercise') {
            show_exercise($action);
        }
        else if ($subaction == 'show_solution_to_lecturer') {   //???
            show_solution_to_lecturer($action);
        }
        else if ($subaction == 'submit_exercise') {
            submit_exercise($action);
        }
        else if ($subaction == 'add_block') {
            add_block($action);
        }
        else if ($subaction == 'assignment_options') {
            assignment_options($action);
        }
        else if ($subaction == 'delete_solutions') {
            delete_solutions($action);
        }
        else {
            list_assignments($action);
        }
    }

    /*********************/
    /* STUDENT FUNCTIONS */
    /*********************/
    else {
        if ($subaction == 'show_exercise') {
            show_exercise($action);
        }
        else if ($subaction == 'submit_exercise') {
            submit_exercise($action);
        }
        else if ($subaction == 'delete_solutions') {
            delete_solutions($action);
        }
        else {
            list_assignments_stud($action);
        }
    }
}

#####################################
#####################################
#                                   #
#          Student Methods          #
#                                   #
#####################################
#####################################

/**
 *
 * EXAM
 * Only possible if exam is selftest: Deletes all the solutions of a student to enable him/her
 * to redo it.
 *
 */
function delete_solutions($p_action) {
        global $vips_db,
               $g_outlet,
               $g_inAssignment_table,
               $g_assignment_table,
               $g_solution_points_table,
               $g_solution_table;

        $assignment_id = vips_get_int_param("assignment_id");

        if (vips_get_submit_param('cancel'))
        {
                list_assignments_stud($p_action);
        }
        else if (vips_get_submit_param('delete_solutions_confirmed'))
        {
                $vips_db->executeUpdate("UPDATE $g_solution_table
                                        SET isLastSolution = 'f'
                                        WHERE Assignmentid = '$assignment_id'
                                        AND Userid = '$g_outlet->userID'");

                $vips_db->executeUpdate("DELETE FROM vips_aufgaben_zeit
                                        WHERE Kurs              = '$g_outlet->courseID'
                                        AND $g_assignment_table = '$assignment_id'
                                        AND user_id             = '$g_outlet->userID'");

                $message['mode']        = 'msg';
                $message['message']     = 'Die Lsungen wurden gelscht!';
                $_POST['message']       = $message;
                list_assignments_stud($p_action);
        }
        else
        {
                $_POST['delete_assignment_id']  = $assignment_id;
                $_POST['dialog']                = 'delete_solutions_confirm';
                list_assignments_stud($p_action);
        }
}

/**
 * SHEETS/EXAMS
 *
 * Is called when the submit button at the bottom of an exercise is called.
 * If there is already a solution of this exercise by the same user or same group,
 * a dialog pops up to confirm the submission. On database-level: EVERY solution is stored
 * (even the unconfirmed ones), with the last solution being marked as last.
 *
 */

function submit_exercise($p_action) {
    global $vips_db,
           $g_outlet,
           $g_inAssignment_table,
           $g_assignment_table,
           $g_solution_table;

    $exercise_type     = vips_get_option_param("exercise_type");
    $exercise_id       = vips_get_int_param("exercise_id");
    $exercise_position = vips_get_int_param("exercise_position");
    $assignment_id     = vips_get_int_param("assignment_id");

    // NOTE number_of_answers may contain comma separated values here
    $number_of_answers  = vips_get_string_param("number_of_answers");
    $student_comment    = vips_get_string_param("student_comment", true);

    if (vips_get_int_param("in_rnd_exercise")) {
        $exercise_id = vips_get_int_param("rnd_exercise_id");
    }

    $message = array();

    $pl_out = '';

    $solved = vips_get_int_param("solved");
    $forced = vips_get_int_param("forced");


    ##################################################################
    # in case student solution is submitted by tutor or lecturer     #
    # (can happen if the student submits his/her solution by email)  #
    ##################################################################

    $solver_user_id = vips_get_option_param("solver_user_id");

    if ($solver_user_id == '' || $g_outlet->status == 'student') {
        $solver_user_id = $g_outlet->userID;
    }

    // from here on we only use $solver_user_id and not $g_outlet->userID

    ###################################
    #  get info of Selbsttest (EXAMS) #
    ###################################

    $result = $vips_db->executeQuery("SELECT * FROM $g_assignment_table
                                      WHERE ID  = '$assignment_id'
                                      ");
    if ($result->next()) {
        $entry = $result->getRow();
        $selftest = $entry['Selbsttest'];
        $ende = $entry['Ende'];
    }

    ############################
    # Checks before submission #
    ############################

    // if time has elapsed
    if ((getRemainingMinutes($assignment_id) <= -2 || date('Y-m-d H:i:s') > $ende) &&
        $selftest == 0 && $p_action == 'exams' &&
        $g_outlet->status == 'student')
    {
        $message['mode']    = 'error';
        $message['message'] = 'Ihre Zeit ist leider abgelaufen! Ihre Lsung wurde gespeichert und mit dem Abgabezeitpunkt versehen.';
        $_POST['message']   = $message;
        //list_assignments_stud($p_action); #vvvvvv continue to store vvvvvvvvv
    }
    // if nobody is logged on
    else if ($g_outlet->status == 'nobody')
    {
        $message['mode']    = 'error';
        $message['message'] = 'Ihre Session ist leider abgelaufen. Bitte loggen Sie sich erneut ein!';
        $_POST['message']   = $message;
        list_assignments_stud($p_action);
    }

    if ($g_outlet->status != 'nobody') {
        /*
         * special case for Prolog exercises:
         * submit_exercise() can be called without actually wanting to submit a solution but to
         * send a solution to the PROLOG server
         */
        if (vips_get_submit_param("pl_exercise_mode_query") ||
            vips_get_submit_param("pl_exercise_mode_querynext") ||
            vips_get_submit_param('pl_exercise_mode_upload') ||
            vips_get_submit_param('pl_exercise_mode_download'))
        {
            require_once("exercises/pl_exercise.php");

            $o_exercise = new pl_exercise;

            //$solution_XML         = $o_exercise->genSolution($exercise_id, $number_of_answers); //needed?

            $answer_0 = vips_get_string_param('answer_0', true);

            if (vips_get_submit_param('pl_exercise_mode_query')) {
                $pl_out  = $o_exercise->evaluateQuery($solver_user_id, $g_outlet->courseID);
            }
            elseif (vips_get_submit_param('pl_exercise_mode_querynext')) {
                $pl_count = vips_get_int_param('pl_count') + 1;
                $pl_out   = $o_exercise->evaluateQuery($solver_user_id, $g_outlet->courseID, $pl_count);
            }

            elseif (vips_get_submit_param('pl_exercise_mode_upload')) {
                $answer_0 = addslashes($o_exercise->uploadSolution());  // magic_quotes_gpc
            }

            elseif (vips_get_submit_param('pl_exercise_mode_download')) {
                $sql = "SELECT Name FROM vips_aufgabe WHERE ID = '$exercise_id'";
                $result = $vips_db->executeQuery($sql);
                if ($result->next()) {
                    $filename = $result->getString('Name');
                }
                $o_exercise->downloadSolution(preg_replace('/\W/', '_', $filename));
            }
            $_POST['assignment_id']     = $assignment_id;
            $_POST['exercise_position'] = $exercise_position;
            $_POST['answer_0']          = $answer_0;
            $_POST['pl_query']          = vips_get_string_param('pl_query', true);
            $_POST['pl_out']            = $pl_out;
            $_POST['pl_count']          = $pl_count;
            $_POST['message']           = $message;
            unset($_GET['next']);  // to prevent show_exercise from displaying the next question
            unset($_GET['prev']);  //                                          ... previous ...

            show_exercise($p_action);
        }  // end prolog-specific
        // default: store exercise
        else {
            $flag = false;   //is set to TRUE further down in the confirmation dialog (just internal communication...)
            $new_solution_id = 0;

            /* if an exercise has been submitted */
            if (vips_get_submit_param('submit_exercise') || $forced)
            {
                require_once("exercises/$exercise_type.php");

                $o_exercise     = new $exercise_type;
                $solution_XML   = $o_exercise->genSolution($exercise_id, $number_of_answers);

                //get group id if applicable
                $result2 = $vips_db->executeQuery(
                    "SELECT vips_inGruppe.* FROM
                            vips_inGruppe,
                            vips_gruppe
                    WHERE user_id = '$solver_user_id'
                    AND  vips_gruppe.Gruppenid                      = vips_inGruppe.group_id
                    AND  vips_gruppe.Kursid                         = '$g_outlet->courseID'
                    AND  vips_inGruppe.end IS NULL");

                $group_id = 0;
                if ($result2->next()) {
                    $group = $result2->getRow();
                    $group_id = $group['group_id'];
                }

                $current_ip = $_SERVER['REMOTE_ADDR'];

                $sql = "INSERT INTO $g_solution_table
                        SET
                            Loesung                 = '$solution_XML',
                            Aufgabenid              = '$exercise_id',
                            Assignmentid    = '$assignment_id',
                            Userid                  = '$solver_user_id',
                            Gruppenid               = '$group_id',
                            Zeitpunkt               = NOW(),
                            isLastSolution  = 'f',
                            Anmerkung               = '$student_comment'
                        ";
                //store IP address of current user too
                if ($p_action == 'exams') {
                    $sql .= ", IPAdresse    = '$current_ip'";
                }

                $vips_db->executeUpdate($sql);
                $vips_idgen = $vips_db->getIdGenerator();
                $new_solution_id = $vips_idgen->getId();
                $_POST['new_solution_id'] = $new_solution_id;

                //if exercise has been solved before, envoke confirmation dialog but keep ID of
                //new solution "in mind"
                if ($solved == 1 && $forced == 0 && $g_outlet->status == 'student' && $p_action == 'sheets') {
                    $dialog = 'overwrite';
                    $flag = true;
                    $_POST['dialog'] = $dialog;
                    $_POST['new_solution_id'] = $new_solution_id;
                    unset($_GET['next']);
                    unset($_GET['prev']);
                    show_exercise($p_action);
                }
            }

            // if the submission has been cancelled (from dialog)
            if (vips_get_submit_param('cancel'))
            {
                list_assignments_stud($p_action);
            }
            // else store user solution
            elseif (!$flag)
            {
                //get group id if applicable
                $result2 = $vips_db->executeQuery(
                    "SELECT vips_inGruppe.* FROM
                            vips_inGruppe,
                            vips_gruppe
                    WHERE user_id = '$solver_user_id'
                    AND  vips_gruppe.Gruppenid = vips_inGruppe.group_id
                    AND  vips_gruppe.Kursid    = '$g_outlet->courseID'
                    AND  vips_inGruppe.end IS NULL");
                $group_id = 0;
                if ($result2->next()) {
                    $group = $result2->getRow();
                    $group_id = $group['group_id'];
                }

                ################################################
                # UPDATE old solutions                         #
                # (set all solutions of this user and exercise #
                # on isLastSolution='f')                       #
                ################################################
                if ($p_action == 'exams') {
                    $sql = "UPDATE $g_solution_table
                            SET isLastSolution      = 'f'
                            WHERE Aufgabenid        = '$exercise_id'
                            AND   Assignmentid      = '$assignment_id'
                            AND   Userid            = '$solver_user_id'";
                } else if ($group_id != 0) {
                    $sql = "UPDATE $g_solution_table
                            SET isLastSolution      = 'f'
                            WHERE Aufgabenid        = '$exercise_id'
                            AND   Assignmentid      = '$assignment_id'
                            AND   Gruppenid         = '$group_id'";
                } else {
                    $sql = "UPDATE $g_solution_table
                            SET isLastSolution      = 'f'
                            WHERE Aufgabenid        = '$exercise_id'
                            AND   Assignmentid      = '$assignment_id'
                            AND   Userid            = '$solver_user_id'";
                }

                $vips_db->executeUpdate($sql);

                ###############################################
                # UPDATE new solution                                             #
                # (set new solution on isLastSolution = 't')  #
                ###############################################

                $new_solution_id = vips_get_int_param('new_solution_id');
                $sql = "UPDATE $g_solution_table
                        SET isLastSolution      = 't'
                        WHERE Loesungsid        = '$new_solution_id'";
                $vips_db->executeUpdate($sql);

                $_POST['assignment_id']         = $assignment_id;
                $_POST['exercise_position']     = $exercise_position;
                $_POST['solved'] = true;
                $_POST['dialog'] = 'submitted';
                unset($_GET['next']);
                unset($_GET['prev']);
                show_exercise($p_action);
            }
        }
    }
}

/**
 * SHEETS/EXAMS
 *
 * Displays an exercise (from student perspective)
 *
 * This function is also invoked by a function in vips_solutions.inc.php to show the
 * solution of a student to a lecturer.
 */
function show_exercise($p_action)
{
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table,
        $g_solution_table,
        $g_solution_points_table,
        $vipsTemplateFactory,
        $vipsPlugin;

    $assignment_id     = vips_get_int_param('assignment_id');
    $exercise_position = vips_get_int_param('exercise_position'); //identifier! (instead of exercise_id)

    // If exercise is in a rnd_exercise, it has no assignment or position! Then: get its id.
    $in_rnd_exercise = vips_get_int_param("in_rnd_exercise");
    if ($in_rnd_exercise) {
        $exercise_id           = vips_get_int_param("exercise_id");
        $rnd_exercise_id       = vips_get_int_param("rnd_exercise_id");
        $rnd_exercise_name     = vips_get_string_param("rnd_exercise_name");
        $rnd_exercise_position = vips_get_int_param("rnd_exercise_position");
    }

    $solver_id        = harmless(vips_get_string_param("solver_id"));  // solver is handed over via address line, ie. user is either a lecturer or a student who is looking at corrections
    $solver_user_id   = vips_get_option_param("solver_user_id");  // solver is handed over via address line, lecturer is submitting exercise for student
    $solver_type      = vips_get_option_param('solver_type');
    $single_solver    = vips_get_int_param("single_solver"); //user is not participant in any group
    $submitted        = vips_get_int_param("submitted");    //from edit_solution (for Prolog)
    $print            = vips_get_int_param('print');
    $print_correction = vips_get_int_param('print_correction');
    $samplesolution   = vips_get_int_param("samplesolution");
    $view             = vips_get_option_param('view');
    $ajax_updating    = vips_get_int_param('ajax_updating');
    $solution_view    = vips_get_option_param('solution_view');
    $korrektur        = 0;

    // this prevents URL-hacking to get the correction view
    if ($solver_id && vips_get_option_param('subaction') != 'show_exercise') {
        $korrektur = 1;
    }

    else {
        if ($g_outlet->status == 'student' || !$solver_user_id) {
            $solver_user_id = $g_outlet->userID;
        }

        $solver_id = $solver_user_id;                   // "solver_id" is the student himself
        $single_solver = true;

        if ($p_action == 'sheets') {
            $query = "SELECT group_id
                FROM vips_inGruppe, vips_gruppe
                WHERE user_id = '".$solver_user_id."'
                    AND vips_gruppe.Kursid = '".$g_outlet->courseID."'
                    AND vips_inGruppe.group_id = vips_gruppe.Gruppenid
                    AND vips_inGruppe.end IS NULL";
            $result = $vips_db->executeQuery($query);

            if ($result->next()) {
                $solver_id = $result->get("group_id");  // "solver_id" is the group the student is in
                $single_solver = false;
            }
        }
    }

    $message = vips_get_string_array_param('message');   //??? needed???

    $dialog          = vips_get_string_param('dialog');                   //internal communication :-) not from forms
    $new_solution_id = vips_get_int_param('new_solution_id');

    if (vips_get_int_param('next'))  //$_GET variable in link "Zur naechsten Aufgabe"
    {
        $exercise_position += 1;
    }
    else if (vips_get_int_param('prev'))
    {
        $exercise_position -= 1;
    }

    $solved        = 0 ;   //vips_get_int_param('solved');
    $show_solution = 0 ;

    //get number of exercises in assignment (for footer)


    ################################
    #    get Selbsttest for exams  #
    ################################

    $selftest = 0;

    // fetch all exercises in the assignment
    $result = $vips_db->executeQuery(
        "SELECT vips_aufgabe.*
        FROM  vips_aufgabe, $g_inAssignment_table
        WHERE $g_inAssignment_table.$g_assignment_table = '$assignment_id'
        AND vips_aufgabe = vips_aufgabe.ID
        ");
    $number_of_exercises = $result->getRecordCount();

    $result = $vips_db->executeQuery("SELECT Selbsttest, mc_auswertung
        FROM $g_assignment_table
        WHERE ID  = '$assignment_id'
        ");
    if ($result->next()) {
        $entry = $result->getRow();
        $selftest = $entry['Selbsttest'];
        $mc_auswertung = $entry['mc_auswertung'];

        // don't show group solutions to a student in selftest!
        if ($selftest == 1 && $g_outlet->status == 'student') {
            $single_solver = true;
            $solver_id     = $solver_user_id;
        }
    }



    #####################################
    #       get exercise info           #
    #####################################

    // if the display function is called from a rnd_exercise, the exercise
    // has no assignment and no position, just an id...
    if ($in_rnd_exercise)
    {
        // fetch info, type from subexercise, points from rnd_exercise
        $query = "SELECT DISTINCT
                `vips_aufgabe`.`Aufgabe`,
                `vips_aufgabe`.`Name`,
                `vips_aufgabe`.`ID`,
                `vips_aufgabe`.`URI`,
                `vips_aufgaben_typen`.`Typenname`,
                $g_inAssignment_table.Punkte
            FROM
                `vips_aufgabe`,
                `vips_aufgaben_typen`,
                $g_inAssignment_table
            WHERE
                `vips_aufgabe`.`ID` = '$exercise_id'
                AND `vips_aufgaben_typen`.`URI` = `vips_aufgabe`.`URI`
                AND $g_inAssignment_table.vips_aufgabe = $rnd_exercise_id
            ";
    }
    // default
    else
    {
        // fetch exercise info, type and points, determined by assignment and position
        $query = "SELECT DISTINCT
                vips_aufgabe.*,
                vips_aufgaben_typen.Typenname,
                $g_inAssignment_table.Punkte
            FROM
                vips_aufgabe,
                vips_aufgaben_typen,
                $g_inAssignment_table
            WHERE
                $g_inAssignment_table.$g_assignment_table = '$assignment_id'
                AND $g_inAssignment_table.vips_aufgabe    = vips_aufgabe.ID
                AND $g_inAssignment_table.Position        = '$exercise_position'
                AND vips_aufgaben_typen.URI               = vips_aufgabe.URI
            ";
    }

    $result = $vips_db->executeQuery($query);
    $result->next();

    $exercise_XML        = $result->get('Aufgabe');
    $exercise_name       = $result->get('Name');
    $exercise_id         = $result->get('ID');
    $exercise_type       = $result->get('URI');
    $exercise_title      = $result->get('Typenname');
    $exercise_max_points = $result->getFloat('Punkte');

    // special treatment for rnd_exercise: choose subexercise from XML and get info from db
    if ($exercise_type == "rnd_exercise")
    {
        $in_rnd_exercise    = 1;
        $rnd_exercise_id    = $exercise_id;
        $rnd_exercise_empty = 1;

        require_once("exercises/rnd_exercise.php");
        $rnd0_exercise = new rnd_exercise();

        $subexercise_id = $rnd0_exercise->choose_subexercise($exercise_XML, $solver_id, $exercise_id);  // false if no subexercises exist

        if ($subexercise_id)
        {
            $rnd_exercise_empty = 0;

            $query = "SELECT Aufgabe, Name, ID, vips_aufgabe.URI, Typenname
                FROM vips_aufgabe, vips_aufgaben_typen
                WHERE ID = '".$subexercise_id."'
                    AND vips_aufgaben_typen.URI = vips_aufgabe.URI
                ";
            $result = $vips_db->executeQuery($query);
            $result->next();

            $exercise_XML   = $result->get("Aufgabe");
            $exercise_name  = $result->get("Name");
            $exercise_id    = $result->get("ID");
            $exercise_type  = $result->get("URI");
            $exercise_title = $result->get("Typenname");
        }
    }



    ##############################################################
    #    check for ip_address, remaining time and interrupted    #
    ##############################################################

    $error = false;

    // restrict access for students!
    if ($g_outlet->status == 'student')
    {
            // fetch end time for assignment
            $query = "SELECT Ende
                    FROM $g_assignment_table
                    WHERE ID = '$assignment_id'
                            AND Kurs = '".$g_outlet->courseID."'";
            $result = $vips_db->executeQuery($query);
            $result->next();
            $assignment_end_time = $result->get("Ende");

            // if student wants to see the Korrekturansicht, but assignment is still running
            if ($korrektur && !$print && time() <= strtotime($assignment_end_time))
            {
                    // if one changes the system time and this way tries to see the
                    // Korrekturansicht, this won't work, as assignment is not
                    // accessible any more after it has run out
                    $error = true;
                    $message["mode"] = 'error';
                    $message["message"] = _("Die Korrekturen drfen erst nach Ablaufen des Aufgabenblattes betrachtet werden")."!";
                    $_POST["message"] = $message;
                    list_assignments_stud($p_action);
            }

            if ($p_action == 'exams' && $selftest == 0 && !$korrektur) {
                    if (ip_permitted($assignment_id) == FALSE) {
                            $error = true;
                            $message['mode']        = 'error';
                            $message['message'] = 'Kein Zugriff!';
                            $_POST['message']       = $message;
                            list_assignments_stud($p_action);
                    }
                    elseif (getRemainingMinutes($assignment_id) <= 0) {
                            $error = true;
                            $message['mode']        = 'error';
                            $message['message'] = 'Die Zeit ist leider abgelaufen!';
                            $_POST['message']       = $message;
                            list_assignments_stud($p_action);
                    }
                    elseif (checkInterrupted($assignment_id) == 'y') {
                            $error = true;
                            $message['mode']        = 'error';
                            $message['message'] = 'Die Klausur wurde vom Dozenten unterbrochen!';
                            $_POST['message']       = $message;
                            list_assignments_stud($p_action);
                    }
            }
            else {
                    if (checkInterrupted($assignment_id) == 'y') {
                            $error = true;
                            $message['mode']        = 'error';
                            $message['message'] = 'Das bungsblatt wurde vom Dozenten unterbrochen!';
                            $_POST['message']       = $message;
                            list_assignments_stud($p_action);
                    }
            }
    }

    if (!$error) {

        $remainingMinutes = getRemainingMinutes($assignment_id);
        $studentAbgabe    = getAbgabezeitpunkt($assignment_id);  //different for different students!



        ##########################################################
        # get number of exercises in assignment (for navigation) #
        ##########################################################

        $result = $vips_db->executeQuery("SELECT vips_aufgabe.*
            FROM vips_aufgabe, $g_inAssignment_table
            WHERE $g_inAssignment_table.$g_assignment_table = '$assignment_id'
            AND vips_aufgabe = vips_aufgabe.ID");
        $number_of_exercises = $result->getRecordCount();

        // enter user start time the moment he/she first clicks on any exercise
        $result = $vips_db->executeQuery("SELECT * FROM vips_aufgaben_zeit
            WHERE Kurs              = '$g_outlet->courseID'
            AND $g_assignment_table = '$assignment_id'
            AND user_id             = '$g_outlet->userID'");

        if ($result->getRecordCount() == 0) {
            $ip_address = $_SERVER['REMOTE_ADDR'];
            $vips_db->executeUpdate("INSERT INTO vips_aufgaben_zeit
                SET Kurs                = '$g_outlet->courseID',
                    $g_assignment_table = '$assignment_id',
                    user_id             = '$g_outlet->userID',
                    Beginn              = NOW(),
                    ip                  = '$ip_address',
                    vips_aufgabe        = '$exercise_id'");
        }



        require_once("exercises/$exercise_type.php");
        $o_exercise = new $exercise_type($exercise_XML);
        $o_exercise->setEvaluationMode($mc_auswertung);

        $exercise_question = $o_exercise->getQuestion();
        $exercise_comment  = $o_exercise->getCommentAllowed();
        $exercise_hint     = $o_exercise->getHint();
        $number_of_answers = $o_exercise->getNumberOfAnswers();
        $answer_array      = $o_exercise->getAnswerArray();
        $correct_array     = $o_exercise->getCorrectArray();
        $quirks            = $o_exercise->getQuirks();

        // TODO this needs to be moved into the pl_exercise class
        if ($exercise_type == 'pl_exercise') {
            $answer = str_replace('m_l_', '', $answer_array[0]);
            list($answer, $tests) = split('%T', $answer);
            $answer_array[0] = $answer;
        }


        ###################################
        # get user solution if applicable #
        ###################################

        // check if there is already a user's solution OR a group solution

        if ($p_action == 'sheets' && !$single_solver) {
            // check if there is already a group solution
            $solver_field = 'Gruppenid';
        } else {
            // check if the user is in no group but has still submitted an answer
            $solver_field = 'Userid';
        }

        $sql = "SELECT * FROM $g_solution_table
                WHERE Aufgabenid      = '".($in_rnd_exercise ? $rnd_exercise_id : $exercise_id)."'
                 AND  Assignmentid    = '$assignment_id'
                 AND  $solver_field   = '$solver_id'
                 AND  isLastSolution  = 't'";
        $result2 = $vips_db->executeQuery($sql);

        if ($result2->next()) {
            $solution           = $result2->getRow();
            $solution_XML       = $solution['Loesung'];
            //vv important: student_array with complete user solution vv
            $student_array      = $o_exercise->getTagsFromXML($solution_XML, "answer");

            $student_comment    = $solution['Anmerkung'];
            $solution_id        = $solution['Loesungsid'];
            $lecturer_comment   = $solution['Korrektur'];
            $commented_solution = $solution['bearbeiteteLoesung'];
            $solved = 1;
        }

        // mapping exercise: if exercise has been worked on, check if student gave an
        // answer twice
        if ($exercise_type == "rh_exercise" && $solved) {
            if (!$o_exercise->checkSolutionXML($solution_XML)) {
                $message = array('mode' => 'info',
                                 'message' => _('Vorsicht! Einer einzelnen Vorgabe wurden mehrere Texte zugeordnet.'));
            }
        }

        $max_points = 1;
        $solution_proz = round(($reached_points*100)/$max_points);

        if (!$reached_points) { $reached_points = 0; }
        if (!$solution_proz) { $solution_proz = 0; }


        ##################################################
        #  lecturer: get all course participants         #
        #  (to list for submission of student solution)  #
        ##################################################

        if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
            $course_participants = array();

            $result = $vips_db->executeQuery("SELECT seminar_user.*,
                        auth_user_md5.username,
                        auth_user_md5.Vorname,
                        auth_user_md5.Nachname,
                        auth_user_md5.perms
                FROM seminar_user,
                         auth_user_md5
                WHERE   seminar_user.Seminar_id = '$g_outlet->courseID'
                AND     seminar_user.user_id    = auth_user_md5.user_id
                ORDER BY auth_user_md5.Nachname, auth_user_md5.Vorname");

            while ($result->next()) {
                    $course_participant = $result->getRow();
                    $course_participants[] = $course_participant;
            }
        }

        #####################################################
        #   set show_solution (to display correct solution  #
        #   in blue in addition to user solution)            #
        #####################################################
        //if a solution has been submitted during a selftest OR lecturer is correcting
        if ($korrektur || ($dialog == 'submitted' && $selftest)) {
            $show_solution = 1;
        }

        ################################
        #     construct templates      #
        ################################

        if (!$show_solution && !$korrektur && !$print) {  // solve exercise
            $t_main_header = $vipsTemplateFactory->open('exercises/show_exercise_header');
            $t_main        = $o_exercise->getSolveTemplate($solution_XML);
            $t_main_footer = $vipsTemplateFactory->open('exercises/show_exercise_footer');
        }  // TODO Hier auch fr Korrektur und Druck umbauen
        else {
            $t_main_header = $vipsTemplateFactory->open('exercises/show_exercise_header');
            $t_main        = $vipsTemplateFactory->open('exercises/show_'.$exercise_type.'.php');
            $t_main_footer = $vipsTemplateFactory->open('exercises/show_exercise_footer');

            $tbquer = false;
            if ($exercise_type == 'tb_exercise' && !$print) {
                $t_main = $vipsTemplateFactory->open('exercises/show_'.$exercise_type.'_tbquer');
                $tbquer = true;
            }

            /***** set variables for exercise specific template (t_main) ******/
            /******************************************************************/
            $t_main->set_attributes(array(
                'picturepath'        => $g_outlet->picturepath,
                'vips_imagepath'     => $vipsPlugin->getPluginURL().'/images/',
                'solved'             => $solved,  // and now the *template* "knows" what to display
                'exercise_name'      => $exercise_name,      // for tb_quer
                'exercise_question'  => $exercise_question,  // for tb_quer
                'exercise_hint'      => $exercise_hint,      // for tb_quer

                'status'             => $g_outlet->status,
                'show_solution'      => $show_solution,
                'korrektur'          => $korrektur,
                'commented_solution' => $commented_solution,
                'submitted'          => $submitted,

                'dialog'             => $dialog,

                'answer_array'       => $answer_array,
                'correct_array'      => $correct_array,

                // student answers:
                'student_array'      => $student_array,

                'view'               => $view,
                'mc_auswertung'      => $mc_auswertung,

                'student_comment'    => $student_comment,
                'user_solution'      => $user_solution,

                'rnd_exercise_empty' => $rnd_exercise_empty,

                'exercise_id'        => $exercise_id,
                'assignment_id'      => $assignment_id,
                'assignment_type'    => $p_action == 'exams' ? 'vips_klausur' : 'vips_uebungsblatt',
                'exercise_position'  => $exercise_position,
                'solver_id'          => $solver_id,
                'single_solver'      => $single_solver,
                'solver_type'        => $solver_type,
                'solution_view'      => $solution_view,
                'in_rnd_exercise'    => $in_rnd_exercise,
                'rnd_exercise_id'    => $rnd_exercise_id,
                'ajax_updating'      => $ajax_updating,
                'quirks'             => $quirks));
        }



        ##############################
        #   set template variables   #
        ##############################

        /****  set general exercise info variables (show_exercise_header) ****/
        /*********************************************************************/

        /* don't display negative remaining time (unused) */
        if ($remainingMinutes <= 0) $remainingMinutes = 0;

        $t_main_header->set_attributes(array(
            'action'                => $p_action,
            'picturepath'           => $g_outlet->picturepath,
            'message'               => $message,
            'tbquer'                => $tbquer,
            'solved'                => $solved,
            'selftest'              => $selftest,  //fuer uhranzeige (nur wenn kein selbsttest)
            'show_solution'         => $show_solution,
            'course_participants'   => $course_participants,
            'solver_user_id'        => $solver_user_id,

            'time'                  => time(),
            'remainingMinutes'      => $remainingMinutes,
            'studentAbgabe'         => $studentAbgabe,
            'new_solution_id'       => $new_solution_id,

            'subaction'             => $show_solution ? 'show_solution'
                                                      : 'submit_exercise',

            'assignment_id'         => $assignment_id,
            'exercise_id'           => $exercise_id,
            'exercise_type'         => $exercise_type,
            'exercise_position'     => $exercise_position,
            'exercise_title'        => $exercise_title,
            'exercise_name'         => $exercise_name,
            'exercise_question'     => $exercise_question,
            'exercise_hint'         => $exercise_hint,

            'solution_id'           => $solution_id,  // can be empty
            'number_of_exercises'   => $number_of_exercises,

            'solver_id'             => $solver_id,
            'single_solver'         => $single_solver,
            'solver_type'           => $solver_type,
            'solved'                => $solved,
            'lecturer_comment'      => $lecturer_comment,
            'reached_points'        => $reached_points,
            'max_points'            => $exercise_max_points,

            'status'                => $g_outlet->status,

            'korrektur'             => $korrektur,

            'in_rnd_exercise'       => $in_rnd_exercise,
            'rnd_exercise_id'       => $rnd_exercise_id,
            'rnd_exercise_name'     => $rnd_exercise_name,
            'rnd_exercise_position' => $rnd_exercise_position,

            'dialog'                => $dialog));  // internal communication :-)

        if (in_array($exercise_type, array('tb_exercise', 'oct_exercise', 'oc_exercise', 'lt_exercise', 'pl_exercise'))) {
            $t_main_header->set_attribute('number_of_answers', 1);
            $t_main_header->set_attribute('exercise_number', 1);  // necessary for oc_, oct_, pl_exercises (and in lt and tb it doesn't hurt)
        }
        else {
            $t_main_header->set_attribute('number_of_answers', $number_of_answers);
        }



        /*********** set exercise footer variables (t_main_footer) ************/
        /**********************************************************************/

        //$t_main_footer->set_attribute('exercise_solution_proz',       $solution_proz);

        $t_main_footer->set_attributes(array(
            'action'              => $p_action,
            'picturepath'         => $g_outlet->picturepath,
            'korrektur'           => $korrektur,
            'status'              => $g_outlet->status,
            'show_solution'       => $show_solution,
            'student_comment'     => $student_comment,  // only set if solved, else empty
            'exercise_comment'    => $exercise_comment,  // true or false
            'assignment_id'       => $assignment_id,
            'exercise_id'         => $exercise_id,
            'exercise_position'   => $exercise_position,
            'exercise_type'       => $exercise_type,
            'dialog'              => $dialog,  // internal communication :-)
            'number_of_exercises' => $number_of_exercises,
            'solved'              => $solved,
            'solution_proz'       => $solution_proz));

        if ($g_outlet->status == "lecturer")
        {
            // Setting this attribute will result in that a link is shown in the
            // footer which allows to jump back to editing the random exercise.
            // This is ONLY wanted if the LECTURER is looking at the exercise,
            // NOT if a STUDENT is solving the exercise!
            $t_main_footer->set_attribute("in_rnd_exercise", $in_rnd_exercise);
        }
        $t_main_footer->set_attribute("rnd_exercise_id", $rnd_exercise_id);
        $t_main_footer->set_attribute("rnd_exercise_name", $rnd_exercise_name);
        $t_main_footer->set_attribute("rnd_exercise_position", $exercise_position);
        $t_main_footer->set_attribute("rnd_exercise_empty", $rnd_exercise_empty);

        if ($print) {
            $t_main_header->set_attribute("print", true);
            $t_main->set_attribute("print", true);
            $t_main->set_attribute('korrektur', $print_correction);
            $t_main_footer->set_attribute("print", true);
            // determine whether to show lecturer's corrections
            //$t_main->set_attribute('show_solution', false);
            $t_main->set_attribute("samplesolution", $samplesolution);
        }



        if ($ajax_updating)  // update correction view
        {
            ob_end_clean();
            // note: IE requires a declared ecoding of windows-1252
            header('Content-Type: text/html; charset=windows-1252');

            setlocale(LC_NUMERIC, NULL);
            echo $t_main->render();
            setlocale(LC_NUMERIC, 'C');

            ob_start('discard_buffer');
        }
        else  // standard display exercise
        {
            setlocale(LC_NUMERIC, NULL);
            echo $t_main_header->render();
            setlocale(LC_NUMERIC, 'C');

            if (!$dialog || $dialog == 'submitted')  {
                echo $t_main->render();  // normal show exercise
                echo $t_main_footer->render();
            }
        }
    } // endif user has access to exam
}  // end function show_exercise()

/**
 *
 * Displays all running assignments "work-on ready" for students (view of students when clicking on
 * "Reiter" Uebungsblatt or Klausur)
 *
 */
function list_assignments_stud($p_action) {
        global $vips_db,
               $g_outlet,
               $g_title_p,
               $g_inAssignment_table,
               $g_assignment_table,
               $g_solution_table,
               $vipsTemplateFactory;

        // messages for no access, no time, interrupted (array)
        $dialog = vips_get_string_param('dialog');
        $message = vips_get_string_array_param('message');

        $delete_assignment_id   = vips_get_int_param('delete_assignment_id');

        $assignments_select = array();
        $assignments        = array();

        $t_list = $vipsTemplateFactory->open('list_assignments_stud');
        $t_list->set_attribute('picturepath', $g_outlet->picturepath);
        $t_list->set_attribute('action', $p_action);
        $t_list->set_attribute('status', $g_outlet->status);
        $t_list->set_attribute('assignment_id', $delete_assignment_id);

        if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
                if ($assignment_id = vips_get_int_param('assignment_id')) {       //lecturer wants to display selected assignment for himself
                        $result = $vips_db->executeQuery("SELECT * FROM $g_assignment_table WHERE ID = '$assignment_id'");
                        $result->next();
                        $assignment = $result->getRow();
                        $assignments_select[] = $assignment;
                        $t_list->set_attribute('assignment_id', $assignment_id);

                        $now = date("Y-m-d H:i:s");
                        if ($now > $assignment["Ende"]) {
                                $running_status = "expired";
                        } else if ($now < $assignment["Beginn"]) {
                                $running_status = "pending";
                        } else {
                                $running_status = "running";
                        }

                        // fetch information on users which are working on this assignment
                        $query = "SELECT DISTINCT
                                        `vips_aufgaben_zeit`.`user_id`,
                                        `vips_aufgaben_zeit`.`Beginn`,
                                        `vips_aufgaben_zeit`.`ip`,
                                        `auth_user_md5`.`username`,
                                        ".$GLOBALS['_fullname_sql']['full']." AS fullname
                                FROM
                                        `vips_aufgaben_zeit`,
                                        `auth_user_md5`
                                LEFT JOIN
                                        user_info USING (user_id)
                                WHERE `vips_aufgaben_zeit`.`Kurs` = '".$g_outlet->courseID."'
                                        AND `vips_aufgaben_zeit`.`vips_klausur` = '$assignment_id'
                                        AND `vips_aufgaben_zeit`.`user_id` = `auth_user_md5`.`user_id`
                                GROUP BY `vips_aufgaben_zeit`.`user_id`
                                ORDER BY
                                        `auth_user_md5`.`Nachname`,
                                        `auth_user_md5`.`Vorname`
                                ";
                        $result = $vips_db->executeQuery($query);

                        while ($result->next())
                        {
                                $remaining_time = clip_remaining_time($result->get('Beginn'), $assignment['Dauer'], $assignment['Ende']);
                                $running_exam_info[] = array
                                        ( "username"       => $result->get("username")
                                        , "name"           => $result->get("fullname")
                                        , "ip"             => $result->get("ip")." (".gethostbyaddr($result->get("ip")).")"
                                        , "start_time"     => date("H:i:s", strtotime($result->get("Beginn")))
                                        , "remaining_time" => $remaining_time);
                        }
                }
        }
        else { //get all running exercises for student

                $result3 = $vips_db->executeQuery("SELECT * FROM $g_assignment_table
                                        WHERE Kurs='$g_outlet->courseID'
                                        ORDER BY Beginn, Position");
                while ($result3->next()) {
                        $assignment = $result3->getRow();
                        $m_start_timestamp = strtotime($assignment['Beginn']);
                        $m_end_timestamp   = strtotime($assignment['Ende']);
                        if (time() <= $m_end_timestamp) {
                                if (time() < $m_start_timestamp) {
                                        $assignment['hidden'] = TRUE;
                                }
                                $assignments_select[] = $assignment;
                        }
                }
        }

        foreach ($assignments_select as $assignment) {
                $assignment_id = $assignment['ID'];

                $exercises = array();
                $sql =  "SELECT vips_aufgabe.*,
                                                $g_inAssignment_table.Position,
                                                $g_inAssignment_table.Punkte,
                                                vips_aufgaben_typen.Typenname
                                        FROM  vips_aufgabe,
                                                  $g_inAssignment_table,
                                                  $g_assignment_table,
                                                  vips_aufgaben_typen
                                        WHERE $g_inAssignment_table.$g_assignment_table = $g_assignment_table.ID
                                        AND   $g_assignment_table.ID            = '$assignment_id'
                                        AND   vips_aufgaben_typen.URI           = vips_aufgabe.URI
                                        AND   vips_aufgabe.ID                           = $g_inAssignment_table.vips_aufgabe
                                        ORDER BY $g_inAssignment_table.Position
                                                ";
                $result2 = $vips_db->executeQuery($sql);

                $assignment['ippermitted'] = true;
                if ($g_outlet->status == 'student' && $p_action == 'exams') {
                        $assignment['ippermitted'] = ip_permitted($assignment_id);
                }


                #########################
                # get old user solution #
                #########################

                //get group id if applicable
                ////////////////////////////////new///////////////////////////////////////
                $result4 = $vips_db->executeQuery(
                                "SELECT vips_inGruppe.* FROM
                                                vips_inGruppe,
                                                vips_gruppe
                                        WHERE user_id = '$g_outlet->userID'
                                        AND  vips_gruppe.Gruppenid      = vips_inGruppe.group_id
                                        AND  vips_gruppe.Kursid         = '$g_outlet->courseID'
                                        AND  vips_inGruppe.end IS NULL");
                $group_id = 0;
                if ($result4->next()) {
                        $group = $result4->getRow();
                        $group_id = $group['group_id'];
                }


                while ($result2->next()) {
                        $exercise = $result2->getRow();
                        $exercise['Punkte'] = $result2->getFloat('Punkte');
                        $exercise_id    = $exercise['ID'];
                        $user_id                = $g_outlet->userID;

                        if ($p_action == 'sheets' && $group_id != 0) {
                                $sql = "SELECT * FROM $g_solution_table
                                                WHERE Aufgabenid = '$exercise_id'
                                                AND Assignmentid = '$assignment_id'
                                                AND (Userid='$user_id' OR Gruppenid = '$group_id')
                                                AND isLastSolution = 't'";
                        } else {
                                $sql = "SELECT * FROM $g_solution_table
                                                WHERE Aufgabenid = '$exercise_id'
                                                AND Assignmentid = '$assignment_id'
                                                AND Userid='$user_id'
                                                AND isLastSolution = 't'";
                        }
                        $result3 = $vips_db->executeQuery($sql);

                        if ($result3->next()) {// means exercise is already answered
                                $exercise['solved'] = 1;
                        }
                        $exercises[] = $exercise;
                }

                ///////////////////////////////////////////////////////////////////////
                $assignment['exercises'] = $exercises;
                if ($p_action == 'exams') {
                        $assignment['remainingMinutes'] = getRemainingMinutes($assignment_id);

                        /* don't display negative remaining time */
                        if ($assignment['remainingMinutes'] <= 0)
                            $assignment['remainingMinutes'] = 0;
                }
                $assignments[] = $assignment;
        }

        $t_list->set_attribute('message', $message);
        $t_list->set_attribute('assignments', $assignments);
        $t_list->set_attribute('dialog', $dialog);
        $t_list->set_attribute('ip_address', $_SERVER['REMOTE_ADDR']);
        $t_list->set_attribute('hostname', gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $t_list->set_attribute('title', "Laufende $g_title_p");
        $t_list->set_attribute("running_status", $running_status);
        $t_list->set_attribute("running_exam_info", $running_exam_info);  // only set in running exams

        setlocale(LC_NUMERIC, NULL);
        echo $t_list->render();
        setlocale(LC_NUMERIC, 'C');
}

#####################################
#####################################
#                                                                       #
#               Lecturer Methods                        #
#                                                                       #
#####################################
#####################################


/**
 * SHEETS/EXAMS (blocks only for sheets)
 *
 * Stores assignment options such as block membership or selftest
 */

function assignment_options($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;

        $number_of_assignments = vips_get_int_param("number_of_assignments");

        for ($i = 0; $i < $number_of_assignments; ++$i) {

                $assignment_id  = vips_get_int_param("assignment_no_$i");
                $block_id               = vips_get_int_param("assign_to_block_$i");
                $selftest               = vips_get_int_param("selftest_for_$assignment_id");

                /**** selftest *****************************************/

                if ($selftest) $selftest = 1;
                $vips_db->executeUpdate("UPDATE $g_assignment_table
                                        SET     Selbsttest      = '$selftest'
                                        WHERE   ID                      = '$assignment_id'");


                /**** Bloecke *************************************************/

                if              ($block_id == "0") {}
                elseif  ($block_id == "-1") {                           //remove membership to block
                        $vips_db->executeUpdate("DELETE FROM vips_inBlock
                                                WHERE assignment_id = '$assignment_id'
                                                ");
                }
                else {                                                                          //assign to block
                        $result = $vips_db->executeQuery("SELECT * FROM vips_inBlock
                                                WHERE   assignment_id = '$assignment_id'
                                                ");

                        if ($result->getRecordCount() == 0) {
                                $vips_db->executeUpdate("INSERT INTO vips_inBlock
                                                                        SET block_id            = '$block_id',
                                                                                assignment_id   = '$assignment_id'
                                                                                ");
                        }
                        else {
                                $vips_db->executeUpdate("UPDATE vips_inBlock
                                                SET     block_id                = '$block_id'
                                                WHERE   assignment_id   = '$assignment_id'
                                                ");
                        }
                }
        }
        list_assignments($p_action);  //normal view
}



/**
 * SHEETS
 *
 * Adds a block name to the database so sheets can be assigned to this block
 *
 */

function add_block($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;

        $new_block_name = vips_get_string_param('new_block_name', true);

        //TODO auf doppelte Namen ueberpruefen!!!

        $vips_db->executeUpdate("INSERT INTO vips_block
                                SET
                                        Blockname               = '$new_block_name',
                                        Kurs                    = '$g_outlet->courseID'
                                ");

        list_assignments($p_action);
}

/**
 * EXAMS/SHEETS
 *
 * If an assignment hasn't started yet this function sets the start time to NOW
 * so that it's running
 *
 */

function start_assignment($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;

        $assignment_id = vips_get_int_param('assignment_id');
        $date          = date("Y-m-d H:i:s");

        // set new Startzeitpunkt in database
        $vips_db->executeUpdate("UPDATE $g_assignment_table
                    SET Beginn = '$date', istGesperrt = 'n'
                    WHERE ID = '$assignment_id'");

        // delete Startzeitpunkte for exam from database
        $vips_db->executeUpdate("DELETE FROM vips_aufgaben_zeit
                    WHERE Kurs = '".$g_outlet->courseID."'
                    AND $g_assignment_table = '$assignment_id'");

        list_assignments($p_action);
}


/**
 * EXAMS/SHEETS
 *
 * Stops/continues an assignment (no change of start/end time but temporary closure)
 *
 */
function stopgo_assignment ($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;

        $assignment_id                  = vips_get_int_param('assignment_id');

        $sql = "UPDATE $g_assignment_table
                        SET istGesperrt = (CASE WHEN istGesperrt = 'y' THEN 'n' ELSE 'y' END)
                        WHERE ID = '$assignment_id'";
        $vips_db->executeUpdate($sql);

        list_assignments($p_action);
}


/**
 * EXAMS/SHEETS
 *
 * TODO: Assignment don't have a position right now (they are listed according to their start date),
 * but it might be nice to have a order which the lecturer can manipulate
 */
function move_assignment($p_action) {
        /*????*/
}

/**
 * EXAMS/SHEETS
 *
 * Deletes an assignment from the course (and block if applicable). All exercises are retained.
 */
function delete_assignment($p_action) {
        global $vips_db,
               $g_outlet,
               $g_inAssignment_table,
               $g_assignment_table;

        $assignment_id  = vips_get_int_param('assignment_id');

        if (vips_get_submit_param('cancel'))
        {
                list_assignments($p_action);
        }
        else if (vips_get_submit_param('delete_assignment_confirmed'))
        {
                $vips_db->executeUpdate("DELETE FROM $g_inAssignment_table
                                        WHERE $g_assignment_table = '$assignment_id'");

                $vips_db->executeUpdate("DELETE FROM vips_inBlock
                                        WHERE assignment_id = '$assignment_id'");
                //Position: ???
                //$vips_db->executeUpdate("UPDATE " . $g_sheetType . " SET Position=Position-1 WHERE Kurs='".$m_range_id."' AND Position>'" . $p_exercise_sheet_position."'");

                $vips_db->executeUpdate("DELETE FROM $g_assignment_table WHERE ID = '$assignment_id'");
                $message['mode']        = 'msg';
                $message['message']     = 'Das Aufgabenblatt wurde gelscht.';
                $_POST['message']       = $message;
                list_assignments($p_action);
        }
        else
        {
                $assignment = get_assignment_data($assignment_id);
                $_POST['delete_assignment_id']   = $assignment_id;
                $_POST['delete_assignment_name'] = $assignment['Name'];
                $_POST['dialog'] = 'delete_assignment_confirm';
                list_assignments($p_action);
        }
}



/**
 * SHEETS/EXAMS
 *
 * Takes an exercise off an assignment. If $reallydelete is set, the exercise is also completely
 * removed from the database.
 */
function delete_exercise($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;

        $exercise_id         = vips_get_int_param('exercise_id');
        $assignment_id       = vips_get_int_param('assignment_id');
        $position            = vips_get_int_param('exercise_position');
        $number_of_exercises = vips_get_int_param('number_of_exercises');
        $reallydelete        = vips_get_option_param('reallydelete');

        // TODO replace this with proper access validation and move output to template
        // The access to this function is forbidden if the assignment has expired! //

        /* disabled for now, see http://plugins.studip.de/trac/ticket/32
        $query = "SELECT `Ende` FROM $g_assignment_table WHERE `ID` = $assignment_id";
        $result = $vips_db->executeQuery($query);
        $result->next();
        if (date("Y-m-d H:i:s") > $result->get("Ende"))
        {
                echo _("Das bungsblatt ist beendet. Aufgaben in abgelaufenen bungsblttern drfen nicht mehr gelscht werden!");
                return;
        }
        */

        // if exercise is in a rnd_exercise: delete it from rnd_exercise and leave
        // assignment sheets and `vips_aufgabe` untouched
        if (vips_get_int_param("in_rnd_exercise"))
        {
                require_once("exercises/rnd_exercise.php");
                $rnd0_exercise = new rnd_exercise();

                $rnd_exercise_id = vips_get_int_param("rnd_exercise_id");

                // fetch rnd_exercise XML
                $query = "SELECT `Aufgabe`
                        FROM `vips_aufgabe`
                        WHERE `ID` = '".$rnd_exercise_id."'
                        ";
                $result = $vips_db->executeQuery($query);
                $result->next();
                $xml = $result->get("Aufgabe");

                $xml = $rnd0_exercise->delete_subexercise($xml, $exercise_id);

                $query = "UPDATE `vips_aufgabe`
                        SET `Aufgabe` = '".$xml."'
                        WHERE `ID` = '".$rnd_exercise_id."'
                        ";
                $vips_db->executeUpdate($query);

                $_POST["assignment_id"]     = $assignment_id;
                $_POST["exercise_id"]       = $rnd_exercise_id;
                $_POST["exercise_type"]     = "rnd_exercise";
                $_POST['exercise_position'] = vips_get_int_param('rnd_exercise_position');
                // jump to editing the rnd_exercise itself, not any subexercise
                unset($_GET['in_rnd_exercise']);
                edit_exercise($p_action);
                return;
        }

        // remove exercise
        $query1 = "DELETE FROM $g_inAssignment_table
                    WHERE vips_aufgabe = '$exercise_id'
                    AND $g_assignment_table = '$assignment_id'
                    AND Position = '$position'";

        // renumber following exercises
        $query2 = "UPDATE $g_inAssignment_table
                    SET Position = Position - 1
                    WHERE $g_assignment_table = '$assignment_id'
                    AND Position > '$position'";

        $affected_rows = $vips_db->executeUpdate($query1);

        if ($affected_rows) {
            $vips_db->executeUpdate($query2);
        }

        if ($reallydelete) {
            $query3 = "DELETE FROM vips_aufgabe WHERE ID = '$exercise_id'";
            $vips_db->executeUpdate($query3);
        }

        $_POST['assignment_id'] = $assignment_id;
        $_POST['number_of_exercises'] = $number_of_exercises-1;
        edit_assignment($p_action);
}

/**
 * List all exercises in the given assignment (AJAX).
 */
function list_exercises_ajax ($action, $assignment_id)
{
    global $vips_db, $g_outlet, $vipsTemplateFactory,
           $g_assignment_table, $g_inAssignment_table;

    #############################################
    # get existing exercises in this assignment #
    #############################################

    $query = "SELECT vips_aufgabe.*,
                  vips_aufgaben_typen.Typenname,
                  $g_inAssignment_table.Position,
                  $g_inAssignment_table.Punkte,
                  $g_inAssignment_table.Bewertet
              FROM vips_aufgabe,
                  vips_aufgaben_typen,
                  $g_inAssignment_table
              WHERE $g_assignment_table = '$assignment_id'
                  AND vips_aufgabe = vips_aufgabe.ID
                  AND vips_aufgaben_typen.URI = vips_aufgabe.URI
              ORDER BY $g_inAssignment_table.Position";

    $result = $vips_db->executeQuery($query);
    $exercises = array();

    while ($result->next()) {
        $entry = $result->getRow();
        $entry['Punkte'] = $result->getFloat('Punkte');
        $exercises[] = $entry;
    }

    $template = $vipsTemplateFactory->open('list_exercises');

    $template->set_attributes(array(
        'picturepath'            => $g_outlet->picturepath,
        'action'                 => $action,
        'assignment_id'          => $assignment_id,
        'exercises'              => $exercises
    ));

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();
    header('Content-Type: text/html; charset=ISO-8859-1');

    setlocale(LC_NUMERIC, NULL);
    echo $template->render();
    setlocale(LC_NUMERIC, 'C');

    ob_start('discard_buffer');
}

/**
 * Takes an exercise off an assignment. If $reallydelete is set, the exercise is also completely
 * removed from the database (AJAX).
 */
function delete_exercise_ajax ($action)
{
    global $vips_db, $g_inAssignment_table, $g_assignment_table;

    $exercise_id         = vips_get_int_param('exercise_id');
    $assignment_id       = vips_get_int_param('assignment_id');
    $position            = vips_get_int_param('exercise_position');
    $reallydelete        = vips_get_option_param('reallydelete');

    // remove exercise
    $query1 = "DELETE FROM $g_inAssignment_table
                WHERE vips_aufgabe = '$exercise_id'
                AND $g_assignment_table = '$assignment_id'
                AND Position = '$position'";

    // renumber following exercises
    $query2 = "UPDATE $g_inAssignment_table
                SET Position = Position - 1
                WHERE $g_assignment_table = '$assignment_id'
                AND Position > '$position'";

    $affected_rows = $vips_db->executeUpdate($query1);

    if ($affected_rows) {
        $vips_db->executeUpdate($query2);
    }

    if ($reallydelete) {
        $query3 = "DELETE FROM vips_aufgabe WHERE ID = '$exercise_id'";
        $vips_db->executeUpdate($query3);
    }

    list_exercises_ajax($action, $assignment_id);
}

/**
 * SHEETS/EXAMS
 *
 * Changes position of an exercise within an assignment
 *
 */
function move_exercise($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;
        $direction              = vips_get_option_param('direction');
        $exercise_id            = vips_get_int_param('exercise_id');
        $assignment_id          = vips_get_int_param('assignment_id');
        $position               = vips_get_int_param('exercise_position');
        $number_of_exercises    = vips_get_int_param('number_of_exercises');

        // TODO replace this with proper access validation and move output to template
        // The access to this function is forbidden if the assignment has expired! //

        /* disabled for now, see http://plugins.studip.de/trac/ticket/32
        $query = "SELECT `Ende` FROM $g_assignment_table WHERE `ID` = $assignment_id";
        $result = $vips_db->executeQuery($query);
        $result->next();
        if (date("Y-m-d H:i:s") > $result->get("Ende"))
        {
                echo _("Das bungsblatt ist beendet. Die Reihenfolge der Aufgaben in abgelaufenen bungsblttern darf nicht mehr verndert werden!");
                return;
        }
        */

        if ($direction == 'down')
            $position_target = $position - 1;
        else
            $position_target = $position + 1;


        // move selected
        $query1 = "UPDATE $g_inAssignment_table
                    SET Position = '$position_target'
                    WHERE vips_aufgabe = '$exercise_id'
                    AND Position = '$position'
                    AND $g_assignment_table = $assignment_id";

        // move other exercises
        $query2 = "UPDATE $g_inAssignment_table
                    SET Position = '$position'
                    WHERE vips_aufgabe != '$exercise_id'
                    AND Position = '$position_target'
                    AND $g_assignment_table = $assignment_id";

        $affected_rows = $vips_db->executeUpdate($query1);

        if ($affected_rows) {
            $vips_db->executeUpdate($query2);
        }

        $_POST['assignment_id'] = $assignment_id;
        $_POST['number_of_exercises'] = $number_of_exercises;
        edit_assignment($p_action);
}

/**
 * Reorder exercise positions within an assignment (AJAX).
 */
function move_exercise_ajax ($action)
{
    global $vips_db, $g_inAssignment_table;

    $assignment_id = vips_get_int_param('assignment_id');
    $list          = vips_get_int_array_param('list');

    /* renumber all exercises in current assignment */
    for ($i = 0; $i < count($list); ++$i)
    {
        $sql = "UPDATE $g_inAssignment_table SET Position = ".($i+1)." WHERE vips_aufgabe = '".$list[$i]."'";
        $vips_db->executeUpdate($sql);
    }

    list_exercises_ajax($action, $assignment_id);
}

/**
 * Return list of semesters with assignments accessible to teachers
 * in the current course. This includes assignments from other courses.
 */
function get_accessible_semesters ()
{
    global $vips_db, $g_outlet, $g_assignment_table;

    $sql = "SELECT DISTINCT
                semester_data.name,
                semester_data.beginn
            FROM
                semester_data,
                $g_assignment_table,
                seminare,
                seminar_user,
                seminar_user AS lecturer
            WHERE
                $g_assignment_table.Kurs = seminare.Seminar_id AND
                seminare.start_time = semester_data.beginn AND
                seminar_user.Seminar_id = seminare.Seminar_id AND
                seminar_user.status = 'dozent' AND
                seminar_user.user_id = lecturer.user_id AND
                lecturer.Seminar_id = '$g_outlet->courseID' AND
                lecturer.status = 'dozent'
            ORDER BY
                beginn DESC";

    $result = $vips_db->executeQuery($sql);
    while ($result->next())
    {
        $semesters[] = array('name'   => $result->getString('name'),
                             'beginn' => $result->getInt('beginn'));
    }

    return $semesters;
}

/**
 * Return data of the semester starting at the given time.
 */
function get_semester_data ($start_time)
{
    global $vips_db;

    $sql = "SELECT name, beginn FROM semester_data WHERE beginn = '$start_time'";

    $result = $vips_db->executeQuery($sql);
    $result->next();
    $semester = array('name'   => $result->getString('name'),
                      'beginn' => $result->getInt('beginn'));

    return $semester;
}

/**
 * Return list of courses with assignments accessible to teachers
 * in the current course. This includes assignments from other courses.
 */
function get_accessible_courses ($start_time)
{
    global $vips_db, $g_outlet, $g_assignment_table;

    $sql = "SELECT DISTINCT
                $g_assignment_table.*
            FROM
                $g_assignment_table,
                seminare,
                seminar_user,
                seminar_user AS lecturer
            WHERE
                $g_assignment_table.Kurs = seminare.Seminar_id AND
                seminare.start_time = '$start_time' AND
                seminar_user.Seminar_id = seminare.Seminar_id AND
                seminar_user.status = 'dozent' AND
                seminar_user.user_id = lecturer.user_id AND
                lecturer.Seminar_id = '$g_outlet->courseID' AND
                lecturer.status = 'dozent'
            ORDER BY
                Beginn, Position";

    $result = $vips_db->executeQuery($sql);
    while ($result->next())
    {
        $course = $result->getString('Kurs');
        $header = html_entity_decode(getHeaderLine($course), ENT_QUOTES);
        $courses[$course]['name'] = $header;
        $courses[$course]['tests'][] = $result->getRow();
    }

    return $courses;
}

/**
 * Return data of the specified assignment.
 */
function get_assignment_data ($test_id)
{
    global $vips_db, $g_assignment_table;

    $sql = "SELECT * FROM $g_assignment_table WHERE ID = '$test_id'";

    $result = $vips_db->executeQuery($sql);
    $result->next();
    $test = $result->getRow();

    return $test;
}

/**
 * Return list of exercises in the specified assignment.
 */
function get_exercises ($test_id)
{
    global $vips_db, $g_assignment_table, $g_inAssignment_table;

    $sql = "SELECT
                vips_aufgabe.*,
                $g_inAssignment_table.Position
            FROM
                vips_aufgabe,
                $g_inAssignment_table
            WHERE
                vips_aufgabe.ID = $g_inAssignment_table.vips_aufgabe AND
                $g_inAssignment_table.$g_assignment_table = '$test_id'
            ORDER BY
                Position";

    $result = $vips_db->executeQuery($sql);
    $exercises = array();

    while ($result->next())
    {
        $exercises[] = $result->getRow();
    }

    return $exercises;
}

/**
 * Return list of exercises matching the search string from all semesters
 * with assignments accessible to teachers in the current course. This
 * includes assignments from other courses.
 */
function get_matching_exercises ($search)
{
    global $vips_db, $g_outlet, $g_assignment_table, $g_inAssignment_table;

    $sql = "SELECT DISTINCT
                semester_data.name AS sem_name,
                semester_data.beginn,
                seminare.Name AS course_name,
                $g_assignment_table.ID as test_id,
                $g_assignment_table.Name as test_name,
                $g_inAssignment_table.Position,
                vips_aufgabe.*
            FROM
                semester_data,
                $g_assignment_table,
                seminare,
                seminar_user,
                seminar_user AS lecturer,
                $g_inAssignment_table,
                vips_aufgabe
            WHERE
                $g_assignment_table.Kurs = seminare.Seminar_id AND
                seminare.start_time = semester_data.beginn AND
                seminar_user.Seminar_id = seminare.Seminar_id AND
                seminar_user.status = 'dozent' AND
                seminar_user.user_id = lecturer.user_id AND
                lecturer.Seminar_id = '$g_outlet->courseID' AND
                lecturer.status = 'dozent' AND
                $g_inAssignment_table.$g_assignment_table = $g_assignment_table.ID AND
                vips_aufgabe.ID = $g_inAssignment_table.vips_aufgabe AND
                vips_aufgabe.aufgabe LIKE '%".htmlspecialchars($search)."%'
            ORDER BY
                beginn DESC, course_name, test_name, Name";

    $result = $vips_db->executeQuery($sql);
    while ($result->next())
    {
        $exercise = $result->getRow();
        // NOTE: strip_tags($exercise['Aufgabe']) removes CDATA sections
        $exercise_xml = domxml_open_mem(utf8_encode($exercise['Aufgabe']));
        $exercise_text = utf8_decode($exercise_xml->get_content());

        // TODO use stripos on PHP 5
        if (strpos(strtolower($exercise_text), strtolower($search)) !== false) {
            $exercises[] = $exercise;
        }
    }

    return $exercises;
}

/**
 * Pick a semester for copying an exercise (AJAX).
 */
function pick_semester_ajax ($action)
{
    global $g_outlet, $vipsPlugin, $vipsTemplateFactory;

    $assignment_id = vips_get_int_param('assignment_id');
    $start_time    = vips_get_int_param('start_time');
    $search        = vips_get_string_param('search');
    $close         = vips_get_int_param('close');

    $template = $vipsTemplateFactory->open('pick_semester');
    $template->set_attributes(array(
        'picturepath'           => $g_outlet->picturepath,
        'vips_imagepath'        => $vipsPlugin->getPluginURL().'/images/',
        'action'                => $action,
        'assignment_id'         => $assignment_id,
        'search'                => $search
    ));

    // get semester, course, assignment and exercise data for copying.
    // available are all assignments that all lecturers in this course
    // have access to (including other courses)

    $template->set_attribute('semester', get_semester_data($start_time));

    if (empty($close))
    {
        $template->set_attribute('start_time', $start_time);
        $template->set_attribute('courses', get_accessible_courses($start_time));
    }

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();
    header('Content-Type: text/html; charset=ISO-8859-1');

    echo $template->render();

    ob_start('discard_buffer');
}

/**
 * Pick a test for copying an exercise (AJAX).
 */
function pick_test_ajax ($action)
{
    global $g_outlet, $vipsPlugin, $vipsTemplateFactory;

    $assignment_id = vips_get_int_param('assignment_id');
    $start_time    = vips_get_int_param('start_time');
    $test_id       = vips_get_int_param('test_id');
    $search        = vips_get_string_param('search');
    $close         = vips_get_int_param('close');

    $template = $vipsTemplateFactory->open('pick_test');
    $template->set_attributes(array(
        'picturepath'           => $g_outlet->picturepath,
        'vips_imagepath'        => $vipsPlugin->getPluginURL().'/images/',
        'action'                => $action,
        'assignment_id'         => $assignment_id,
        'start_time'            => $start_time,
        'search'                => $search
    ));

    // get semester, course, assignment and exercise data for copying.
    // available are all assignments that all lecturers in this course
    // have access to (including other courses)

    $template->set_attribute('semester', get_semester_data($start_time));
    $template->set_attribute('test', get_assignment_data($test_id));

    if (empty($close))
    {
        $template->set_attribute('test_id', $test_id);
        $template->set_attribute('all_exercises', get_exercises($test_id));
    }

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();
    header('Content-Type: text/html; charset=ISO-8859-1');

    echo $template->render();

    ob_start('discard_buffer');
}

/**
 * SHEETS/EXAMS
 *
 * Displays the form for editing an exercise.
 *
 * Is called when editing an existing exercise or creating a new exercise.
 *
 *
 */
function edit_exercise($p_action, $p_copy = false)
{
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table,
        $vipsTemplateFactory,
        $vipsPlugin;

    $assignment_id         = vips_get_int_param('assignment_id');
    $exercise_id           = vips_get_int_param('exercise_id');
    $exercise_position     = vips_get_int_param('exercise_position');

    $dialog                = vips_get_string_array_param('dialog');

    // Determines whether the newly generated exercise will be linked to the assignment or to the calling random exercise.
    $in_rnd_exercise       = vips_get_int_param("in_rnd_exercise");
    $rnd_exercise_id       = vips_get_int_param("rnd_exercise_id");
    $rnd_exercise_name     = vips_get_string_param("rnd_exercise_name");
    $rnd_exercise_position = vips_get_int_param("rnd_exercise_position");

    // TODO replace this with proper access validation and move output to template
    // The access to this function is forbidden if the assignment has expired! //

    /* disabled for now, see http://plugins.studip.de/trac/ticket/32
    $query = "SELECT `Ende` FROM $g_assignment_table WHERE `ID` = $assignment_id";
    $result = $vips_db->executeQuery($query);
    $result->next();
    if (date("Y-m-d H:i:s") > $result->get("Ende"))
    {
        echo _("Das bungsblatt ist beendet. Aufgaben in abgelaufenen bungsblttern drfen nicht mehr bearbeitet werden!");
        return;
    }
    */

    // get mc_auswertung
    $result = $vips_db->executeQuery("SELECT mc_auswertung FROM $g_assignment_table WHERE ID = '$assignment_id'");
    $result->next();
    $assignment = $result->getRow();
    $mc_auswertung = $assignment['mc_auswertung'];

    // fetch all exercises in the assignment
    $result = $vips_db->executeQuery(
                    "SELECT vips_aufgabe.*
                      FROM  vips_aufgabe, $g_inAssignment_table
                      WHERE $g_inAssignment_table.$g_assignment_table = '$assignment_id'
                      AND vips_aufgabe = vips_aufgabe.ID
                      ");
    $number_of_exercises = $result->getRecordCount();

    // fetch all existing exercise types
    $sql = "SELECT `Typenname`, `URI` FROM `vips_aufgaben_typen`";
    $result = $vips_db->executeQuery($sql);
    while ($result->next()) {
        $uri = $result->get('URI');
        $name = $result->get('Typenname');
        $all_exercise_types[$uri] = $name;
    }
    // The array $all_exercise_types will be passed to the footer template
    // later.

    // create new exercise
    if (vips_get_int_param('new_exercise')) {
        $exercise_type      = vips_get_option_param('new_exercise_type');
        $exercise_typenname = $all_exercise_types[$exercise_type];

        include_once("exercises/$exercise_type.php");
        $o_exercise = new $exercise_type();
        $o_exercise->setDefaultValues();

        /*******  default values for exercises *********/
        $exercise_id = '';  // necessary
    }
    else // edit already existing exercise
    {
        if ($p_copy || $in_rnd_exercise) {  // edit copy or subexercise
            // fetch exercise info using exercise id
            $query = "SELECT
                    vips_aufgabe.*,
                    vips_aufgaben_typen.Typenname
                FROM
                    vips_aufgabe,
                    vips_aufgaben_typen
                WHERE
                    vips_aufgabe.ID = $exercise_id
                    AND vips_aufgaben_typen.URI = vips_aufgabe.URI
                ";
        }
        else {
            // assure valid exercise position (to prevent SQL failure)
            $exercise_position = min(max(1, $exercise_position), $number_of_exercises);

            // fetch exercise info using assignment and position
            $query = "SELECT
                    vips_aufgabe.*,
                    vips_aufgaben_typen.Typenname
                FROM
                    vips_aufgabe,
                    $g_inAssignment_table,
                    vips_aufgaben_typen
                WHERE
                    $g_inAssignment_table.$g_assignment_table = $assignment_id
                    AND $g_inAssignment_table.Position = $exercise_position
                    AND $g_inAssignment_table.vips_aufgabe = vips_aufgabe.ID
                    AND vips_aufgaben_typen.URI = vips_aufgabe.URI
                ";
        }

        $result = $vips_db->executeQuery($query);
        $result->next();
        $entry = $result->getRow();

        $exercise_id        = $result->getInt('ID');
        $exercise_name      = $entry['Name'];
        $xml                = $entry['Aufgabe'];
        $exercise_type      = $entry['URI'];
        $exercise_typenname = $entry['Typenname'];



        include_once("exercises/$exercise_type.php");
        $o_exercise = new $exercise_type($xml);
        $o_exercise->setEvaluationMode($mc_auswertung);

        $exercise_question = $o_exercise->getQuestion('edit');  // 'edit' param needed in cloze exercise
        $exercise_hint     = $o_exercise->getHint();
        $exercise_comment  = $o_exercise->getCommentAllowed();
    }

    /**************************************************************************/
    $t_header = $vipsTemplateFactory->open('exercises/edit_exercise_header');
    $t_main   = $o_exercise->getEditTemplate();
    $t_footer = $vipsTemplateFactory->open('exercises/edit_exercise_footer');
    /**************************************************************************/
    //if $exercise_id is empty in the template, the system knows that it has
    //to create a new exercise:
    if ($p_copy) {
        $exercise_id = '';  // to create a new instance of an existing exercise
    }

    $t_header->set_attributes(array(
        'picturepath'           => $g_outlet->picturepath,
        'dialog'                => $dialog,
        'action'                => $p_action,
        'assignment_id'         => $assignment_id,
        'exercise_title'        => $exercise_typenname,
        'exercise_id'           => $exercise_id,
        'exercise_position'     => $exercise_position,
        'exercise_name'         => $exercise_name,
        'exercise_type'         => $exercise_type,
        'exercise_question'     => $exercise_question,
        'number_of_exercises'   => $number_of_exercises,
        "in_rnd_exercise"       => $in_rnd_exercise,
        "rnd_exercise_id"       => $rnd_exercise_id,
        "rnd_exercise_name"     => $rnd_exercise_name,
        'rnd_exercise_position' => $rnd_exercise_position));
    /**************************************************************************/
    $t_footer->set_attributes(array(
        'exercise_hint'         => $exercise_hint,
        'exercise_comment'      => $exercise_comment,
        'picturepath'           => $g_outlet->picturepath,
        'action'                => $p_action,
        'assignment_id'         => $assignment_id,
        'in_rnd_exercise'       => $in_rnd_exercise,
        'rnd_exercise_id'       => $rnd_exercise_id,
        'rnd_exercise_name'     => $rnd_exercise_name,
        'rnd_exercise_position' => $rnd_exercise_position,
        'exercise_id'           => $exercise_id,
        'exercise_type'         => $exercise_type,
        'exercise_position'     => $exercise_position,
        'all_exercise_types'    => $all_exercise_types,
        'number_of_exercises'   => $number_of_exercises));
    /**************************************************************************/

    echo $t_header->render();
    echo $t_main->render();
    echo $t_footer->render();
}


/**
 * SHEETS/EXAMS
 *
 * Inserts/Updates an exercise into the database
 */

function store_exercise($p_action) {
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table;

    $assignment_id       = vips_get_int_param('assignment_id');
    $exercise_id         = vips_get_int_param('exercise_id');
    $exercise_type       = vips_get_option_param('exercise_type');
    $exercise_name       = vips_get_string_param('exercise_name', true);
    $exercise_position   = vips_get_int_param('exercise_position');

    // NOTE number_of_answers may contain comma separated values here
    $number_of_answers   = vips_get_string_param('number_of_answers');  //wrong if mc/mco-exercise number of answers has been changed!!!
    $number_of_exercises = vips_get_int_param('number_of_exercises');

    /* For random exercises: check if "create" button has been pressed and a */
    /* valid exercise type has been chosen for creation. */
    if ($exercise_type == "rnd_exercise" && vips_get_submit_param('create') && vips_get_option_param('exercise_types') != 'NULL')
    {
        $_POST["new_exercise"] = 1;  // flag to indicate that a new exercise is to be created
        $_POST["new_exercise_type"] = vips_get_option_param('exercise_types');  // from dropdown field

        // some additional parameters to steer that the new exercise is linked
        // to random exercise and not to assignment sheet
        $_POST["in_rnd_exercise"] = 1;  // flag: values 0,1
        $_POST["rnd_exercise_id"] = $exercise_id;  // TODO Ob das wohl klappt, wenn die rnd-Aufgabe noch keine ID hat, also ganz am Anfang vom Anlegen? Wohl nicht.... Mal was ausdenken.
        $_POST["rnd_exercise_name"] = $exercise_name;
        $_POST["rnd_exercise_position"] = $exercise_position;
    }
    else {
        $dialog['mode']         = 'msg';
        $dialog['message']      = 'Die Aufgabe wurde eingetragen';
    }

    if (!$exercise_id) {
        // find out max Position
        $result = $vips_db->executeQuery("
            SELECT MAX(Position) AS pos
            FROM $g_inAssignment_table
            WHERE $g_assignment_table = '$assignment_id'
            ");
        $result->next();
        $exercise_position = $result->getInt('pos') + 1;
    }

    if ($exercise_name == '') {
        $exercise_name = sprintf(_('Aufgabe %d'), $exercise_position);
    }

    /*** store exercise */

    require_once("exercises/$exercise_type.php");
    $o_exercise = new $exercise_type();  // exercise created from request parameters

    // all exercise types whose points are dependent on number of answers
    $update_points_exercise_types = array(
        'mc_exercise',
        'mco_exercise',
        'sc_exercise',
        'sco_exercise',
        'rh_exercise',
        'cloze_exercise');

    ////////////////////////
    // fetch exercise XML //
    ////////////////////////

    $xml = $o_exercise->getExerciseConstruct();  //create xml with question, answers...

    /*** store already existing exercise */
    if ($exercise_id) {
        // Update name and XML for this exercise.
        $query = "UPDATE
                vips_aufgabe
            SET
                Name = '$exercise_name',
                Aufgabe = '$xml'
            WHERE
                ID = '$exercise_id'
            ";
        $vips_db->executeUpdate($query);

        // update maximum points
        if (in_array($exercise_type, $update_points_exercise_types)) {
            $exercise_points = $o_exercise->getMaxPoints();
            // TODO marschro Was passiert hier bei Aufgaben, die nur in einer rnd_exercise stecken und nicht im Aufgabenblatt?
            $vips_db->executeUpdate("
                UPDATE $g_inAssignment_table
                SET Punkte = '$exercise_points'
                WHERE vips_aufgabe = '$exercise_id'
                    AND $g_assignment_table = '$assignment_id'
                ");
        }
    }

    /*** store new exercise */
    else {
        // Store exercise in database.
        $query = "
            INSERT INTO vips_aufgabe
            SET Name = '$exercise_name',
                Aufgabe = '$xml',
                URI = '$exercise_type',
                Userid = '$g_outlet->userID'
            ";
        $vips_db->executeUpdate($query);
        $vips_idgen = $vips_db->getIdGenerator();
        $exercise_id = $vips_idgen->getId();



        // Add new exercise as subexercise to random exercise.
        if (vips_get_int_param('in_rnd_exercise')) {
            require_once("exercises/rnd_exercise.php");
            $rnd0_exercise = new rnd_exercise();

            $rnd_exercise_id = vips_get_int_param('rnd_exercise_id');

            // Fetch rnd_exercise-XML from database.
            $query = "
                SELECT `Aufgabe`
                FROM `vips_aufgabe`
                WHERE ID = '".$rnd_exercise_id."'
                ";
            $result = $vips_db->executeQuery($query);
            $result->next();
            $rnd_xml = $result->get("Aufgabe");

            $rnd_xml = $rnd0_exercise->appendNewSubexercise($rnd_xml, $exercise_id);

            // Store extended XML for rnd_exercise.
            $query = "
                UPDATE `vips_aufgabe`
                SET Aufgabe = '$rnd_xml'
                WHERE ID = '".$rnd_exercise_id."'
                    AND URI = 'rnd_exercise'
                    AND Userid = '".$g_outlet->userID."'
                ";
            $vips_db->executeUpdate($query);
        }

        // Link new exercise to the assignment.
        else {
            if (in_array($exercise_type, $update_points_exercise_types)) {
                $exercise_points = $o_exercise->getMaxPoints();
            }

            $query2 = "
                INSERT INTO $g_inAssignment_table
                SET vips_aufgabe = '$exercise_id',
                    $g_assignment_table = '$assignment_id',
                    Punkte = '$exercise_points',
                    Bewertet = 't',
                    Position = '$exercise_position'
                ";
            $vips_db->executeUpdate($query2);
        }
    }

    $_POST['assignment_id']       = $assignment_id;
    $_POST['exercise_id']         = $exercise_id;  // necessary if target is for edit_exercise
    $_POST['exercise_type']       = $exercise_type;
    $_POST['exercise_position']   = $exercise_position;
    $_POST['number_of_exercises'] = $number_of_exercises;
    $_POST['number_of_answers']   = $number_of_answers;
    $_POST['dialog']              = $dialog;
    $_POST['in_rnd_exercise']     = vips_get_int_param('in_rnd_exercise');

    edit_exercise($p_action);  // show the same exercise
}

/**
 * Copy the selected exercises into this assignment.
 */
function copy_exercise ($action, $use_ajax)
{
    global $vips_db, $g_outlet, $g_inAssignment_table, $g_assignment_table;

    $assignment_id = vips_get_int_param('assignment_id');
    $exercise_ids  = vips_get_int_array_param('exercise_ids');

    // find out max Position
    $result = $vips_db->executeQuery(
            "SELECT MAX(Position) AS pos FROM $g_inAssignment_table".
            " WHERE $g_assignment_table = '$assignment_id'");
    $result->next();
    $exercise_position = $result->getInt('pos');

    $vips_idgen = $vips_db->getIdGenerator();

    foreach ($exercise_ids as $exercise_id)
    {
        ++$exercise_position;
        $vips_db->executeUpdate("INSERT INTO vips_aufgabe (Name, Aufgabe, URI, Userid)
                                     SELECT Name, Aufgabe, URI, '$g_outlet->userID'
                                     FROM vips_aufgabe WHERE ID = '$exercise_id'");
        $new_id = $vips_idgen->getId();

        $result = $vips_db->executeQuery("SELECT * FROM $g_inAssignment_table WHERE vips_aufgabe = '$exercise_id'");
        $result->next();
        $punkte = $result->getFloat('Punkte');

        $vips_db->executeUpdate("INSERT INTO $g_inAssignment_table (vips_aufgabe, $g_assignment_table, Punkte, Bewertet, Position)
                                 VALUES ($new_id, '$assignment_id', $punkte, 't', $exercise_position)");
    }

    if ($use_ajax) {
        list_exercises_ajax($action, $assignment_id);
    } else {
        edit_assignment($action);
    }
}

/**
 * Exports all exercises in this assignment in a plain text format.
 */
function export_assignment ($action)
{
    global $vips_db, $g_outlet, $g_inAssignment_table, $g_assignment_table, $g_title_s;

    $assignment_id = vips_get_int_param('assignment_id');

    if ($g_outlet->status == 'student')
    {
        $message['mode'] = 'error';
        $message['message'] = _('Zugriff nur fr Dozenten erlaubt.');
        $_POST['message'] = $message;

        return list_assignments_stud($action);
    }

    #######################
    # get assignment data #
    #######################

    $query = "SELECT * FROM $g_assignment_table WHERE ID = '$assignment_id'";
    $result = $vips_db->executeQuery($query);
    $result->next();

    $course_name      = html_entity_decode(getHeaderLine($g_outlet->courseID), ENT_QUOTES);
    $assignment_name  = $result->getString('Name');
    $assignment_start = $result->getString('Beginn');

    #####################
    # get all exercises #
    #####################

    $query = "SELECT vips_aufgabe.*
              FROM vips_aufgabe, $g_inAssignment_table
              WHERE $g_assignment_table = '$assignment_id'
                  AND vips_aufgabe = vips_aufgabe.ID
              ORDER BY Position";

    $result = $vips_db->executeQuery($query);

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();
    header('Content-Type: text/plain; charset=ISO-8859-1');
    header('Content-Disposition: attachment; filename="'.$assignment_name.'.txt"');
    header('Pragma: public');

    echo "# $course_name\n";
    echo "# $g_title_s: $assignment_name\n";
    echo "# Beginn: $assignment_start\n\n";

    while ($result->next())
    {
        $exercise_type      = $result->getString('URI');
        $exercise_xml       = $result->getString('Aufgabe');

        require_once("exercises/$exercise_type.php");

        $exercise = new $exercise_type($exercise_xml);
        echo $exercise->exportText();
        echo "\n";
    }

    ob_start('discard_buffer');
}

/**
 * SHEETS/EXAMS
 *
 * Stores the specification (Grunddaten) of an assignment
 * OR add new exercise, edit Punkte/Bewertung (basically everything that can be done on
 * page edit_exercise())
 */

function store_assignment($p_action) {
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table;

    $assignment_id          = vips_get_int_param('assignment_id');
    $assignment_name        = vips_get_string_param('assignment_name', true);
    $assignment_description = vips_get_string_param('assignment_description', true);
    $start_day              = vips_get_int_param('start_day');
    $start_month            = vips_get_int_param('start_month');
    $start_year             = vips_get_int_param('start_year');
    $start_h                = vips_get_int_param('start_h');
    $start_m                = vips_get_int_param('start_m');
    $end_day                = vips_get_int_param('end_day');
    $end_month              = vips_get_int_param('end_month');
    $end_year               = vips_get_int_param('end_year');
    $end_h                  = vips_get_int_param('end_h');
    $end_m                  = vips_get_int_param('end_m');

    $exam_length            = vips_get_int_param('exam_length');

    $number_of_exercises    = vips_get_int_param('number_of_exercises');
    $printable              = vips_get_int_param('printable');
    $ip_range               = vips_get_string_param('ip_range', true);
    $mc_auswertung          = vips_get_int_param('mc_auswertung');

    if (vips_get_submit_param('store'))
    {
        // find out max Position
        $result = $vips_db->executeQuery("SELECT MAX(Position) AS pos FROM $g_assignment_table WHERE Kurs = '$g_outlet->courseID'");
        $result->next();
        $position = $result->getInt('pos') + 1;     // TODO: use getRecordCount() here, position of assignment not yet implemented

        $start_unix_ts = mktime($start_h, $start_m, 0, $start_month, $start_day, $start_year); //$start = "$start_year-$start_month-$start_day $start_h:$start_m:00";
        $start         = date("Y-m-d H:i:s", $start_unix_ts);
        $end_unix_ts   = mktime($end_h, $end_m, 0, $end_month, $end_day, $end_year); //$end = "$end_year-$end_month-$end_day $end_h:$end_m:00";
        $end           = date("Y-m-d H:i:s", $end_unix_ts);
        $length        = ($end_unix_ts - $start_unix_ts) / 60;

        if ($length <= 0)  // start is *later* than end!
        {
            $end_unix_ts      = $start_unix_ts;
            $end              = $start;
            $message['mode']  = 'info';
            $message['message'] = _('Bitte berprfen Sie den Start- und den Endzeitpunkt!');
            $_POST['message'] = $message;
        }

        if ($p_action == 'exams')
        {
            $length = $exam_length;  // as specified above
        }

        /*** store basic data (Grunddaten) of assignment */
        if ($assignment_id) {
            // check whether the exam's start time has been moved
            $query = "SELECT Beginn FROM $g_assignment_table WHERE ID = '$assignment_id'";
            $result = $vips_db->executeQuery($query);
            $result->next();

            // delete start times for users which already have begun to solve the exam
            if ($result->get("Beginn") != $start && time() <= $start_unix_ts)
            {
                $query = "DELETE FROM vips_aufgaben_zeit
                    WHERE Kurs = '$g_outlet->courseID'
                    AND $g_assignment_table = '$assignment_id'";
                $vips_db->executeUpdate($query);
            }

            $query = "UPDATE $g_assignment_table
                SET Name = '$assignment_name',
                    Beginn = '$start',
                    Ende = '$end',
                    Dauer = '$length',
                    Thema = '$assignment_description',
                    Sichtbarkeit = '$printable',
                    IPBereich = '$ip_range',
                    mc_auswertung = '$mc_auswertung'
                WHERE
                    ID = '$assignment_id'";
            $vips_db->executeUpdate($query);
        }
        else
        {
            $query = "INSERT INTO $g_assignment_table
                SET Name = '$assignment_name',
                    Beginn = '$start',
                    Ende = '$end',
                    Kurs = '$g_outlet->courseID',
                    Dauer = '$length',
                    Thema = '$assignment_description',
                    Sichtbarkeit = '$printable',
                    IPBereich = '$ip_range',
                    Position = '$position',
                    mc_auswertung = '$mc_auswertung'";
            $vips_db->executeUpdate($query);
            $vips_idgen = $vips_db->getIdGenerator();
            $assignment_id = $vips_idgen->getId();
        }

        $points = vips_get_float_array_param('exercise_points_array');      // [1,3,5,4,3,3,3,3,...]
        $eval   = vips_get_option_array_param('exercise_eval_array');       // [f,t,t,f,t,f,t,f,...]
        for ($i = 0; $i < $number_of_exercises; $i++) {
            if (!$eval[$i]) $eval[$i] = 't';
            $points[$i] = round_to_half_point($points[$i]);

            $query = "UPDATE $g_inAssignment_table
                SET Punkte = '$points[$i]',
                    Bewertet = '$eval[$i]'
                WHERE $g_assignment_table='$assignment_id'
                    AND Position = ".($i+1);
            $vips_db->executeUpdate($query);
        }

        $_POST['assignment_id'] = $assignment_id;

        edit_assignment($p_action);  // show the same template from edit PLUS vv
    }
    else if (vips_get_submit_param('create_exercise'))
    {
        /*** create a new exercise */
        $_POST['new_exercise'] = 1;

        edit_exercise($p_action);
    }
    else if (vips_get_submit_param('copy_exercise'))
    {
        /*** copy existing exercise */
        copy_exercise($p_action, false);
    }
    else if (vips_get_submit_param('simulate_assignment'))
    {
        /*** display assignment as if for students */
        list_assignments_stud($p_action);
    }
    else if (vips_get_submit_param('export_assignment'))
    {
        /*** export assignment to text file */
        export_assignment($p_action);
    }
    else if (vips_get_submit_param('clear_search'))
    {
        /*** clear current search */
        $_POST['search'] = '';

        edit_assignment($p_action);
    }
    else /* no parameter name (JavaScript form submit) */
    {
        /*** search existing exercises */
        edit_assignment($p_action);
    }
}



/**
 * SHEETS/EXAMS
 *
 * Displays form to edit an existing assignment
 *
 */

function edit_assignment($p_action)
{
    global $vips_db,
        $g_outlet,
        $g_title_s,
        $g_title_p,
        $g_assignment_table,
        $g_inAssignment_table,
        $vipsPlugin,
        $vipsTemplateFactory;

    $assignment_id  = vips_get_int_param('assignment_id');
    $message        = vips_get_string_array_param('message');
    $sem_start_time = vips_get_int_param('start_time');
    $test_id        = vips_get_int_param('test_id');
    $search         = vips_get_string_param('search');

    #######################
    # get assignment data #
    #######################

    $query = "SELECT * FROM $g_assignment_table WHERE ID = '$assignment_id'";
    $result = $vips_db->executeQuery($query);
    $result->next();
    $entry = $result->getRow();

    list($start_date, $start_time) = split(" ", $entry['Beginn']);
    list($start_year, $start_month, $start_day) = split("-", $start_date);
    list($start_h, $start_m, $start_s) = split(":", $start_time);

    list($end_date, $end_time) = split(" ", $entry['Ende']);
    list($end_year, $end_month, $end_day) = split("-", $end_date);
    list($end_h, $end_m, $end_s) = split(":", $end_time);

    $assignment_id          = $entry['ID'];
    $assignment_name        = $entry['Name'];
    $assignment_description = $entry['Thema'];
    $length                 = $entry['Dauer'];
    $printable              = $entry['Sichtbarkeit'];
    $ip_range               = $entry['IPBereich'];
    $mc_auswertung          = $entry['mc_auswertung'];

    $t_main = $vipsTemplateFactory->open('edit_assignment');
    $t_main->set_attributes(array(
        'picturepath'            => $g_outlet->picturepath,
        'vips_imagepath'         => $vipsPlugin->getPluginURL().'/images/',
        'use_ajax'               => vips_use_ajax(),
        'g_title_s'              => $g_title_s,
        'g_title_p'              => $g_title_p,
        'action'                 => $p_action,
        'message'                => $message,
        'assignment_id'          => $assignment_id,
        'assignment_name'        => $assignment_name,
        'assignment_description' => $assignment_description,
        'end_year'               => $end_year,
        'end_month'              => $end_month,
        'end_day'                => $end_day,
        'end_h'                  => $end_h,
        'end_m'                  => $end_m,
        'start_year'             => $start_year,
        'start_month'            => $start_month,
        'start_day'              => $start_day,
        'start_h'                => $start_h,
        'start_m'                => $start_m,
        'printable'              => $printable,
        'ip_range'               => $ip_range,
        'mc_auswertung'          => $mc_auswertung,
        'exam_length'            => $length,
        'start_time'             => $sem_start_time,
        'test_id'                => $test_id,
        'search'                 => $search
    ));

    // After an assignment has expired, the lecturer is only allowed to adjust
    // points and to mark an exercise as not to be evaluated. He must neither
    // add, remove or edit exercises nor change the exercise order!
    /* disabled for now, see http://plugins.studip.de/trac/ticket/32
    $now = date("Y-m-d H:i:s");
    if ($now > $entry["Ende"]) {
        $running_status = "expired";
    } else if ($now < $entry["Beginn"]) {
        $running_status = "pending";
    } else {
        $running_status = "running";
    }
    $t_main->set_attribute("running_status", $running_status);
    */


    #############################################
    # get existing exercises in this assignment #
    #############################################

    $query = "SELECT vips_aufgabe.*,
            vips_aufgaben_typen.Typenname,
            $g_inAssignment_table.Position,
            $g_inAssignment_table.Punkte,
            $g_inAssignment_table.Bewertet
        FROM vips_aufgabe,
            vips_aufgaben_typen,
            $g_inAssignment_table
        WHERE $g_assignment_table = '$assignment_id'
            AND vips_aufgabe = vips_aufgabe.ID
            AND vips_aufgaben_typen.URI = vips_aufgabe.URI
        ORDER BY $g_inAssignment_table.Position
        ";
    $result = $vips_db->executeQuery($query);

    $exercises = array();
    while ($result->next()) {
        $entry = $result->getRow();
        $entry['Punkte'] = $result->getFloat('Punkte');
        $exercises[] = $entry;
    }
    $t_main->set_attribute('exercises', $exercises);

    $number_of_exercises = $result->getRecordCount();
    $t_main->set_attribute('number_of_exercises', $number_of_exercises);


    #######################################
    # get exercise types for new exercise #
    #######################################

    $query = "SELECT * FROM vips_aufgaben_typen";
    $result = $vips_db->executeQuery($query);
    $exercise_types = array();
    while ($result->next())
    {
        $entry = $result->getRow();
        $exercise_types[] = $entry;
    }
    $t_main->set_attribute('exercise_types', $exercise_types);

    // get semester, course, assignment and exercise data for copying.
    // available are all assignments that all lecturers in this course
    // have access to (including other courses)

    $t_main->set_attribute('semesters', get_accessible_semesters());

    if ($start_time != '') {
        $t_main->set_attribute('courses', get_accessible_courses($sem_start_time));
    }

    if ($test_id != '') {
        $t_main->set_attribute('all_exercises', get_exercises($test_id));
    }

    if ($search != '') {
        $t_main->set_attribute('search_result', get_matching_exercises($search));
    }

    setlocale(LC_NUMERIC, NULL);
    echo $t_main->render();
    setlocale(LC_NUMERIC, 'C');
}

/**
 * Show preview of an existing exercise (using print view for now).
 */
function preview_exercise ($action)
{
    vips_output_begin();

    $_POST['print'] = 1;
    show_exercise($action);

    vips_output_end();
}

/**
 * SHEETS/EXAMS
 *
 * Copies assignment completely, i.e. inserts new assignment plus copied exercises from old
 * assignment
 *
 */

function copy_assignment($p_action) {
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table;

    $assignment_id  = vips_get_int_param('assignment_id');
    $message = array();

    #################################
    # copy and paste new assignment #
    #################################

    // find out max Position
    $sql = "SELECT MAX(Position) AS pos FROM $g_assignment_table WHERE `Kurs` = '$g_outlet->courseID'";
    $result = $vips_db->executeQuery($sql);
    $result->next();
    $new_assignment_position = $result->getInt('pos') + 1;

    // Change name, course and position of copy
    $vips_db->executeUpdate("INSERT INTO $g_assignment_table
        (Name, Beginn, Ende, Kurs, Dauer, Selbsttest, Sichtbarkeit, IPBereich, Position, Thema, mc_auswertung)
        SELECT CONCAT('"._('Kopie von')." ', Name), Beginn, Ende, '$g_outlet->courseID', Dauer, Selbsttest, Sichtbarkeit, IPBereich, $new_assignment_position, Thema, mc_auswertung
        FROM $g_assignment_table
        WHERE ID = '$assignment_id'");
    $vips_idgen = $vips_db->getIdGenerator();
    $new_assignment_id = $vips_idgen->getId();

    ####################################################################
    # copy and paste all exercises associated with original assignment #
    # and copy and paste info from inAssignment table                  #
    ####################################################################

    $sql = "SELECT vips_aufgabe.*
        FROM vips_aufgabe,
            $g_inAssignment_table
        WHERE vips_aufgabe.ID = $g_inAssignment_table.vips_aufgabe
        AND $g_inAssignment_table.$g_assignment_table = '$assignment_id'";

    $result = $vips_db->executeQuery($sql);

    while ($result->next()) {
        $entry   = $result->getRow();
        $ID      = $entry['ID'];
        $Name    = $entry['Name'];
        $Aufgabe = $entry['Aufgabe'];
        $URI     = $entry['URI'];

        #############################################
        # change very old exercises into new format #
        #############################################

        if ($URI == 'yn_exercise')
        {
            $Aufgabe = str_replace('<Answer Correct="1"><![CDATA[]]></Answer>',
                '<Answer Correct="1">Ja</Answer><Answer Correct="0">Nein</Answer>',
                $Aufgabe);
            $Aufgabe = str_replace('<Answer Correct="0"><![CDATA[]]></Answer>',
                '<Answer Correct="0">Ja</Answer><Answer Correct="1">Nein</Answer>',
                $Aufgabe);
        }
        ////////////////////////////////////////////////////////////////////////////

        $Userid = $g_outlet->userID;

        $sql = "INSERT INTO vips_aufgabe
            SET Name    = '".addslashes($Name)."',
                Aufgabe = '".addslashes($Aufgabe)."',
                URI     = '$URI',
                Userid  = '$Userid'
            ";
        $vips_db->executeUpdate($sql);
        $vips_idgen = $vips_db->getIdGenerator();
        $new_exercise_id = $vips_idgen->getId();

        ####################################################
        # copy and paste information in inAssignment_table #
        ####################################################

        $sql = "SELECT * FROM $g_inAssignment_table
            WHERE $g_assignment_table = '$assignment_id'
            AND vips_aufgabe = '$ID'";

        $result2 = $vips_db->executeQuery($sql);

        if ($result2->next()) {
            $punkte   = $result2->getFloat('Punkte');
            $position = $result2->getInt('Position');
        }


        $sql = "INSERT INTO $g_inAssignment_table
            SET vips_aufgabe        = '$new_exercise_id',
                $g_assignment_table = '$new_assignment_id',
                Punkte              = '$punkte',
                Bewertet            = 't',
                Position            = '$position'";
        $vips_db->executeUpdate($sql);
    }
    $message['mode']    = 'msg';
    $message['message'] = 'Das Aufgabenblatt wurde kopiert.';
    $_POST['message']   = $message;
    list_assignments($p_action);
}



/**
 * SHEETS/EXAMS
 *
 * Displays form to create a new assignment
 * Uses the same template as edit_assignment
 *
 */
function new_assignment($p_action)
{
    global $vips_db, $g_outlet, $g_title_s, $g_title_p, $vipsTemplateFactory;

    $t_main = $vipsTemplateFactory->open('edit_assignment');

    // default values
    $t_main->set_attributes(array(
        'picturepath'            => $g_outlet->picturepath,
        'g_title_s'              => $g_title_s,
        'g_title_p'              => $g_title_p,
        'action'                 => $p_action,
        'assignment_name'        => $g_title_s,
        'assignment_description' => '',
        'end_year'               => date('Y'),
        'end_month'              => date('m'),
        'end_day'                => date('d'),
        'end_h'                  => date('H'),
        'end_m'                  => '00',
        'start_year'             => date('Y'),
        'start_month'            => date('m'),
        'start_day'              => date('d'),
        'start_h'                => date('H'),
        'start_m'                => '00',
        'printable'              => 0,
        'ip_range'               => '',
        'mc_auswertung'          => 0,
        'exam_length'            => 0,
        'number_of_exercises'    => 0
    ));

    setlocale(LC_NUMERIC, NULL);
    echo $t_main->render();
    setlocale(LC_NUMERIC, 'C');
}

/**
 * SHEETS/EXAMs
 *
 * Main page of sheets/exams.
 * Lists all the assignments (sheets or exams) in the course, grouped by "not yet started",
 * "running" and "finished".
 */
function list_assignments($p_action) {
        global $vips_db,
               $g_outlet,
               $g_title_p,
               $g_title_s,
               $g_assignment_table,
               $g_inAssignment_table,
               $vipsTemplateFactory;

        $dialog = vips_get_string_param('dialog');           //just in case (for header)
        $message = vips_get_string_array_param('message');         //just in case (for header)

        $delete_assignment_id   = vips_get_int_param('delete_assignment_id');
        $delete_assignment_name = vips_get_string_param('delete_assignment_name');

        $t_list_header  = $vipsTemplateFactory->open('list_assignments_header');

        $t_list_header->set_attributes(array(
                'title' => $g_title_p,
                'picturepath' => $g_outlet->picturepath));
        $t_list_header->set_attribute('dialog', $dialog);
        $t_list_header->set_attribute('message', $message);
        $t_list_header->set_attribute('assignment_id', $delete_assignment_id);
        $t_list_header->set_attribute('assignment_name', $delete_assignment_name);
        $t_list_header->set_attribute('action', $p_action);

        $t_list_footer  = $vipsTemplateFactory->open('list_assignments_footer');
        $t_list_footer->set_attribute('picturepath', $g_outlet->picturepath);
        $t_list_footer->set_attribute('action', $p_action);
        //$t_list_footer->set_attribute('status', $g_outlet->status);


        ##########################
        #       get blocks                       #
        ##########################

        $blocks = array();

        $result = $vips_db->executeQuery("SELECT DISTINCT * FROM vips_block
                                                        WHERE   Kurs = '$g_outlet->courseID'
                                                        ");

        while ($result->next()) {
                $entry = $result->getRow();
                $blocks[] = $entry;
        }

        ######################################
        #   get assignments in this course   #
        ######################################

        $query = "SELECT * FROM $g_assignment_table
                        WHERE
                                Kurs='$g_outlet->courseID'
                        ORDER BY
                                Beginn, Position";

        $result = $vips_db->executeQuery($query);

        $m_not_started_assignments = array();
        $m_running_assignments     = array();
        $m_finished_assignments    = array();
        $m_all_assignments         = array();

        $assignment_ids             = array();

        while ($result->next()) {
                $entry = $result->getRow();
                $assignment_id = $entry['ID'];
                $result2 = $vips_db->executeQuery("SELECT vips_block.*
                                                    FROM vips_inBlock,
                                                             vips_block
                                                    WHERE vips_inBlock.assignment_id= '$assignment_id'
                                                    AND   vips_inBlock.block_id     = vips_block.id");
                if ($result2->next()) {
                        $block = $result2->getRow();
                        $entry['block_id'] = $block['id'];
                }
                else $entry['block_id'] = "0";

                $m_start_timestamp = strtotime($entry['Beginn']);
                $m_end_timestamp   = strtotime($entry['Ende']);

                if (time() < $m_start_timestamp)
                    $m_not_started_assignments[] = $entry;
                else if ($m_end_timestamp < time())
                    $m_finished_assignments[]    = $entry;
                else
                    $m_running_assignments[]     = $entry;

         }

        ##############################################################
        #   get assignments made by lecturer (course-independent)    #
        ##############################################################

        $query = "SELECT DISTINCT $g_assignment_table.*,
                         seminare.Name AS course_name,
                         semester_data.name AS sem_name
                  FROM
                         seminare,
                         seminar_user,
                         seminar_user AS lecturer,
                         semester_data,
                         $g_assignment_table
                  WHERE
                         seminare.Seminar_id = seminar_user.Seminar_id
                     AND seminar_user.user_id = lecturer.user_id
                     AND seminar_user.status = 'dozent'
                     AND lecturer.Seminar_id = '$g_outlet->courseID'
                     AND lecturer.status = 'dozent'
                     AND semester_data.beginn = seminare.start_time
                     AND $g_assignment_table.Kurs = seminare.Seminar_id
                  ORDER BY
                         semester_data.beginn DESC, course_name,
                         $g_assignment_table.Name";
        $result = $vips_db->executeQuery($query);

        while ($result->next()) {
                $m_all_assignments[] = $result->getRow();
        }

        ##################
        #        Templates       #
        ##################

        $t_list1                = $vipsTemplateFactory->open('list_assignments_list');
        $t_list2                = $vipsTemplateFactory->open('list_assignments_list');
        $t_list3                = $vipsTemplateFactory->open('list_assignments_list');

        $count_not_started      = count($m_not_started_assignments);
        $count_running          = count($m_running_assignments);
        $count_finished         = count($m_finished_assignments);

        if ($count_not_started) {
                $t_list1->set_attribute('picturepath', $g_outlet->picturepath);
                $t_list1->set_attribute('action',$p_action);
                $t_list1->set_attribute('assignments', $m_not_started_assignments);
                $t_list1->set_attribute('title', "Noch nicht gestartete $g_title_p");
                $t_list1->set_attribute('status_assignment', 'not started');
                $t_list1->set_attribute('blocks', $blocks);
                $t_list1->set_attribute('number_of_assignments', $count_not_started);
        }
        if ($count_running) {
                $t_list2->set_attribute('picturepath', $g_outlet->picturepath);
                $t_list2->set_attribute('action',$p_action);
                $t_list2->set_attribute('assignments', $m_running_assignments);
                $t_list2->set_attribute('title', "Laufende $g_title_p");
                $t_list2->set_attribute('status_assignment', 'running');
                $t_list2->set_attribute('blocks', $blocks);
                $t_list2->set_attribute('number_of_assignments', $count_running);
        }
        if ($count_finished) {
                $t_list3->set_attribute('picturepath', $g_outlet->picturepath);
                $t_list3->set_attribute('action',$p_action);
                $t_list3->set_attribute('assignments', $m_finished_assignments);
                $t_list3->set_attribute('title', "Beendete $g_title_p");
                $t_list3->set_attribute('status_assignment', 'finished');
                $t_list3->set_attribute('blocks', $blocks);
                $t_list3->set_attribute('number_of_assignments', $count_finished);
        }

        $t_list_header->set_attribute('number_of_assignments', $count_not_started + $count_running + $count_finished);
        $t_list_footer->set_attribute('all_assignments', $m_all_assignments);
        $t_list_footer->set_attribute('action', $p_action);
        $t_list_footer->set_attribute('g_title_s', $g_title_s);

        echo $t_list_header->render();

        // list notstarted assignments
        if ($count_not_started && $g_outlet->status != 'student')
                 echo $t_list1->render();
        // list running assignments
        if ($count_running)
                echo $t_list2->render();
        //list finished assignments
        if ($count_finished && $g_outlet->status != 'student')
                 echo $t_list3->render();

        echo $t_list_footer->render();
}


#####################################
#####################################
#                                       #
#               Help Methods            #
#                                       #
#####################################
#####################################


/**
 * EXAM
 *
 * Calculates the time at which the student has to finish the exam (starting time of the
 * student PLUS length of the exam)
 * If the global endzeitpunkt of the assignment is closer, then this is the Abgabezeitpunkt
 * @return: Abgabezeitpunkt (timestamp)
 */
function getAbgabezeitpunkt($p_assignment_id) {
        global $vips_db,
                   $g_outlet,
                   $g_title_s,
                   $g_assignment_table,
                   $g_inAssignment_table;

        $result = $vips_db->executeQuery("SELECT * FROM $g_assignment_table WHERE ID = '$p_assignment_id'");
        $result->next();

        $assignment = $result->getRow();

        $length = $assignment['Dauer'];
        $end    = $assignment['Ende'];
        $start  = $assignment['Beginn'];
        $now    = date("Y-m-d H:i:s");

        $remainingMinutes = 0;

        $studentStart = $now;

        /*********************************************************************/
        /* Query database table "AufgabenZeiten" for earliest time entry for
        /* user in this exercise_sheet.
        /*
        /* Kurs, vips_klausur and user_id make up the primary key together.
        /*********************************************************************/

        $result = $vips_db->executeQuery("SELECT Beginn FROM vips_aufgaben_zeit
                                WHERE   Kurs                    = '$g_outlet->courseID'
                                AND     $g_assignment_table     = '$p_assignment_id'
                                AND     user_id                 = '$g_outlet->userID'
                                ORDER BY Beginn");

        if ($result->next())
        {
                $user_time = $result->getRow();
                $studentStart = $user_time['Beginn'];
        }

        $lengthSec = $length * 60;  //length in seconds
        $abgabeSec = strtotime($studentStart) + $lengthSec;

        $abgabe = min(strtotime($end), $abgabeSec);  // timestamp!

        return $abgabe;
}

/**
 * EXAM
 *
 * Calculates the remaining minutes for a student
 * TODO: avoid code duplication with getAbgabezeitpunkt()
 *
 * @return: remaining minutes of the exam
 *
 */
function getRemainingMinutes($p_assignment_id) {
        global $vips_db,
                   $g_outlet,
                   $g_title_s,
                   $g_assignment_table,
                   $g_inAssignment_table;

        $result = $vips_db->executeQuery("SELECT * FROM $g_assignment_table WHERE ID = '$p_assignment_id'");
        $result->next();

        $assignment = $result->getRow();

        $m_length       = $assignment['Dauer'];
        $m_end          = $assignment['Ende'];
        $m_start        = $assignment['Beginn'];
        $m_now          = date("Y-m-d H:i:s");

        $m_remainingMinutes = 0;

        /* also show remaining minutes for not yet started exams */
        if($m_now < $m_end /* && $m_now > $m_start */) {

                /*********************************************************************/
                /* Query database table "AufgabenZeiten" for earliest time entry for
                /* user in this exercise_sheet.
                /*
                /* Kurs, vips_klausur and user_id make up the primary key together.
                /*********************************************************************/

                $result = $vips_db->executeQuery("SELECT Beginn FROM vips_aufgaben_zeit
                                        WHERE   Kurs                    = '$g_outlet->courseID'
                                        AND     $g_assignment_table     = '$p_assignment_id'
                                        AND     user_id                 = '$g_outlet->userID'
                                        ORDER BY Beginn");

                if ($result->next())
                        $user_start = $result->get('Beginn');

                $m_remainingMinutes = clip_remaining_time($user_start, $m_length, $m_end);
        }

        return $m_remainingMinutes;
}

/**
 * Clips remaining time to end of exam or test.
 *
 * @param user_start    string  start time of exam (date format)
 * @param exam_length   int     maximum permitted length (minutes)
 * @param exam_end      string  end of exam (date format)
 */
function clip_remaining_time ($user_start, $exam_length, $exam_end)
{
        $time = time();
        $remaining_time = $exam_length * 60;

        if (isset($user_start))
        {
                $elapsed_time = $time - strtotime($user_start);
                $remaining_time -= $elapsed_time;
        }

        // clip remaining time if necessary
        $remaining_time = min($remaining_time, strtotime($exam_end) - $time);

        return ceil($remaining_time / 60);
}

/**
 * EXAM
 *
 * Checks whether the IP address of the user is listed among the IP addresses given by
 * the lecturer (if applicable)
 *
 */

function ip_permitted($p_assignment_id) {
        global $vips_db,
                   $g_outlet,
                   $g_title_s,
                   $g_assignment_table,
                   $g_inAssignment_table;


        $result = $vips_db->executeQuery("SELECT IPBereich
                                        FROM  $g_assignment_table
                                        WHERE ID = '$p_assignment_id'");
        $result->next();
        $assignment = $result->getRow();
        $m_exercise_ip_rangeSpaceSeparatedList  = $assignment["IPBereich"];

        /**************************************************************************/
        /* Explode space separated list into an array and check the resulting single
        /* IPs against the current user IP.
        /* No IP given: user has access.
        /* One or more IPs are given and user IP matches at least one: user has
        /* access.
        /**************************************************************************/

        // Explode seems to hand separator characters, too?!
        // Therefore, this is not too good solution:
        $m_hasAccess = TRUE;
        if(strlen($m_exercise_ip_rangeSpaceSeparatedList) > 0) {

                $m_hasAccess = FALSE;
                $m_exercise_ip_rangeStringArray = split('[ ,]+', $m_exercise_ip_rangeSpaceSeparatedList);

                foreach($m_exercise_ip_rangeStringArray AS $m_IP) {
                        trim($m_IP);
                        $own_IP = $_SERVER['REMOTE_ADDR'];
                        if($m_IP != "") {
                                if (strpos($m_IP,"-")) {
                                        $startend       = explode("-",$m_IP);
                                        $start          = $startend[0];
                                        $end            = $startend[1];
                                        if ($_SERVER['REMOTE_ADDR'] >= $start && $_SERVER['REMOTE_ADDR'] <= $end)
                                                $m_hasAccess = TRUE;

                                }
                                else {
                                        $m_tmp = strpos(" ".$own_IP, $m_IP);
                                        if ($m_tmp == 1) $m_hasAccess = TRUE;
                                }
                        }
                }
        }
        return $m_hasAccess;
}

/**
 * EXAM
 *
 * @return info on Gesperrtsein of exam
 */
function checkInterrupted($p_assignment_id) {
                global $vips_db,
                   $g_outlet,
                   $g_assignment_table,
                   $g_inAssignment_table;

        $result = $vips_db->executeQuery("SELECT istGesperrt
                                                FROM  $g_assignment_table
                                                WHERE ID = '$p_assignment_id'");
        $result->next();
        $assignment = $result->getRow();
        return $assignment["istGesperrt"];
}

?>
