<?php
/*
 * vips_solutions.inc.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2007  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/**
 *
 * Handles student solutions and corrections
 *
 * @author Martin Schrder <MarSchro@Uni-Osnabrueck.De>
 * @version 2007-01-03
 */

require_once 'vips_assignments.inc.php';

/**************************************************************************/
/* Switches between type of exercise sheet to allow database access.
/* Used for function "vips_exercise_sheet_show_solutions".
/**************************************************************************/

function vips_solutions ($subaction = '')
{
    global $g_outlet,
        $g_assignment_table,
        $g_inAssignment_table,
        $g_solution_table,
        $g_solution_points_table;

    /**************************************************************************/
    /* Set global variables
    /**************************************************************************/

    $assignment_type = vips_get_option_param("assignment_type");

    if ($assignment_type == "vips_klausur") {
        $g_inAssignment_table    = 'vips_inKlausur';
        $g_solution_table        = 'vips_K_Loesung';
        $g_solution_points_table = 'vips_K_Loesung_Punkte';
        $g_assignment_table      = 'vips_klausur';  // used as table and attribute reference
    }
    else if ($assignment_type == "vips_uebungsblatt") {
        $g_inAssignment_table    = 'vips_inUebungsblatt';
        $g_solution_table        = 'vips_T_Loesung';
        $g_solution_points_table = 'vips_T_Loesung_Punkte';
        $g_assignment_table      = 'vips_uebungsblatt';  // used as table and attribute reference
    }

    if ($g_outlet->status == "lecturer" || $g_outlet->status == "tutor")
    {
        // checkboxes ("corrections visible") have been changed
        if ($subaction == "change_corrections_visible") {
            change_corrections_visible();  // database update
            show_assignments_overview();  // show overview again
        }
        // shows overview over students' solutions in an assignment
        else if ($subaction == "show_assignment_solutions") {
            show_assignment_solutions();
        }
        else if ($subaction == "autocorrect_solutions") {
            delete_autocorrected_solutions();
            correct_uncorrected_solutions();
            show_assignment_solutions();
        }
        else if ($subaction == "store_correction") {
            store_correction();
        }
        else if ($subaction == "show_participants_overview") {
            show_participants_overview();
        }
        else if ($subaction == 'show_final_grades') {
            show_final_grades();
        }
        // TODO detaillierte Auflistung aller Uebungsblaetter/Klausuren fuer einen Studenten
        else if ($subaction == 'view_student_data') {
            view_student_data();
        }
        else if ($subaction == "edit_solution") {
            edit_solution();
        }
        else if ($subaction == "show_student_assignment_solutions") {
            show_student_assignment_solutions();
        }
        else {
            // default: overview over all assignments that are currently not running
            show_assignments_overview();
        }
    }
    else
    {
        if ($subaction == "edit_solution") {
            edit_solution();
        }
        else if ($subaction == "show_student_assignment_solutions") {
            show_student_assignment_solutions();
        }
        else {
            // default: overview over all assignments that are currently not running
            show_assignments_overview();
        }
    }
}



/******************************************************************************/
/*
/* S H O W   S H E E T S   O V E R V I E W
/*
/* (vips_klausur + vips_uebungsblatt in two functions)
/*
/******************************************************************************/



/**
 * Displays all "Klausuren" and "Uebungsblaetter".
 * Lecturer can select what sheet to correct.
 *
 *
 * @date 2004-03-17
 */

function show_assignments_overview()
{
    global $vips_db, $g_outlet;

    /*********************************************************************/
    /* Defining type of exercise sheets to print.
    /*
    /*********************************************************************/

    show_assignments('vips_uebungsblatt', _('bungsbltter'));

    show_assignments('vips_klausur', _('Klausuren'));

    // display course results
    if ($g_outlet->status == 'student')
    {
        $sql = "SELECT * FROM vips_noten
                WHERE Kurs = '$g_outlet->courseID' AND Prozent != ".UNUSED_GRADE;
        $result = $vips_db->executeQuery($sql);

        // if grades are defined for this course
        if ($result->getRecordCount()) {
            $sql = "SELECT ID FROM vips_klausur
                    WHERE Kurs = '$g_outlet->courseID' AND
                          (korrekturenSichtbar != 't' OR Ende > NOW())
                    UNION
                    SELECT ID FROM vips_uebungsblatt
                    WHERE Kurs = '$g_outlet->courseID' AND
                          (korrekturenSichtbar != 't' OR Ende > NOW())";
            $result = $vips_db->executeQuery($sql);

            // if all assignments are finished and released
            if ($result->getRecordCount() == 0) {
                show_final_grades_student();
            }
        }
    }
}

function show_participants_overview()
{
    global $g_outlet, $vips_db, $vipsTemplateFactory;

    $format = vips_get_option_param('format');

    $exams = array();
    $exercises = array();
    $participants = array();

    // fetch all finished exercises in the course
    $query = "SELECT `ID`
            , `Name`
            , SUM(`Punkte`) as `max_points`
        FROM `vips_uebungsblatt`
            , `vips_inUebungsblatt`
        WHERE `Kurs` = '".$g_outlet->courseID."'
            AND `vips_uebungsblatt` = `ID`
            AND `Ende` < '".date("Y-m-d H:i:s")."'
        GROUP BY `ID`
        ORDER BY `Beginn`, `vips_uebungsblatt`.`Position`
        ";
    $result = $vips_db->executeQuery($query);
    while ($result->next())
    {
        array_push($exercises, array(
            "id" => $result->get("ID"),
            "name" => $result->get("Name"),
            "max_points" => $result->getFloat("max_points")));
    }

    // fetch all finished exams in the course
    $query = "SELECT `ID`
            , `Name`
            , SUM(`Punkte`) as `max_points`
        FROM `vips_klausur`
            , `vips_inKlausur`
        WHERE `Kurs` = '".$g_outlet->courseID."'
            AND `vips_klausur` = `ID`
            AND `Ende` < '".date("Y-m-d H:i:s")."'
        GROUP BY `ID`
        ORDER BY `Beginn`, `vips_klausur`.`Position`
        ";
    $result = $vips_db->executeQuery($query);
    while ($result->next())
    {
        array_push($exams, array(
            "id" => $result->get("ID"),
            "name" => $result->get("Name"),
            "max_points" => $result->getFloat("max_points")));
    }

    /* participant-array:
        [Vorname] => ...,
        [Nachname] => ...,
        [username] => ...,
        [exercises] => array ( [$assignment_id] => Punkte des Nutzers, [$assignment_id] => Punkte des Nutzers, ...)
        [exams] => array ( [$assignment_id] => Punkte des Nutzers, [$assignment_id] => Punkte des Nutzers, ...)
    */
    // fetch all participants in the course (except for lecturers and tutors)
    $query = "SELECT `seminar_user`.`user_id`
            , `username`
            , `Vorname`
            , `Nachname`
        FROM `seminar_user`
            , `auth_user_md5`
        WHERE `Seminar_id` = '".$g_outlet->courseID."'
            AND `seminar_user`.`user_id` = `auth_user_md5`.`user_id`
            AND `status` = 'autor'
        ORDER BY `Nachname`, `Vorname`
        ";
    $result = $vips_db->executeQuery($query);

    while ($result->next())
    {
        $participant_data = $result->getRow();
        $user_id = $result->get('user_id');

        if ($format == 'csv') {
            $participant_data['stud_id'] = get_student_id($user_id);
        }

        // test whether user is in a group
        $query = "SELECT `group_id`
            FROM `vips_gruppe`
                , `vips_inGruppe`
            WHERE `Kursid` = '".$g_outlet->courseID."'
                AND `Gruppenid` = `group_id`
                AND `user_id` = '".$user_id."'
                AND `end` IS NULL
            ";
        $result2 = $vips_db->executeQuery($query);

        if ($result2->next())
            $group_id = $result2->get("group_id");
        else
            $group_id = 0;

        foreach ($exercises as $exercise)
        {
            // SUM() doesn't work - don't ask me why
            $query = "SELECT DISTINCT *
                FROM `vips_T_Loesung`
                    , `vips_T_Loesung_Punkte`
                WHERE ".($group_id ? "`Gruppenid` = ".$group_id : "`Userid` = '".$user_id."'")."
                    AND `Assignmentid` = ".$exercise["id"]."
                    AND `idLoesung` = `Loesungsid`
                    AND `isLastSolution` = 't'
                GROUP BY `Loesungsid`
                ";
            $sum = 0;
            $result2 = $vips_db->executeQuery($query);
            while ($result2->next())
            {
                $sum += $result2->getFloat("Punkte");
            }
            $participant_data['exercises'][$exercise['id']] = max(0, $sum);
        }
        foreach ($exams as $exam)
        {
            // SUM() doesn't work - don't ask me why
            $query = "SELECT DISTINCT *
                FROM `vips_K_Loesung`
                    , `vips_K_Loesung_Punkte`
                WHERE `Userid` = '".$user_id."'
                    AND `Assignmentid` = ".$exam["id"]."
                    AND `idLoesung` = `Loesungsid`
                    AND `isLastSolution` = 't'
                ";
            $sum = 0;
            $result2 = $vips_db->executeQuery($query);
            while ($result2->next())
            {
                $sum += $result2->getFloat("Punkte");
            }
            $participant_data['exams'][$exam['id']] = max(0, $sum);
        }
        $participants[] = $participant_data;
    }

    if ($format == 'csv')
    {
        $tmpl = $vipsTemplateFactory->open("solutions_show_participants_overview_csv");

        // this hack is necessary to disable the standard Stud.IP layout
        ob_end_clean();
        header('Content-Type: text/csv; charset=ISO-8859-1');
        header('Content-Disposition: attachment; filename="'.'Punkteliste'.'.csv"');
        header('Pragma: public');
    }
    else
    {
        $tmpl = $vipsTemplateFactory->open("solutions_show_participants_overview");
    }

    $tmpl->set_attribute("action", 'solutions');
    $tmpl->set_attribute("exercises", $exercises);
    $tmpl->set_attribute("exams", $exams);
    $tmpl->set_attribute("participants", $participants);
    $tmpl->set_attribute("picturepath", $g_outlet->picturepath);
    setlocale(LC_NUMERIC, NULL);
    echo $tmpl->render();
    setlocale(LC_NUMERIC, 'C');

    if ($format == 'csv')
    {
        // this hack is necessary to disable the standard Stud.IP layout
        ob_start('discard_buffer');
    }
}

/**
 * Shows all exercise sheets belonging to a "vips_klausur" or "Uebungsblatt".
 *
 *
 * @date 2004-03-17
 */
function show_assignments($g_sheetType, $g_topic_sheets)
{
        global $g_outlet, $vips_db, $vipsTemplateFactory;

        $m_sum_max_points = 0; // holds the points of all exercise sheets

        if ($g_outlet->status == "lecturer" || $g_outlet->status == "tutor")
        {
                $tmpl = $vipsTemplateFactory->open("solutions_show_assignments_lecturer");
        }
        else
        {
                $tmpl = $vipsTemplateFactory->open("solutions_show_assignments_student");
        }
        $tmpl->set_attribute("t_action", 'solutions');
        $tmpl->set_attribute("t_assignment_type", $g_sheetType);
        $tmpl->set_attribute("t_header_text", $g_topic_sheets);
        $tmpl->set_attribute("picturepath", $g_outlet->picturepath);

    // for lecturers, display switch for changing view between 'corrections'
    // and 'final grades' (but only once, only in exercises!)
    if ($g_outlet->status == 'lecturer' && $g_sheetType == 'vips_uebungsblatt')
    {
        $tmpl->set_attribute('show_switch', true);
    }

        /**********************************************************************/
        /* Iterates through exercise sheets
        /**********************************************************************/
        $m_current_time = date("Y-m-d H:i:s");

        if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor')
        {
                $query = "SELECT ID
                                , Name
                                , ".$g_sheetType.".Position
                                , Beginn
                                , Ende
                                , istGesperrt
                                , korrekturenSichtbar
                        FROM ".$g_sheetType."
                        WHERE Kurs = '".$g_outlet->courseID."'
                        ORDER BY Beginn, ".$g_sheetType.".Position";
        }
        else
        {
                $query = "SELECT ID
                                , Name
                                , ".$g_sheetType.".Position
                        , Beginn
                                , Ende
                                , istGesperrt
                                , korrekturenSichtbar
                        FROM ".$g_sheetType."
                        WHERE Kurs='".$g_outlet->courseID."'
                                AND Ende <= '$m_current_time'
                        ORDER BY Beginn, ".$g_sheetType.".Position";
        }

        $result1 = $vips_db->executeQuery($query);

        if ($result1->getRecordCount() != 0)
        {
                $assignments_array = array();

                while ($result1->next())
                {
                        /******************************************************************/
                        /* Get basic data for exercise sheet from previous query
                        /******************************************************************/
                        $assignment_id                = $result1->get("ID");
                        $assignment_name              = $result1->get("Name");
                        $assignment_date_start= date("d.m.Y, H:i", strtotime($result1->get("Beginn")));
                        $assignment_date_end  = date("d.m.Y, H:i", strtotime($result1->get("Ende")));
                        $m_show_corrections_to_students = $result1->get("korrekturenSichtbar");
                        /******************************************************************/
                        /* Get sum of points for exercise sheet in a new query
                        /******************************************************************/
                        $max_points = get_max_points_assignment($assignment_id, $g_sheetType);
                        $m_sum_max_points += $max_points;
                        if($max_points == "") {
                                $max_points = 0;
                        }

                        array_push($assignments_array, array("title" => $assignment_name, "start" => $assignment_date_start, "end" => $assignment_date_end, "id" => $assignment_id, "block" => $assignment_blockname, "corrections" => $m_show_corrections_to_students, "points" => $max_points));
                }

                $tmpl->set_attribute("t_assignments", $assignments_array);
                $tmpl->set_attribute("t_sum_max_points", $m_sum_max_points);

        }

        setlocale(LC_NUMERIC, NULL);
        echo $tmpl->render();
        setlocale(LC_NUMERIC, 'C');
}

/**
 * Checkboxes ("corrections visible") have been changed. Fetch the new values and store them
 * into database.
 */
function change_corrections_visible()
{
        global $vips_db, $g_outlet, $g_assignment_table;

        // $m_corrections_visible_array stores all ids that are checked.
        // Unchecked ids are NOT stored!!!
        $m_corrections_visible_array = vips_get_int_array_param("corrections_visible");

        // First, reset all korrekturenSichtbar fields. (resets ALL assignments, not only the already finished ones, but doesn't matter...)
        $query = "UPDATE $g_assignment_table
                SET korrekturenSichtbar = 'f'
                WHERE `Kurs` = '".$g_outlet->courseID."'
                ";
        $vips_db->executeUpdate($query);

        // Then, store all ids as checked which are in the array.
        if (count($m_corrections_visible_array))
        {
                $query = "UPDATE $g_assignment_table
                        SET korrekturenSichtbar = 't'
                        WHERE `Kurs` = '".$g_outlet->courseID."' AND
                              `ID` IN (".join(',', array_keys($m_corrections_visible_array)).")";
                $vips_db->executeUpdate($query);
        }
}

/**
 * Shows solution points for each student/group with a link to view solution and correct it.
 */
function show_assignment_solutions()
{
    global $vips_db,
        $g_outlet,
        $g_assignment_table,
        $g_inAssignment_table,
        $g_solution_table,
        $g_solution_points_table,
        $vipsTemplateFactory,
        $vipsPlugin;

    /* check permission */
    if ($g_outlet->status == 'student') {
        return;
    }

    $course_id       = $g_outlet->courseID;
    $assignment_type = vips_get_option_param("assignment_type");
    $assignment_id   = vips_get_int_param("assignment_id");
    $view            = vips_get_option_param('view');

    // fetch name of exam/exercise sheet
    $query = "SELECT Name, Ende, Dauer
              FROM $g_assignment_table
              WHERE ID = $assignment_id";
    $result = $vips_db->executeQuery($query);
    $result->next();
    $assignment_name = $result->get("Name");
    $end      = $result->getTimestamp('Ende', 'U');
    $duration = $result->getInt('Dauer');



    /////////////////////////////////////////////////////
    // Fetch all exercises that are in the assignment. //
    /////////////////////////////////////////////////////

    $query = "SELECT DISTINCT
                $g_inAssignment_table.`vips_aufgabe`
                , `vips_aufgabe`.Name
                , $g_inAssignment_table.`Punkte`
                , $g_inAssignment_table.`Position`
        FROM
                $g_inAssignment_table
                , `vips_aufgabe`
        WHERE
                $g_inAssignment_table.$g_assignment_table = '$assignment_id'
                AND vips_aufgabe.ID = $g_inAssignment_table.`vips_aufgabe`
        ORDER BY
                $g_inAssignment_table.`Position`
        ";
    $result = $vips_db->executeQuery($query);

    $all_exercises         = array();
    $max_points_assignment = 0;
    while ($result->next()) {
        $exercise_id       = $result->getInt('vips_aufgabe');
        $exercise_name     = $result->getString('Name');
        $exercise_position = $result->getInt('Position');
        $max_points        = $result->getFloat('Punkte');

        $all_exercises[$exercise_id] = array(
            'id'             => $exercise_id,
            'name'           => $exercise_name,
            'position'       => $exercise_position,
            'max_points'     => $max_points,
            'reached_points' => 0,
            'corrected'      => 0,
            'safe'           => 'f',
            'solved'         => false);

        $max_points_assignment += $max_points;
    }



    /////////////////////////////////////////////////////////////////////////
    // Fetch/compute running information (user id, ip address, start time, //
    // remaining time) for exams.                                          //
    /////////////////////////////////////////////////////////////////////////
    if ($assignment_type == 'vips_klausur') {
        $sql = "SELECT user_id, Beginn, ip
            FROM vips_aufgaben_zeit
            WHERE vips_aufgaben_zeit.Kurs = '$course_id'
              AND vips_aufgaben_zeit.vips_klausur = '$assignment_id'";
        $result = $vips_db->executeQuery($sql);

        while ($result->next())
        {
            $user_id   = $result->get('user_id');
            $start     = $result->getTimestamp('Beginn', 'U');
            $remaining = clip_remaining_time(date('Y-m-d H:i:s', $start), $duration, date('Y-m-d H:i:s', $end));
            $ip        = $result->get('ip');
            $host      = $ip ? gethostbyaddr($ip) : NULL;

            $running_info[$user_id] = array(
                'start'     => $start,
                'remaining' => $remaining,
                'ip'        => $ip,
                'host'      => $host);
        }
    }



    //////////////////////////////////////////////////////////////////
    // Fetch all users participating in this course plus all others //
    // (= externals) who have submitted a solution.                 //
    //////////////////////////////////////////////////////////////////

    // all participants who are not a group member
    $sql_singles = "SELECT 'single' AS solver_type,
            seminar_user.user_id,
            TRIM(CONCAT(title_front, ' ', Vorname, ' ', Nachname, IF (title_rear != '', CONCAT(', ', title_rear), ''))) AS full_name,
            auth_user_md5.username AS user_name,
            NULL AS group_id,
            NULL AS group_name,
            CONCAT('1', Nachname, ' ', Vorname) AS sort_order
        FROM seminar_user,
             user_info,
             auth_user_md5
        WHERE user_info.user_id = seminar_user.user_id
          AND auth_user_md5.user_id = seminar_user.user_id
          AND Seminar_id = '$course_id'
          AND seminar_user.status = 'autor'";

    // assert that users are not in a group
    $sql_exclude_groups = "AND NOT EXISTS(
        SELECT *
        FROM vips_inGruppe
        LEFT JOIN vips_gruppe
               ON vips_gruppe.Gruppenid = vips_inGruppe.group_id
        WHERE vips_inGruppe.user_id = seminar_user.user_id
          AND vips_gruppe.Kursid = '$course_id'
          AND vips_inGruppe.end IS NULL)";

    // all participants who are in a group
    $sql_group_members = "SELECT 'group_member' AS solver_type,
            seminar_user.user_id,
            TRIM(CONCAT(title_front, ' ', Vorname, ' ', Nachname, IF (title_rear != '', CONCAT(', ', title_rear), ''))) AS full_name,
            auth_user_md5.username AS user_name,
            vips_inGruppe.group_id,
            vips_gruppe.Gruppenname as group_name,
            CONCAT('2', Gruppenname, ' ', Nachname, ' ', Vorname) AS sort_order
        FROM seminar_user,
             user_info,
             auth_user_md5,
             vips_inGruppe,
             vips_gruppe
        WHERE user_info.user_id = seminar_user.user_id
          AND auth_user_md5.user_id = seminar_user.user_id
          AND vips_inGruppe.user_id = seminar_user.user_id
          AND vips_gruppe.Gruppenid = vips_inGruppe.group_id
          AND Seminar_id = '$course_id'
          AND seminar_user.status = 'autor'
          AND vips_gruppe.Kursid = '$course_id'
          AND vips_inGruppe.end IS NULL";

    // all people who have submitted a solution but are no course participant
    $sql_externals = "SELECT 'external' AS solver_type,
            Userid AS user_id,
            Userid AS full_name,
            NULL AS user_name,
            NULL AS group_id,
            NULL AS group_name,
            CONCAT('3', Userid) AS sort_order
        FROM $g_solution_table
        WHERE Assignmentid = '$assignment_id'
          AND $g_solution_table.Userid != 'nobody'
          AND NOT EXISTS(
              SELECT user_id
              FROM seminar_user
              WHERE seminar_user.user_id = $g_solution_table.Userid
                AND Seminar_id = '$course_id')";

    if ($assignment_type == 'vips_klausur') {
        $sql = "$sql_singles ORDER BY sort_order";
    }
    else {
        $sql = "$sql_singles $sql_exclude_groups UNION $sql_group_members UNION $sql_externals ORDER BY sort_order";
    }
    $result = $vips_db->executeQuery($sql);

    $complete_array = array();
    while ($result->next()) {  // each solver
        $solver_type = $result->getString('solver_type');
        $user_id     = $result->getString('user_id');
        $full_name   = $result->getString('full_name');
        $user_name   = $result->getString('user_name');
        $group_id    = $result->getInt('group_id');
        $group_name  = $result->getString('group_name');
        $solver_id   = $assignment_type == 'vips_uebungsblatt' && $solver_type == 'group_member' ? $group_id : $user_id;

        // skip former course members (i.e. users with an md5 hash as id)
        if ($solver_type == 'external' && preg_match('/^\w{32}$/', $user_id)) {
            continue;
        }

        if ($assignment_type == 'vips_klausur')
        {
            if (!isset($running_info[$user_id]['remaining'])) {
                $view_flag = 'pending';         // user has not yet started to work on the exam
            } else if ($running_info[$user_id]['remaining'] > 0) {
                $view_flag = 'working';         // user is still working on the exam
            } else {
                $view_flag = '';                // user has finished the exam
            }

            // if the solver shall not be shown, continue with the next one
            if ($view != $view_flag)
            {
                continue;
            }
        }

        if (!is_array($complete_array[$solver_id])) {
            if ($solver_type != 'group_member') {  // = single or external
                $complete_array[$user_id] = array(
                    'display'        => $assignment_type == 'vips_uebungsblatt' ? false : true,  // default: don't display him in exercise sheets, display him in exams
                    'solver_type'    => $solver_type,
                    'solverID'       => $user_id,
                    'solverName'     => $full_name,
                    'solverUsername' => $user_name,
                    'single_solver'  => true,
                    'reached_points' => 0,  // initialise to 0
                    'max_points'     => $max_points_assignment,
                    'exercises'      => $all_exercises);
            } else {  // = group_member
                $complete_array[$group_id] = array(
                    'display'        => $assignment_type == 'vips_uebungsblatt' ? false : true,  // default: don't display him in exercise sheets, display him in exams
                    'solver_type'    => 'group',
                    'solverID'       => $group_id,
                    'solverName'     => $group_name,
                    'solverUsername' => null,
                    'single_solver'  => false,
                    'reached_points' => 0,  // initialise to 0
                    'max_points'     => $max_points_assignment,
                    'exercises'      => $all_exercises);
            }
        }

        // special treatment for group members: pass information about the
        // individual members, besides the overall group info
        if ($solver_type == 'group_member') {
            $complete_array[$solver_id]['group_members'][] = array(
                'user_name' => $user_name,
                'full_name' => $full_name);
        }
    }



    /////////////////////////////////////
    // Fetch all solutions and points. //
    /////////////////////////////////////

    // keep this LEFT JOIN!
    $sql = "SELECT DISTINCT $g_solution_table.Userid AS 'user_id',
            $g_solution_table.Gruppenid AS 'group_id',
            $g_solution_table.Aufgabenid AS exercise_id,
            $g_solution_table.isAlreadyCorrected AS corrected,
            $g_solution_points_table.Punkte AS reached_points,
            $g_solution_points_table.Safe AS safe
        FROM $g_solution_table
        LEFT JOIN $g_solution_points_table
               ON $g_solution_points_table.idLoesung = $g_solution_table.Loesungsid
        WHERE Assignmentid = '$assignment_id'
          AND isLastSolution = 't'";
    $result = $vips_db->executeQuery($sql);

    while ($result->next()) {  // each solution
        $user_id        = $result->getString('user_id');
        $group_id       = $result->getInt('group_id');
        $solver_id      = $assignment_type == 'vips_uebungsblatt' && !empty($group_id) ? $group_id : $user_id;
        $exercise_id    = $result->getInt('exercise_id');
        $corrected      = $result->getBoolean('corrected');
        $reached_points = $result->getFloat('reached_points');  // may be null
        $safe           = $result->getString('safe');  // may be null
        if (!isset($reached_points)) $reached_points = 0;
        if (!isset($safe)) $safe = 'f';

        if (is_array($complete_array[$solver_id])) {
            $complete_array[$solver_id]['display'] = true;  // display him when he has at least one solution!
            $complete_array[$solver_id]['reached_points'] += $reached_points;
            $complete_array[$solver_id]['exercises'][$exercise_id]['corrected']      = $corrected;
            $complete_array[$solver_id]['exercises'][$exercise_id]['reached_points'] = $reached_points;
            $complete_array[$solver_id]['exercises'][$exercise_id]['safe']           = $safe;
            $complete_array[$solver_id]['exercises'][$exercise_id]['solved']         = true;
        }
    }

    $tmpl = $vipsTemplateFactory->open("solutions_show_assignment_solutions");
    $tmpl->set_attributes(array(
        'picturepath'     => $g_outlet->picturepath,
        'vips_imagepath'  => $vipsPlugin->getPluginURL().'/images/',
        'assignment_type' => $assignment_type,
        'assignment_id'   => $assignment_id,
        'assignment_name' => $assignment_name,
        'action'          => 'solutions',
        'subaction'       => 'show_assignment_solutions',
        'view'            => $view,
        'complete_array'  => $complete_array));

    setlocale(LC_NUMERIC, NULL);
    echo $tmpl->render();
    setlocale(LC_NUMERIC, 'C');
}

/**
 * TODO?
 * Detailed listing of all blocks, sheets and exams for one student (similar to solution view)
 */
function view_student_data() {
        $item_type = vips_get_option_param('item_type');
        $item_id = vips_get_int_param('item_id');

        if ($item_type == 'sheets') {
                $_POST['assignment_type'] = "vips_uebungsblatt";
                $_POST['assignment_id'] = $item_id;

                show_assignment_solutions();

        }
        elseif ($item_type = 'exams') {
                $_POST['assignment_type'] = "vips_klausur";
                $_POST['assignment_id'] = $item_id;

                show_assignment_solutions();
        }
        //TODO:
                /*
                $query = "SELECT *
                        FROM vips_uebungsblatt
                        WHERE ID = $item_id
                        ";
                $result = $vips_db->executeQuery($query);
                $result->next();
                $sheet = $result->getRow();

                ##########################################################
                # get all exercises, max points and user solution points #

                $query = "SELECT DISTINCT
                        vips_inUebungsblatt.vips_aufgabe
                        , vips_aufgabe.Name
                        , vips_inUebungsblatt.Punkte
                        , vips_inUebungsblatt.Position
                FROM
                        vips_inUebungsblatt
                        , `vips_aufgabe`
                WHERE
                        vips_inUebungsblatt.vips_uebungsblatt = '$item_id'
                        AND vips_aufgabe.ID = vips_inUebungsblatt.vips_aufgabe
                ORDER BY
                        vips_inUebungsblatt.Position
                ";
                $result = $vips_db->executeQuery($query);

                $points_sheet = 0;
                $exercises = array();
                while ($result->next())
                {
                        $entry = $result->getRow();
                        $exercises[] = $entry;
                        $points_sheet += $entry["Punkte"];
                }
        //get all exercises and solutions
        //wie in Loesungen
        }*/

}





/******************************************************************************/
/*
/* A U T O K O R R E K T U R
/*
/******************************************************************************/



/**
 * Deletes all solution-points, where the solutions are automatically corrected
 *
 * @date 2004-06-07
 * @author Philipp Hgelmeyer
 */
function delete_autocorrected_solutions()
{
        // Database variables
        global $vips_db, $g_outlet, $g_inAssignment_table, $g_solution_table,
                $g_solution_points_table, $g_assignment_table;

        $assignment_id = vips_get_int_param("assignment_id");

        $query = "SELECT
                        Loesungsid
                FROM
                        $g_solution_table
                        , $g_assignment_table
                WHERE
                        $g_assignment_table.Kurs = '".$g_outlet->courseID."'
                        AND $g_solution_table.Assignmentid = '$assignment_id'
                        AND $g_solution_table.isAlreadyCorrected = '0'
                        AND $g_solution_table.isLastSolution = 't'
                ";
        $result = $vips_db->executeQuery($query);

        while ($result->next())
        {
                $m_id = $result->get("Loesungsid");
                $query = "DELETE FROM $g_solution_points_table WHERE idLoesung = '$m_id'";
                $vips_db->executeUpdate($query);
        }
}



/**
 * Corrects iteratively all still uncorrected solutions.
 *
 * @date 2004-01-08
 * @author Philipp Hgelmeyer
 */

function correct_uncorrected_solutions()
{
    // Database variables
    global $vips_db, $g_outlet, $g_inAssignment_table, $g_solution_table,
        $g_solution_points_table, $g_assignment_table;

    $assignment_id = vips_get_int_param("assignment_id");

    // Select all yet uncorrected solutions (one for each group).
    $query = "SELECT
            $g_solution_table.Loesungsid,
            $g_assignment_table.ID,
            $g_assignment_table.mc_auswertung,
            $g_inAssignment_table.vips_aufgabe
        FROM
            $g_solution_table,
            $g_assignment_table,
            $g_inAssignment_table
        WHERE
            $g_assignment_table.Kurs = '".$g_outlet->courseID."'
            AND $g_assignment_table.ID = $g_inAssignment_table.$g_assignment_table
            AND $g_solution_table.Aufgabenid = $g_inAssignment_table.vips_aufgabe
            AND isLastSolution = 't'
            AND isAlreadyCorrected = '0'
            AND $g_inAssignment_table.$g_assignment_table = '$assignment_id'
        ORDER BY
            Loesungsid
        ";
    $result = $vips_db->executeQuery($query);

    // Correct each of these solutions.
    while ($result->next())
    {
        $_POST["exercise_id"] = $result->get("vips_aufgabe");
        $solution_id = $result->get("Loesungsid");
        $assignment_id = $result->get("ID");
        $mc_auswertung = $result->getInt('mc_auswertung');
        correct_solution($solution_id, $assignment_id, $mc_auswertung);
    }
}



/**
 * Function gets a solution_id and an exercise_sheet_id and stores the points
 * for a solution in the database.
 *
 * @date 2004-01-07
 * @author Philipp Hgelmeyer
 */

function correct_solution($p_solution_id, $p_assignment_id, $mc_auswertung)
{
    // FUNKTIONIERT NICHT FR KLAUSUR (Gruppenid-Abfrage ganz unten) (?)
    global $vips_db
            , $g_inAssignment_table
            , $g_solution_table
            , $g_solution_points_table
            , $g_assignment_table;

     $query = "SELECT
                    Gruppenid
                    , Userid
                    , Aufgabenid
                    , Loesung
            FROM
                    $g_solution_table
            WHERE
                    Loesungsid = $p_solution_id
            ";
    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
        $loesung     = $result->get("Loesung");
        $exercise_id = $result->get("Aufgabenid");
        $group_id    = $result->get("Gruppenid");
        $user_id     = $result->get("Userid");
    }
    else {
        return -1;
    }

    $query = "SELECT
                    URI
                    , Aufgabe
            FROM
                    vips_aufgabe
            WHERE
                    ID = $exercise_id
            ";
    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
        $m_aufgabe     = $result->get("Aufgabe");
        $m_aufgabentyp = $result->get("URI");
    }
    else {
        return -2;
    }

    require_once("exercises/$m_aufgabentyp.php");
    $m_aufgabenObjekt = new $m_aufgabentyp($m_aufgabe);
    $m_aufgabenObjekt->setEvaluationMode($mc_auswertung);

    $query = "SELECT
                    Punkte
            FROM
                    $g_inAssignment_table
            WHERE
                    $g_assignment_table = $p_assignment_id
            AND
                    vips_aufgabe = '$exercise_id'
            ";
    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
        $points = $result->getFloat("Punkte");
    }

    #### cdeiwiks:vvvvvvvvvvvvvvvvvvv
    // prolog exercises need some additional parameters
    // might be that prolog exercise is chosen in a random exercise, so bergib params mit and let rnd_exercise decide what to do with them
    if ($m_aufgabentyp == 'pl_exercise' || $m_aufgabentyp == "rnd_exercise") {
        $_POST["user_id"] = $user_id;  // needed in rnd's evaluate function in order to determine the correct subexercise
        $evaluation = $m_aufgabenObjekt->evaluate($m_aufgabe, $loesung, $user_id, $points);
    }
    else {
        $evaluation = $m_aufgabenObjekt->evaluate($m_aufgabe, $loesung);
    }
    #### endcdeiwiks^^^^^^^^^^^^^^^^^^

    $percent = $evaluation['percent'];
    $safe    = $evaluation['safe'];
    $safe    = $safe == 1 ? 't' : 'f';

    $reached_points = round_to_half_point(($percent * $points) / 100);



    /******************************************************************/
    /*
    /* Insert points for a solution in appropriate table.
    /*
    /******************************************************************/

    if ($group_id != 0 && $g_assignment_table == "vips_uebungsblatt") {
        // In "Uebungsblatt"-mode, we need all the people (plural!) who handed the solution in.
        // These people get the points for the solution.
        $query = "SELECT
                        user_id
                FROM
                        vips_inGruppe
                WHERE
                        group_id = $group_id
                        AND end IS NULL";
        $result = $vips_db->executeQuery($query);

        while ($result->next()) {
            $m_userid = $result->get("user_id");
            $query = "INSERT INTO
                            $g_solution_points_table
                            ( `idUser`
                            , `idLoesung`
                            , `Punkte`
                            , `Safe`
                            )
                    VALUES
                            ( '$m_userid'
                            , '$p_solution_id'
                            , '$reached_points'
                            , '$safe'
                            )
                    ";
            $vips_db->executeUpdate($query);
        } // end while
    }
    else {
        // In "vips_klausur"-mode, we need the person who handed the solution in.
        // This person gets the points for the solution.

        $query = "SELECT
                                Userid
                        FROM
                                $g_solution_table
                        WHERE
                                Loesungsid = $p_solution_id";
        $result = $vips_db->executeQuery($query);
        $result->next();
        $m_userid = $result->get("Userid");

        $query = "INSERT INTO
                        $g_solution_points_table
                        ( `idUser`
                        , `idLoesung`
                        , `Punkte`
                        , `Safe`
                        )
                VALUES (
                        '$m_userid'
                        , '$p_solution_id'
                        , '$reached_points'
                        , '$safe'
                        )
                ";
        $vips_db->executeUpdate($query);
    }
    return 1;
}





/**
 * Invokes [xx]_exercise.php and gets formular field that allows lecturer
 * to correct the student's solution.
 */
function edit_solution()
{
    global $vipsTemplateFactory,
        $g_outlet,
        $vips_db;

    // Database variables
    global $g_inAssignment_table,
        $g_solution_table,
        $g_solution_points_table,
        $g_sheetType,
        $g_assignment_table;

    $exercise_id       = vips_get_int_param("exercise_id");
    $assignment_id     = vips_get_int_param("assignment_id");
    $assignment_type   = vips_get_option_param("assignment_type");
    $solver_id         = harmless(vips_get_string_param("solver_id"));
    $solver_type       = vips_get_string_param('solver_type');
    $exercise_position = vips_get_int_param("exercise_position");
    $single_solver     = vips_get_int_param("single_solver");
    $view              = vips_get_option_param('view');

    // prevent url hacking
    if ($g_outlet->status == 'student') {
        $solver_id     = $g_outlet->userID;  // don't rely on passed solver_id
        $single_solver = 1;                  // and single_solver

        if ($assignment_type == 'vips_uebungsblatt') {
            // fetch user's group when assignment was handed in
            $sql = "SELECT vips_inGruppe.group_id
                FROM vips_inGruppe, vips_gruppe, $g_solution_table
                WHERE vips_inGruppe.user_id = '$solver_id'
                  AND vips_gruppe.Kursid = '$g_outlet->courseID'
                  AND vips_inGruppe.group_id = vips_gruppe.Gruppenid
                  AND $g_solution_table.Aufgabenid = $exercise_id
                  AND $g_solution_table.Gruppenid = vips_gruppe.Gruppenid
                  AND $g_solution_table.isLastSolution = 't'
                  AND $g_solution_table.Zeitpunkt >= vips_inGruppe.start
                  AND ($g_solution_table.Zeitpunkt < vips_inGruppe.end OR vips_inGruppe.end IS NULL)";
            $result = $vips_db->executeQuery($sql);

            if ($result->next()) {  // group found
                $solver_id     = $result->getInt('group_id');
                $single_solver = 0;
            }
        }
    }

    if ($solver_type != 'external') {
        $solver_name = get_solver_fullname($solver_id);
    } else {
        $solver_name = $solver_id;
    }

    /*************************************************************************/
    /* Database: get exercise data for given exercise
    /*************************************************************************/

    $query = "SELECT vips_aufgabe.URI
            , vips_aufgabe.Name
            , $g_inAssignment_table.Position
        FROM vips_aufgabe
            , $g_inAssignment_table
        WHERE vips_aufgabe.ID = '$exercise_id'
            AND $g_inAssignment_table.vips_aufgabe = vips_aufgabe.ID
            AND $g_inAssignment_table.$g_assignment_table = '$assignment_id'
        ";
    $result = $vips_db->executeQuery($query);
    $result->next();

    $exercise_type     = $result->get("URI");
    $exercise_name     = $result->get("Name");
    $exercise_position = $result->get("Position");
    if (!isset($exercise_position))  // is not set in random exercises
        $exercise_position = vips_get_int_param("exercise_position");

    // get id of the following exercise //
    $query = "SELECT *
        FROM $g_inAssignment_table
        WHERE $g_assignment_table = $assignment_id
            AND Position > '$exercise_position'
        ORDER BY Position
        ";
    $result = $vips_db->executeQuery($query);
    #cdeiwiks: nicht im Uebungsblatt naechste Aufgabe, sondern naechste *geloeste* Aufgabe:
    while ($result->next()) {
        $entry = $result->getRow();
        $exercise_id_temp   = $entry['vips_aufgabe'];
        $assignment_id_temp = $entry[$g_assignment_table];
        $position_temp      = $entry['Position'];
        $query2 = "SELECT *
            FROM $g_solution_table
            WHERE Aufgabenid = '$exercise_id_temp'
                AND Assignmentid = '$assignment_id_temp'
                AND isLastSolution = 't'
            ";
        if ($assignment_type == 'vips_klausur' || $single_solver)
            $query2 .= " AND Userid = '$solver_id'";
        else
            $query2 .= " AND Gruppenid = '$solver_id'";
        $result2 = $vips_db->executeQuery($query2);

        if ($result2->getRecordCount()) {
            $next_exercise_id = $exercise_id_temp; //$result->get("vips_aufgabe");
            $next_exercise_position = $position_temp; //$result->get("Position");
            break;
        }
    }



    #################################################
    # get ID of the next USER for the same exercise #
    #################################################

    $next_solver_id     = false;
    $next_single_solver = false;
    $next_name          = false;

    // first, look for other single solvers
    if ($solver_type == 'single')
    {
        if ($assignment_type == 'vips_uebungsblatt') {
            /* There must not exist a group the user currently is member in! */
            $sql = "SELECT vips_inGruppe.user_id
                    FROM vips_inGruppe, vips_gruppe
                    WHERE vips_gruppe.Gruppenid = vips_inGruppe.group_id
                        AND vips_inGruppe.end IS NULL
                        AND vips_gruppe.Kursid = '$g_outlet->courseID'";
            $result = $vips_db->executeQuery($sql);

            while ($result->next()) {
                $user_array[] = $result->getString('user_id');
            }
        }

        if (isset($user_array)) {
            $not_in_group = "AND Userid NOT IN ('".join("','", $user_array)."')";
        } else {
            $not_in_group = '';
        }

        // fetch all single solver users who have submitted a solution
        $sql = "
            SELECT
                seminar_user.user_id
            FROM
                $g_solution_table,
                auth_user_md5,
                seminar_user
            WHERE
                Assignmentid = '$assignment_id'
                AND Aufgabenid = '$exercise_id'
                AND isLastSolution = 't'
                AND seminar_user.Seminar_id = '$g_outlet->courseID'
                AND seminar_user.user_id = Userid
                AND seminar_user.status = 'autor'
                AND seminar_user.user_id = auth_user_md5.user_id
                $not_in_group
            ORDER BY
                Nachname, Vorname
            ";
        $result = $vips_db->executeQuery($sql);

        // skip all users before the current one
        while ($result->next() && $result->getString('user_id') != $solver_id);

        if ($result->next())  // take the next user (if exists...)
        {
            $next_solver_id     = $result->getString('user_id');
            $next_single_solver = 1;
            $next_solver_type   = 'single';
            $next_name          = get_fullname($next_solver_id);
        }
    }

    // if none found, look for groups
    if (!$next_solver_id && $solver_type != 'external' && $assignment_type == 'vips_uebungsblatt')
    {
        $sql = "
            SELECT
                vips_gruppe.Gruppenid,
                vips_gruppe.Gruppenname
            FROM
                $g_solution_table,
                vips_gruppe
            WHERE
                Assignmentid = '$assignment_id'
                AND Aufgabenid = '$exercise_id'
                AND isLastSolution = 't'
                AND $g_solution_table.Gruppenid != 0
                AND $g_solution_table.Gruppenid = vips_gruppe.Gruppenid
            GROUP BY
                vips_gruppe.Gruppenname
            ";
        $result = $vips_db->executeQuery($sql);

        // CURRENT solver might be a single solver, while NEXT solver might be a group!
        if (!$single_solver)
        {
            // skip all users before the current one
            while ($result->next() && $result->getString('Gruppenid') != $solver_id);
        }

        if ($result->next())  // take the next user
        {
            $next_solver_id     = $result->getString('Gruppenid');
            $next_single_solver = 0;
            $next_solver_type   = 'group';
            $next_name          = $result->getString('Gruppenname');
        }
    }

    // if none found, look for external solvers
    if (!$next_solver_id && $assignment_type == 'vips_uebungsblatt') {
        $sql = "SELECT Userid
            FROM $g_solution_table
            WHERE Assignmentid = $assignment_id
              AND Aufgabenid = $exercise_id
              AND isLastSolution = 't'
              AND NOT EXISTS(
                  SELECT user_id
                  FROM seminar_user
                  WHERE seminar_user.user_id = $g_solution_table.Userid
                    AND Seminar_id = '$g_outlet->courseID')
              AND Userid != 'nobody'
            ORDER BY Userid";
        $result = $vips_db->executeQuery($sql);

        if ($solver_type == 'external') {
            // skip other externals before
            while ($result->next() && $result->getString('Userid') != $solver_id);
        }

        // next external
        if ($result->next()) {
            $next_solver_id     = $result->getString('Userid');
            $next_single_solver = true;
            $next_solver_type   = 'external';
            $next_name          = $next_solver_id;
        }
    }



    /*************************************************************************/
    /* Database: get solution data for given exercise and given group/user
    /*************************************************************************/

    $query = "SELECT Loesungsid, Loesung, visibleForStudent
        FROM $g_solution_table
        WHERE Aufgabenid = '$exercise_id'
            AND ".($assignment_type == "vips_uebungsblatt" && !$single_solver ? "Gruppenid" : "Userid")." = '$solver_id'
            AND Assignmentid = '$assignment_id'
            AND isLastSolution = 't'
        ";
    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
        $solution_id         = $result->get('Loesungsid');
        $solution_xml        = $result->get('Loesung');
        $visible_for_student = $result->get('visibleForStudent');
    }

    ###############################################################################
    # cdeiwiks vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    #

    ###########################################################
    # Prolog functions for tutors to try out student solution #
    ###########################################################

    if (vips_get_submit_param("pl_exercise_mode_query") ||
        vips_get_submit_param("pl_exercise_mode_querynext") ||
        vips_get_submit_param("pl_exercise_mode_eval") ||
        vips_get_submit_param('pl_exercise_mode_upload') ||
        vips_get_submit_param('pl_exercise_mode_download'))
    {
        // store the current version of the lecturer's commented solution in the
        // database IF it is neither empty nor identical to the student solution
        $commented_solution = vips_get_string_param('commented_solution', true);
        if (!empty($commented_solution)) {  // must not be empty
            // extract the student solution text from XML (don't use getTagsFromXML()!)
            preg_match('/\<body\>(.*)\<\/body\>/s', $solution_xml, $student_solution_text);
            $student_solution_text = $student_solution_text[1];

            if ($commented_solution != $student_solution_text) {  // must be different from student solution
                $sql = "
                    UPDATE $g_solution_table
                    SET bearbeiteteLoesung = '$commented_solution'
                    WHERE Loesungsid = '$solution_id'
                        AND isLastSolution = 't'
                    ";
                $vips_db->executeQuery($sql);
            }
        }

        $submitted = true;
        require_once("exercises/pl_exercise.php");

        $o_exercise = new pl_exercise;

        $answer_0 = vips_get_string_param('answer_0', true);

        if (vips_get_submit_param('pl_exercise_mode_query')) {
            $pl_out  = $o_exercise->evaluateQuery($solver_id, $g_outlet->courseID);
        }
        elseif (vips_get_submit_param('pl_exercise_mode_querynext')) {
            $pl_count = vips_get_int_param('pl_count') + 1;
            $pl_out   = $o_exercise->evaluateQuery($solver_id, $g_outlet->courseID, $pl_count);
        }
        elseif (vips_get_submit_param('pl_exercise_mode_upload')) {
            $answer_0 = addslashes($o_exercise->uploadSolution());      // magic_quotes_gpc
        }
        elseif (vips_get_submit_param('pl_exercise_mode_download')) {
            $filename = get_solver_fullname($solver_id).'_'.$exercise_name;
            $o_exercise->downloadSolution(preg_replace('/\W/', '_', $filename));
        }
        elseif (vips_get_submit_param('pl_exercise_mode_eval')) {

            $query = "SELECT Aufgabe
                FROM vips_aufgabe
                WHERE ID = '$exercise_id'
                ";
            $result = $vips_db->executeQuery($query);
            $result->next();
            $exercise_xml = $result->get('Aufgabe');

            $query = "SELECT Punkte
                FROM $g_inAssignment_table
                WHERE $g_assignment_table = $assignment_id
                    AND vips_aufgabe = '$exercise_id'
                ";
            $result = $vips_db->executeQuery($query);
            $result->next();
            $gesamtpunktzahl = $result->getFloat('Punkte');

            $solution_view = vips_get_option_param('solution_view');
            $answer_param = $solution_view == 'solution_edit'
                ? 'commented_solution'
                : 'answer_0';
            $answer = vips_get_unquoted_string_param($answer_param, true);

            $resultat_array = $o_exercise->evaluate($exercise_xml, 0, $solver_id, $gesamtpunktzahl, $answer);
            $pl_out = $resultat_array[6];
            $pl_score = $resultat_array[5];
            $pl_validity = $resultat_array[2];
        }

        $_POST['assignment_id']     = $assignment_id;
        $_POST['exercise_position'] = $exercise_position;
        $_POST['answer_0']          = $answer_0;
        $_POST['pl_query']          = vips_get_string_param('pl_query', true);
        $_POST['pl_out']            = $pl_out;
        $_POST['pl_score']          = $pl_score;
        $_POST['pl_validity']       = $pl_validity;
        $_POST['pl_count']          = $pl_count;
        unset($_GET['next']);  // to prevent show_exercise from displaying the next question
    }

    $query = "SELECT Punkte
        FROM $g_inAssignment_table
        WHERE $g_assignment_table = $assignment_id
            AND     vips_aufgabe = '$exercise_id'
        ";
    $result = $vips_db->executeQuery($query);
    $result->next();
    $gesamtpunktzahl = $result->getFloat('Punkte');

    #
    # cdeiwiks ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    ###################################################################################



    #################################
    # Store the commented solution. #
    #################################

    if (vips_get_submit_param('store_commented_solution'))
    {
        $commented_solution = vips_get_string_param('commented_solution', true);
        $sql = "UPDATE $g_solution_table
            SET `bearbeiteteLoesung` = '".$commented_solution."'
            WHERE `Loesungsid` = $solution_id
            ";
        $vips_db->executeQuery($sql);
    }



    ##################################
    # Delete the commented solution. #
    ##################################

    if (vips_get_submit_param('delete_commented_solution'))
    {
        $sql = "UPDATE $g_solution_table
            SET `bearbeiteteLoesung` = ''
            WHERE `Loesungsid` = $solution_id
            ";
        $vips_db->executeQuery($sql);
    }



    $query = "SELECT DISTINCT
            Korrektur AS LecturerComment
            , $g_solution_points_table.Punkte AS ReachedPoints
            , $g_inAssignment_table.Punkte AS MaxPoints
        FROM
            $g_inAssignment_table
            , $g_solution_table
            , $g_solution_points_table
        WHERE
            $g_assignment_table = $assignment_id
            AND vips_aufgabe = $exercise_id
            AND Position = $exercise_position
            AND Assignmentid = $assignment_id
            AND Aufgabenid = $exercise_id
            AND isLastSolution = 't'
            AND idLoesung = Loesungsid
            AND ".($assignment_type == "vips_klausur"||$single_solver ? "Userid" : "Gruppenid")." = '$solver_id'
        ";
    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
        $reached_points = $result->getFloat("ReachedPoints");
        $max_points = $result->getFloat("MaxPoints");
        $lecturer_comment = $result->get("LecturerComment");
    } else {
        // falls noch keine korrektur durchgefuehrt wurde
        $reached_points = 0;
        $max_points = $gesamtpunktzahl;
    }

    // don't show lecturer's form if ajax is just updating
    if (!vips_get_int_param('ajax_updating'))
    {
        // template for showing lecturer's correction form //
        $tmpl = $vipsTemplateFactory->open("solutions_edit_solution");
        $tmpl->set_attributes(array(
            "status"                   => $g_outlet->status,
            "message"                  => $message,
            "t_exercise_type"          => $exercise_type,
            "t_exercise_position"      => $exercise_position,
            "t_exercise_id"            => $exercise_id,
            "t_exercise_name"          => $exercise_name,
            "t_assignment_type"        => $assignment_type,
            "t_assignment_id"          => $assignment_id,
            "t_solver_id"              => $solver_id,
            'solver_type'              => $solver_type,
            "t_single_solver"          => $single_solver,
            "t_solver_name"            => $solver_name,
            "solution_id"              => $solution_id,
            "t_reached_points"         => $reached_points,
            "t_max_points"             => $max_points,
            "t_lecturer_comment"       => $lecturer_comment,
            "t_visible_for_student"    => $visible_for_student,
            "t_next_exercise_id"       => $next_exercise_id,
            "t_next_exercise_position" => $next_exercise_position,
            'next_solver_id'           => $next_solver_id,
            'next_single_solver'       => $next_single_solver,
            'next_solver_type'         => $next_solver_type,
            'next_name'                => $next_name,
            'view'                     => $view));
        setlocale(LC_NUMERIC, NULL);
        echo $tmpl->render();
        setlocale(LC_NUMERIC, 'C');
    }

    // show exercise body below the correction form //
    $_POST["exercise_id"]     = $exercise_id;
    $_POST["assignment_type"] = $assignment_type;
    $_POST["solver_id"]       = $solver_id;
    $_POST["single_solver"]   = $single_solver;
    $_POST['submitted']       = $submitted;

    if ($exercise_type == "rnd_exercise")
    {
        $_POST["assignment_id"]         = $assignment_id;
        $_POST["exercise_position"]     = $exercise_position;
        $_POST["in_rnd_exercise"]       = 1;
        $_POST["rnd_exercise_id"]       = $exercise_id;
        $_POST["rnd_exercise_name"]     = $exercise_name;
        $_POST["rnd_exercise_position"] = $exercise_position;
    }

    vips_assignments(($assignment_type == "vips_klausur" ? "exams" : "sheets"), "show_exercise");
}

/**
 * Stores the comment of the lecturer and the corrected points
 * for a solution.
 *
 * @date 6. november 2006
 * @author Martin Schrder
 */

function store_correction()
{
    global $vips_db, $vipsTemplateFactory;
    // Database variables
    global $g_outlet,
        $g_inAssignment_table,
        $g_solution_table,
        $g_solution_points_table,
        $g_assignment_table;

    $exercise_type       = vips_get_option_param("exercise_type");
    $exercise_id         = vips_get_int_param("exercise_id");
    $exercise_position   = vips_get_int_param("exercise_position");
    $assignment_id       = vips_get_int_param("assignment_id");
    $solution_id         = vips_get_int_param("solution_id");
    $lecturer_comment    = vips_get_string_param("lecturer_comment", true);
    $reached_points      = vips_get_float_param("reached_points");
    $visible_for_student = vips_get_int_param("visible_for_student");
    $max_points          = vips_get_float_param("max_points");
    $solver_id           = harmless(vips_get_string_param("solver_id"));
    $single_solver       = vips_get_int_param("single_solver");
    $solver_type         = vips_get_string_param('solver_type');
    $view                = vips_get_option_param('view');

    $assignment_type     = vips_get_option_param("assignment_type");
    $group_id            = vips_get_int_param("group_id");  // "Uebungsb."-mode
    $user_id             = vips_get_option_param("user_id");   // "vips_klausur"-mode


    // process lecturer's input
    $reached_points = round_to_half_point($reached_points);


    /*************************************************************************/
    /* Check whether given points of lecturer are higher than max points
    /* for an exercise.
    /*
    /* Lecturer may
    /*   a) correct the given points, or
    /*   b) give the extra points.
    /*
    /* If extra points are given, the formular simply calls the same method
    /* store_exercise_correction() with a modified max_points value and all
    /* other values as given before.
    /*************************************************************************/

    $tmpl = $vipsTemplateFactory->open("solutions_store_solution");
    $tmpl->set_attribute("picturepath", $g_outlet->picturepath);
    $tmpl->set_attribute("t_assignment_type", $assignment_type);
    $tmpl->set_attribute("t_assignment_id", $assignment_id);
    $tmpl->set_attribute('view', $view);

    // bonus points given //
    if ($max_points < $reached_points)
    {
            $tmpl->set_attribute("t_points_exceeded", 1);
            $tmpl->set_attribute("t_reached_points", $reached_points);
            $tmpl->set_attribute("t_max_points", $max_points);
    }

    // negative points given //
    if ($reached_points < 0)
    {
            $tmpl->set_attribute("t_points_negative", 1);
            $tmpl->set_attribute("t_reached_points", $reached_points);
            $tmpl->set_attribute("t_max_points", $max_points);
    }

    $visible_for_student = $visible_for_student == "" ? 'f' : 't';

    $query = "UPDATE
                    $g_solution_table
            SET
                    Korrektur = '$lecturer_comment'
                    , isAlreadyCorrected = 1
                    , visibleForStudent = '$visible_for_student'
            WHERE
                    Loesungsid = '$solution_id'
            ";
    $vips_db->executeUpdate($query);

    $query = "SELECT
                    `Punkte`
            FROM
                    $g_solution_points_table
            WHERE
                    `idLoesung` = '$solution_id'
            ";
    $result = $vips_db->executeQuery($query);

    // update database //
    if ($result->next())
    {
            if ($result->getFloat("Punkte") != $reached_points)
            {
                    $query = "UPDATE
                                    $g_solution_points_table
                            SET
                                    `Punkte` = '$reached_points'
                            WHERE
                                    `idLoesung` = '$solution_id'
                            ";
                    $vips_db->executeUpdate($query);
            }
    }

    // insert into database //
    else
    {
            $query = "INSERT INTO $g_solution_points_table
                            ( `idUser`
                            , `idLoesung`
                            , `Punkte`
                            , `Safe`
                            )
                    VALUES
                            ( '$solver_id'
                            , '$solution_id'
                            , '$reached_points'
                            , 'f'
                            )
                    ";
            $vips_db->executeUpdate($query);
    }

    // fetch ids of next exercises //
    $query = "SELECT
                    Aufgabenid, Position
            FROM
                    $g_inAssignment_table, $g_solution_table
            LEFT JOIN
                    $g_solution_points_table
            ON
                    $g_solution_points_table.idLoesung = $g_solution_table.Loesungsid
            WHERE
                    $g_inAssignment_table.vips_aufgabe = $g_solution_table.Aufgabenid
                    AND $g_solution_table.Assignmentid = '$assignment_id'
                    AND ".($assignment_type == "vips_klausur" || $single_solver ? "Userid" : "Gruppenid")." = '$solver_id'
                    AND isLastSolution = 't'
                    AND isAlreadyCorrected = '0'
                    AND (Safe IS NULL OR Safe = 'f')
                    AND Position > $exercise_position
            ORDER BY
                    Position
            ";
    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
        // assign next uncorrected exercise id and position
        $next_uncorrected_exercise_id = $result->get("Aufgabenid");
        $next_uncorrected_exercise_position = $result->get("Position");
    }

    $tmpl->set_attribute("next_uncorrected_exercise_id", $next_uncorrected_exercise_id);
    $tmpl->set_attribute("next_uncorrected_exercise_position", $next_uncorrected_exercise_position);
    $tmpl->set_attribute("solver_id", $solver_id);
    $tmpl->set_attribute("single_solver", $single_solver);
    $tmpl->set_attribute('solver_type', $solver_type);
    setlocale(LC_NUMERIC, NULL);
    echo $tmpl->render();
    setlocale(LC_NUMERIC, 'C');

    // show exercise and correction form again //
    edit_solution();
}



/**
 * Shows all corrected exercises of an exercise sheet that are set to
 * 1. visibleForStudent='t' in the database-table vips_K-Loesung/vips_T-Loesung, or
 * 2. korrekturenSichtbar='t' in the database-table vips_klausur/Uebungsblatt
 *
 * BUT: the entry in the database-table vips_klausur/vips_uebungsblatt dominates
 */

function show_student_assignment_solutions()
{
    global $g_outlet, $vips_db, $vipsTemplateFactory;

    // Database variables
    global $g_inAssignment_table,
        $g_solution_table,
        $g_solution_points_table,
        $g_assignment_table;

    $assignment = vips_get_int_param("assignment_id");
    $assignment_type = vips_get_option_param("assignment_type");

    // Security check - is assignment really freigegeben for students?
    $query = "SELECT `korrekturenSichtbar`, `Name`
        FROM $g_assignment_table
        WHERE ID = '$assignment'
            AND Kurs = '$g_outlet->courseID'
        ";
    $result = $vips_db->executeQuery($query);
    $result->next();
    if ($result->get("korrekturenSichtbar") != 't')
    {
        show_assignments_overview();
        return;
    }
    // end: security check

    // access allowed, get assignment name from the same query...
    $assignment_name = $result->get('Name');
    $korrekturenSichtbar = $result->get('korrekturenSichtbar');



    $sum_reached_points = 0;
    $sum_max_points     = 0;

    $user_id = $g_outlet->userID;

    $solver_id = $user_id;
    $single_solver = 1;

    if ($assignment_type == "vips_uebungsblatt")
    {
        // fetch group id where user is in from database
        $query = "SELECT
                Gruppenid
            FROM
                vips_gruppe,
                vips_inGruppe
            WHERE vips_gruppe.Kursid = '$g_outlet->courseID'
                AND vips_gruppe.Gruppenid = vips_inGruppe.group_id
                AND vips_inGruppe.user_id = '$user_id'
                AND vips_inGruppe.end IS NULL
            ";
        $result = $vips_db->executeQuery($query);

        // if user is member of a group, take it as solver id
        if ($result->next())
        {
            $single_solver = 0;
            $solver_id = $result->getInt('Gruppenid');
        }
        // otherwise, solver id remains user id
    }



    // get all exercises in the assignment
    $sql = "SELECT `vips_aufgabe`.`ID`,
            `vips_aufgabe`.`Name`,
            $g_inAssignment_table.`Punkte`,
            $g_inAssignment_table.`Position`
        FROM `vips_aufgabe`,
            $g_inAssignment_table,
            $g_assignment_table
        WHERE `vips_aufgabe`.`ID` = $g_inAssignment_table.`vips_aufgabe`
            AND $g_inAssignment_table.$g_assignment_table = $g_assignment_table.`ID`
            AND $g_assignment_table.`ID` = $assignment
            AND $g_assignment_table.`Kurs` = '$g_outlet->courseID'
        ORDER BY $g_inAssignment_table.`Position`
        ";
    $ex = $vips_db->executeQuery($sql);

    // for each exercise
    while ($ex->next())
    {
        $exercise_id       = $ex->get('ID');
        $exercise_name     = $ex->get('Name');
        $exercise_position = $ex->get('Position');
        $max_points        = $ex->getFloat('Punkte');
        $sum_max_points   += $max_points;

        // get solution
        $sql = "SELECT *
            FROM $g_solution_table,
                $g_solution_points_table
            WHERE $g_solution_table.`".($assignment_type == "vips_uebungsblatt" && !$single_solver ? "Gruppenid" : "Userid")."` = '$solver_id'
                AND `Aufgabenid` = $exercise_id
                AND `Assignmentid` = $assignment
                AND `isLastSolution` = 't'
                AND `Loesungsid` = $g_solution_points_table.`idLoesung`
            ";
        $sol = $vips_db->executeQuery($sql);
        if ($sol->next())  // solution exists
        {
            $reached_points      = $sol->getFloat('Punkte');
            $sum_reached_points += $reached_points;
            $manually_corrected  = $sol->getString('Korrektur') != '';
        }
        else  // no solution exists
        {
            $reached_points = 0;
        }

        $exercise_array[] = array(
            "exercise_id"         => $exercise_id,
            "exercise_name"       => $exercise_name,
            "exercise_position"   => $exercise_position,
            "reached_points"      => $reached_points,
            "corrections_visible" => $korrekturenSichtbar,
            "max_points"          => $max_points,
            'manually_corrected'  => $manually_corrected
        );
    }

    if ($sum_reached_points < 0)
        $sum_reached_points = 0;

    $tmpl = $vipsTemplateFactory->open("solutions_show_corrections");
    $tmpl->set_attribute("t_assignment_id", $assignment);
    $tmpl->set_attribute("t_assignment_type", $assignment_type);
    $tmpl->set_attribute("t_assignment_name", $assignment_name);
    $tmpl->set_attribute("t_exercise_array", $exercise_array);
    $tmpl->set_attribute("t_solver_id", $solver_id);
    $tmpl->set_attribute("t_single_solver", $single_solver);
    $tmpl->set_attribute("t_sum_reached_points", $sum_reached_points);
    $tmpl->set_attribute("t_sum_max_points", $sum_max_points);
    $tmpl->set_attribute("picturepath", $g_outlet->picturepath);
    setlocale(LC_NUMERIC, NULL);
    echo $tmpl->render();
    setlocale(LC_NUMERIC, 'C');
} // end function







###############################################
#                                             #
#   C O M P U T E   F I N A L   G R A D E S   #
#                                             #
#     E N D N O T E N B E R E C H N U N G     #
#                                             #
###############################################



/**
 * Display the course results of the current student.
 */
function show_final_grades_student()
{
    global $g_outlet;
    show_final_grades($g_outlet->userID);
}

/**
 * Displays a table of all users with all their blocks', sheets' and exams' marks
 * plus their final grade
 */
function show_final_grades($user_id = NULL) {
    global $vips_db, $g_outlet, $vipsTemplateFactory;

    $format = vips_get_option_param('format');

    if ($format == 'csv')
    {
        $t_main = $vipsTemplateFactory->open('solutions_final_grades_csv');

        // this hack is necessary to disable the standard Stud.IP layout
        ob_end_clean();
        header('Content-Type: text/csv; charset=ISO-8859-1');
        header('Content-Disposition: attachment; filename="'.'Notenliste'.'.csv"');
        header('Pragma: public');
    }
    else if (!$user_id)  // template shows all participants to lecturer
    {
        $t_main = $vipsTemplateFactory->open('solutions_final_grades');
    }
    else  // template shows only one single participant to the student self
    {
        $t_main = $vipsTemplateFactory->open('solutions_final_grade_student');
    }
    $t_main->set_attribute('picturepath', $g_outlet->picturepath);

    ###########################################
    # get all blocks, "free" sheets and exams #
    ###########################################

    #####################################
    # get used blocks and their weights #

    $now = date('Y-m-d H:i:s');
    $all_weights = array();    //if there's no number different from 0 in all_weights, don't calculate with weights

    $blocks = array();
    $query = "SELECT vips_block.*, vips_gewichtung.Gewichtung".
             " FROM vips_block LEFT JOIN vips_gewichtung ON id = Item_id AND Item_type = 'blocks'".
             " WHERE Kurs = '$g_outlet->courseID'";
    $result = $vips_db->executeQuery($query);

    while ($result->next()) {
        $entry = $result->getRow();
        $block_id = $result->get('id');
        $weight = $result->getFloat('Gewichtung');

        if ($weight == NULL)
            $weight = 0;

        $entry['weight'] = $weight;
        $all_weights[] = $weight;

        $result2 = $vips_db->executeQuery("SELECT * FROM vips_inBlock WHERE block_id = '$block_id'");
        if ($result2->getRecordCount() > 0)
            $blocks[] = $entry;
    }

    ###################################################
    # ... and remaining assignments and their weights #

    // fetch (expired!) exercise sheets
    $sheets = array();
    $query = "SELECT vips_uebungsblatt.*, vips_gewichtung.Gewichtung".
             " FROM vips_uebungsblatt LEFT JOIN vips_gewichtung ON ID = Item_id AND Item_type = 'sheets'".
             " LEFT JOIN vips_inBlock ON ID = assignment_id".
             " WHERE Kurs = '$g_outlet->courseID' AND block_id IS NULL".
             " ORDER BY Beginn, Position";
    $result = $vips_db->executeQuery($query);

    while ($result->next()) {
        $entry = $result->getRow();
        $weight = $result->getFloat('Gewichtung');

        if ($weight == NULL)
            $weight = 0;

        $entry['weight'] = $weight;
        $all_weights[] = $weight;

        if (strcmp($result->get('Ende'), $now) < 0)
            $sheets[] = $entry;
    }

    $groups = array();
    $query = "SELECT vips_klausur_gruppierung.*, k1.Name AS k1_name, k2.Name AS k2_name, vips_gewichtung.Gewichtung".
             " FROM vips_klausur AS k1, vips_klausur AS k2, vips_klausur_gruppierung LEFT JOIN vips_gewichtung".
             " ON vips_klausur_gruppierung.id = Item_id AND Item_type = 'groups'".
             " WHERE k1 = k1.ID AND k2 = k2.ID AND vips_klausur_gruppierung.kurs='$g_outlet->courseID'";
    $result = $vips_db->executeQuery($query);

    while ($result->next()) {
        $entry = $result->getRow();
        $weight = $result->getFloat('Gewichtung');

        if ($weight == NULL)
            $weight = 0;

        $entry['weight'] = $weight;
        $groups[] = $entry;
    }

    // fetch (expired!) exams
    $exams = array();
    $query = "SELECT vips_klausur.*, vips_gewichtung.Gewichtung".
             " FROM vips_klausur LEFT JOIN vips_gewichtung ON vips_klausur.ID = Item_id AND Item_type = 'exams'".
             " LEFT JOIN vips_klausur_gruppierung ON vips_klausur.ID = k1 OR vips_klausur.ID = k2".
             " WHERE vips_klausur.Kurs = '$g_outlet->courseID' AND vips_klausur_gruppierung.id IS NULL".
             " ORDER BY Beginn, Position";
    $result = $vips_db->executeQuery($query);

    while ($result->next()) {
        $entry = $result->getRow();
        $weight = $result->getFloat('Gewichtung');

        if ($weight == NULL)
            $weight = 0;

        $entry['weight'] = $weight;
        $all_weights[] = $weight;

        if (strcmp($result->get('Ende'), $now) < 0)
            $exams[] = $entry;
    }

    $number_of_items = count($blocks)+count($sheets)+count($exams)+count($groups); //for number of columns

    // Does the lecturer use weighting?
    $weighting = array_sum($all_weights) != 0 ? true : false;



    ###############################
    #  get course participant(s)  #
    ###############################

    $course_participants = array();

    // If the function has been called without the $user_id parameter get *all*
    // course participants, else get *only this* participant.
    $query = "SELECT seminar_user.*,
            auth_user_md5.username,
            auth_user_md5.Vorname,
            auth_user_md5.Nachname,
            auth_user_md5.perms
        FROM seminar_user,
            auth_user_md5
        WHERE seminar_user.Seminar_id = '$g_outlet->courseID'
            AND seminar_user.user_id = auth_user_md5.user_id
            AND seminar_user.status = 'autor'
            ".($user_id ? "AND `seminar_user`.`user_id` = '$user_id'" : "")."
        ORDER BY Nachname, Vorname
        ";
    $result = $vips_db->executeQuery($query);

    #################################
    #  for each course participant  #
    #################################

    while ($result->next()) {

        $course_participant = $result->getRow();
        $user_id = $course_participant['user_id'];

        if ($format == 'csv') {
            $course_participant['stud_id'] = get_student_id($user_id);
        }

        ####################
        #  for each block  #
        ####################

        $blocks_percentages_weighted = array();
        $blocks_percentages_unweighted = array();

        $course_participant['blocks_points_reached'] = array();
        $course_participant['blocks_points_max'] = array();

        foreach ($blocks as $block) {

            $block_id = $block['id'];
            $block_weight = $block['weight'];

            $points_block_user = 0;

            // fetch (expired!) exercise sheets in block
            $sql = "SELECT vips_uebungsblatt.*
                FROM vips_inBlock,
                    vips_uebungsblatt
                WHERE vips_inBlock.block_id = '$block_id'
                    AND vips_uebungsblatt.id = vips_inBlock.assignment_id
                    AND `vips_uebungsblatt`.`Ende` < '".date('Y-m-d H:i:s')."'
                ";
            $result3 = $vips_db->executeQuery($sql);

            ################################
            # for all sheets in this block #
            ################################

            while ($result3->next()) {
                $sheet = $result3->getRow();
                $sheet_end = $sheet['Ende'];

                ##############################################################
                # get group which the student belonged to at Abgabezeitpunkt #
                # of sheet                                                   #

                $group_id = -1;

                $query = "SELECT `group_id`, `end`
                    FROM `vips_inGruppe`, `vips_gruppe`
                    WHERE `user_id` = '$user_id'
                        AND `Kursid` = '$g_outlet->courseID'
                        AND `Gruppenid` = `group_id`
                    ";
                $result4 = $vips_db->executeQuery($query);

                while ($result4->next()) {  // user changed group
                    $group       = $result4->getRow();
                    $group_start = $group['start'];
                    $group_end   = $group['end'];

                    if ($sheet_end >= $group_start &&
                        ($group_end > $sheet_end || $group_end == NULL))
                    {
                        $group_id = $group['group_id'];
                        break;
                    }
                }

                ###########################
                # get solution and points #

                $sql = "
                    SELECT DISTINCT
                        `vips_T_Loesung`.`Loesungsid`,
                        `vips_T_Loesung_Punkte`.`Punkte`
                    FROM
                        `vips_uebungsblatt`,
                        `vips_T_Loesung`,
                        `vips_T_Loesung_Punkte`
                    WHERE
                        `vips_uebungsblatt`.`Kurs` = '$g_outlet->courseID'
                        AND `vips_uebungsblatt`.`ID` = '".$sheet['ID']."'
                        AND `vips_uebungsblatt`.`ID` = `vips_T_Loesung`.`Assignmentid`
                        AND `vips_T_Loesung_Punkte`.`idLoesung` = `vips_T_Loesung`.`Loesungsid`
                        AND `vips_T_Loesung`.`isLastSolution` = 't'
                        AND `vips_T_Loesung`.".($group_id == -1 ? "`Userid` = '$user_id'" : "`Gruppenid` = '$group_id'")."
                    ";
                $result2 = $vips_db->executeQuery($sql);

                $points_sheet = 0;
                while ($result2->next()) {
                    $points_sheet += $result2->getFloat('Punkte');
                }
                $points_sheet = max($points_sheet, 0);
                $points_block_user += $points_sheet;
            }
            $course_participant['blocks_points_reached'][] = $points_block_user;

            ######################
            #  get total points  #

            $result2 = $vips_db->executeQuery("SELECT vips_inUebungsblatt.*
                FROM vips_inUebungsblatt,
                    vips_uebungsblatt,
                    vips_inBlock
                WHERE   vips_uebungsblatt.kurs          = '$g_outlet->courseID'
                    AND vips_uebungsblatt.id            = vips_inUebungsblatt.vips_uebungsblatt
                    AND vips_inBlock.block_id           = '$block_id'
                    AND vips_inBlock.assignment_id      = vips_uebungsblatt.id
                ");
            $points_block = 0;
            while ($result2->next()) {
                $points_block += $result2->getFloat('Punkte');
            }
            $course_participant['blocks_points_max'][] = $points_block;

            ################################
            # calculate percentage of user #

            if (!$weighting)
                $block_weight = 100;

            if ($points_block > 0) {
                $blocks_percentages_weighted[] = roundLikeFloor((($points_block_user/$points_block)*$block_weight),2);
                $blocks_percentages_unweighted[] = roundLikeFloor(($points_block_user/$points_block)*100,2);
            } else {
                $blocks_percentages_weighted[] = $block_weight;
                $blocks_percentages_unweighted[] = 100;
            }
        }



        ############################################
        #  for each (remaining) sheet (do the same)#
        ############################################

        $sheets_percentages_weighted = array();
        $sheets_percentages_unweighted = array();

        $course_participant['sheets_points_reached'] = array();
        $course_participant['sheets_points_max'] = array();

        foreach ($sheets as $sheet) {

                $sheet_id = $sheet['ID'];
                $sheet_weight = $sheet['weight'];

                $sheet_end = $sheet['Ende'];

                #######################################################################
                # get group which the student belonged to at Abgabezeitpunkt of sheet #
                $course_participant['group_id'] = -1;

            $query = "SELECT `group_id`, `end`
                FROM `vips_inGruppe`, `vips_gruppe`
                WHERE `user_id` = '$user_id'
                    AND `Kursid` = '$g_outlet->courseID'
                    AND `Gruppenid` = `group_id`
                    ";
                        $result4 = $vips_db->executeQuery($query);

                        while ($result4->next()) { //user changed group
                                $group       = $result4->getRow();
                                $group_start = $group['start'];
                                $group_end   = $group['end'];

                                if  ($sheet_end >= $group_start &&
                                        ($group_end > $sheet_end || $group_end == NULL))
                                {
                                        $course_participant['group_id'] = $group['group_id'];
                                        break;
                                }
                        }

                        ###########################
                        # get solution and points #

                        $group_id = $course_participant['group_id'];

            $sql = "SELECT DISTINCT `vips_T_Loesung`.`Loesungsid`,
                    `vips_T_Loesung_Punkte`.`Punkte`
                FROM `vips_T_Loesung_Punkte`,
                    `vips_T_Loesung`,
                    `vips_uebungsblatt`
                WHERE `vips_uebungsblatt`.`Kurs` = '$g_outlet->courseID'
                    AND `vips_uebungsblatt`.`id` = `vips_T_Loesung`.`Assignmentid`
                    AND `vips_T_Loesung`.`isLastSolution` = 't'
                    AND `vips_T_Loesung`.".($group_id == -1 ? "`Userid` = '$user_id'" : "`Gruppenid` = $group_id")."
                    AND `vips_T_Loesung`.`Loesungsid` = `vips_T_Loesung_Punkte`.`idLoesung`
                    AND `vips_uebungsblatt`.`id` = $sheet_id
                ";
                        $result2 = $vips_db->executeQuery($sql);

                        $points_sheet_user = 0;

                        while ($result2->next()) {
                                $points_sheet_user += $result2->getFloat('Punkte');
                        }
                        $course_participant['sheets_points_reached'][] = $points_sheet_user;

                        ######################
                        #  get total points  #

                        $result2 = $vips_db->executeQuery("SELECT vips_inUebungsblatt.*
                                                            FROM vips_inUebungsblatt,
                                                                     vips_uebungsblatt
                                                            WHERE   vips_uebungsblatt.kurs          = '$g_outlet->courseID'
                                                            AND             vips_uebungsblatt.id            = vips_inUebungsblatt.vips_uebungsblatt
                                                            AND             vips_uebungsblatt.id = '$sheet_id'");
                        $points_sheet = 0;

                        while ($result2->next()) {
                                $points_sheet += $result2->getFloat('Punkte');
                        }
                        $points_sheet = max($points_sheet, 0);
                        $course_participant['sheets_points_max'][] = $points_sheet;

                        if (!$weighting)
                            $sheet_weight = 100;

                        if ($points_sheet > 0) {
                            $sheets_percentages_weighted[] = roundLikeFloor((($points_sheet_user/$points_sheet)*$sheet_weight),2);
                            $sheets_percentages_unweighted[] = roundLikeFloor(($points_sheet_user/$points_sheet)*100,2);
                        } else {
                            $sheets_percentages_weighted[] = $sheet_weight;
                            $sheets_percentages_unweighted[] = 100;
                        }
                }


        ##################
        #  for each exam #
        ##################

        $exams_percentages_weighted = array();
        $exams_percentages_unweighted = array();

        $course_participant['exams_points_reached'] = array();
        $course_participant['exams_points_max'] = array();

        foreach ($exams as $exam) {

            $exam_id = $exam['ID'];
            $exam_weight = $exam['weight'];

            #####################
            # get Bestehen info #

            $sql = "SELECT `Bestehen` FROM vips_klausur WHERE ID = '$exam_id';";
            $result2 = $vips_db->executeQuery($sql);
            $result2->next();
            $exam_bestehen = $result2->getInt('Bestehen');

            #####################
            #  get user points  #

            $sql = "SELECT DISTINCT vips_K_Loesung_Punkte.*
                FROM vips_K_Loesung_Punkte,
                    vips_klausur,
                    vips_K_Loesung
                WHERE   vips_klausur.Kurs             = '$g_outlet->courseID'
                    AND vips_klausur.id               = vips_K_Loesung.Assignmentid
                    AND vips_K_Loesung.isLastSolution = 't'
                    AND vips_K_Loesung.Userid         = '$user_id'
                    AND vips_K_Loesung.Loesungsid     = vips_K_Loesung_Punkte.idLoesung
                    AND vips_klausur.id               = '$exam_id'
                ";
                        $result2 = $vips_db->executeQuery($sql);

                        $points_exam_user = 0;

                        while ($result2->next()) {
                                $points_exam_user += $result2->getFloat('Punkte');
                        }
                        $course_participant['exams_points_reached'][] = $points_exam_user;

                        ######################
                        #  get total points  #

                        $result2 = $vips_db->executeQuery("SELECT vips_inKlausur.*
                                                                                FROM vips_inKlausur,
                                                                                         vips_klausur
                                                                                WHERE   vips_klausur.kurs       = '$g_outlet->courseID'
                                                                                AND             vips_klausur.id         = vips_inKlausur.vips_klausur
                                                                                AND             vips_klausur.id = '$exam_id'");
                        $points_exam = 0;

                        while ($result2->next()) {
                                $points_exam += $result2->getFloat('Punkte');
                        }
                        if ($points_exam < 0) $points_exam = 0;
                        $course_participant['exams_points_max'][] = $points_exam;

                        ########################
                        #  check if bestanden  #

                        if (!$weighting)
                            $exam_weight = 100;

                        if ($points_exam > 0) {
                            $exam_user_percentage_weighted   = roundLikeFloor((($points_exam_user/$points_exam)*$exam_weight),2);
                            $exam_user_percentage_unweighted = roundLikeFloor((($points_exam_user/$points_exam)*100),2);
                        } else {
                            $exam_user_percentage_weighted   = $exam_weight;
                            $exam_user_percentage_unweighted = 100;
                        }

                        if ($exam_bestehen <= $exam_user_percentage_unweighted) {           //if passed
                            $exams_percentages_weighted[]   = $exam_user_percentage_weighted;
                            $exams_percentages_unweighted[] = $exam_user_percentage_unweighted;
                        }
                        else {
                            $exams_percentages_weighted[]   = 0;
                            $exams_percentages_unweighted[] = 0;
                        }
        }

                ##########################
                #  for each grouped exam #
                ##########################

                $groups_percentages_weighted = array();
                $groups_percentages_unweighted = array();

                $course_participant['groups_points_reached'] = array();
                $course_participant['groups_points_max'] = array();

                foreach ($groups as $group) {

                        $group_id = $group['id'];
                        $group_weight = $group['weight'];

                        $k1 = $group['k1'];
                        $k2 = $group['k2'];

                        ##############################
                        # get Bestehen info of k1/k2 #

                        $sql = "SELECT * FROM vips_klausur WHERE ID = '$k1'";
                        $result2 = $vips_db->executeQuery($sql);
                        $result2->next();
                        $k1_bestehen = $result2->getFloat('Bestehen');

                        $sql = "SELECT * FROM vips_klausur WHERE ID = '$k2'";
                        $result2 = $vips_db->executeQuery($sql);
                        $result2->next();
                        $k2_bestehen = $result2->getFloat('Bestehen');

                        ##########################
                        #  get user points k1/k2 #

                        $sql = "SELECT vips_K_Loesung_Punkte.*
                                FROM vips_K_Loesung_Punkte,
                                         vips_klausur,
                                         vips_K_Loesung
                                WHERE   vips_klausur.Kurs               = '$g_outlet->courseID'
                                AND             vips_klausur.id                 = vips_K_Loesung.Assignmentid
                                AND             vips_K_Loesung.isLastSolution='t'
                                AND             vips_K_Loesung.Userid           = '$user_id'
                                AND             vips_K_Loesung.Loesungsid       = vips_K_Loesung_Punkte.idLoesung
                                AND             vips_klausur.id = '$k1'";
                        $result2 = $vips_db->executeQuery($sql);

                        $points_k1_user = 0;

                        while ($result2->next()) {
                                $points_k1_user += $result2->getFloat('Punkte');
                        }
                        if ($points_k1_user < 0) $points_k1_user = 0;

                        $sql = "SELECT vips_K_Loesung_Punkte.*
                                FROM vips_K_Loesung_Punkte,
                                         vips_klausur,
                                         vips_K_Loesung
                                WHERE   vips_klausur.Kurs               = '$g_outlet->courseID'
                                AND             vips_klausur.id                 = vips_K_Loesung.Assignmentid
                                AND             vips_K_Loesung.isLastSolution='t'
                                AND             vips_K_Loesung.Userid           = '$user_id'
                                AND             vips_K_Loesung.Loesungsid       = vips_K_Loesung_Punkte.idLoesung
                                AND             vips_klausur.id = '$k2'";
                        $result2 = $vips_db->executeQuery($sql);

                        $points_k2_user = 0;

                        while ($result2->next()) {
                            $points_k2_user += $result2->getFloat('Punkte');
                        }
                        if ($points_k2_user < 0) $points_k2_user = 0;

                        $course_participant['groups_points_reached'][] = $points_k1_user + $points_k2_user;

                        ###########################
                        #  get total points k1/k2 #

                        $result2 = $vips_db->executeQuery("SELECT vips_inKlausur.*
                                                                                FROM vips_inKlausur,
                                                                                         vips_klausur
                                                                                WHERE   vips_klausur.kurs       = '$g_outlet->courseID'
                                                                                AND             vips_klausur.id         = vips_inKlausur.vips_klausur
                                                                                AND             vips_klausur.id = '$k1'");
                        $points_k1 = 0;

                        while ($result2->next()) {
                                $points_k1 += $result2->getFloat('Punkte');
                        }

                        $result2 = $vips_db->executeQuery("SELECT vips_inKlausur.*
                                                                                FROM vips_inKlausur,
                                                                                         vips_klausur
                                                                                WHERE   vips_klausur.kurs       = '$g_outlet->courseID'
                                                                                AND             vips_klausur.id         = vips_inKlausur.vips_klausur
                                                                                AND             vips_klausur.id = '$k2'");
                        $points_k2 = 0;

                        while ($result2->next()) {
                                $points_k2 += $result2->getFloat('Punkte');
                        }

                        $course_participant['groups_points_max'][] = $points_k1 + $points_k2;

                        ###########################################################
                        # calculate correct percentage using auswertungsparameter #

                        if ($points_k1 > 0) {
                            $k1_user_percentage = roundLikeFloor((($points_k1_user/$points_k1)*100),2);
                        } else {
                            $k1_user_percentage = 100;
                        }
                        if ($points_k2 > 0) {
                            $k2_user_percentage = roundLikeFloor((($points_k2_user/$points_k2)*100),2);
                        } else {
                            $k2_user_percentage = 100;
                        }

                        if ($k1_bestehen > $k1_user_percentage) {
                            $fail_k1 = true;
                            $k1_user_percentage = 0;
                        }
                        if ($k2_bestehen > $k2_user_percentage) {
                            $fail_k2 = true;
                            $k2_user_percentage = 0;
                        }

                        $percentage = 0;
                        if ($fail_k1) {                                         //user has failed k1
                                if (!$fail_k2) {                                //user has passed k2
                                        if ($group['fail_k1'] == 'bestehen_k2')   //lecturer decides what happens:either student gets
                                                $percentage = $k2_bestehen;                       // k2 pass percentage (corresponds 4,0) or his/her own
                                        else $percentage = $k2_user_percentage;   // percentage of k2
                                }
                        }
                        else {
                                if ($group['pass_k1'] == 'max_k1_k2')
                                        $percentage = max($k1_user_percentage,$k2_user_percentage);
                                else $percentage = $k2_user_percentage;
                        }

                        if (!$weighting)
                            $group_weight = 100;

                        $groups_percentages_weighted[] = roundLikeFloor($percentage*($group_weight/100),2);
                        $groups_percentages_unweighted[] = $percentage;
                }


        ###################################################
        # check whether any assignments are still running #
        ###################################################

        $now = date('Y-m-d H:i:s');
        $sql = "SELECT `ID`
            FROM `vips_uebungsblatt`
            WHERE `Kurs` = '$g_outlet->courseID'
                AND `Ende` > '$now'
            UNION
            SELECT `ID`
            FROM `vips_klausur`
            WHERE `Kurs` = '$g_outlet->courseID'
                AND `Ende` > '$now'
            ";
        $ids = $vips_db->executeQuery($sql);

        $all_assignments_expired = $ids->getRecordCount() == 0 ? true : false;



        ##############################
        # calculate total percentage #
        ##############################

        $percentage_final = 0;
        foreach ($blocks_percentages_weighted as $perc)
            $percentage_final += $perc;
        foreach ($sheets_percentages_weighted as $perc)
            $percentage_final += $perc;
        foreach ($exams_percentages_weighted as $perc)
            $percentage_final += $perc;
        foreach ($groups_percentages_weighted as $perc)
            $percentage_final += $perc;

        if (!$weighting)
            $percentage_final = roundLikeFloor($percentage_final / $number_of_items, 2);

        #########################
        # calculate final grade #
        #########################

        $query = "SELECT Prozent, Note
                    FROM vips_noten
                    WHERE Kurs = '$g_outlet->courseID'
                        AND Prozent != ".UNUSED_GRADE."
                    ORDER BY Prozent";
        $result4 = $vips_db->executeQuery($query);

        if ($result4->getRecordCount()) {
            // calculate final grade
            $grade_final = '5,0';

            while ($result4->next() && $percentage_final >= $result4->getFloat('Prozent')) {
                $grade_final = $result4->get('Note');
            }

            // FIXME this mapping needs to be configurable
            if ($grade_final <= '1,5') {
                $ects_grade = 'A';
            } else if ($grade_final <= '2,0') {
                $ects_grade = 'B';
            } else if ($grade_final <= '3,0') {
                $ects_grade = 'C';
            } else if ($grade_final <= '3,5') {
                $ects_grade = 'D';
            } else if ($grade_final <= '4,0') {
                $ects_grade = 'E';
            } else {
                $ects_grade = 'F';
            }
        } else {
            $grade_final = '-';
            $ects_grade  = '-';
        }

        $course_participant['blocks_percentages_weighted'] = $blocks_percentages_weighted;
        $course_participant['sheets_percentages_weighted'] = $sheets_percentages_weighted;
        $course_participant['exams_percentages_weighted']  = $exams_percentages_weighted;
        $course_participant['groups_percentages_weighted'] = $groups_percentages_weighted;

        $course_participant['blocks_percentages_unweighted'] = $blocks_percentages_unweighted;
        $course_participant['sheets_percentages_unweighted'] = $sheets_percentages_unweighted;
        $course_participant['exams_percentages_unweighted']  = $exams_percentages_unweighted;
        $course_participant['groups_percentages_unweighted'] = $groups_percentages_unweighted;

        $course_participant['points_exercises']     = array_sum($course_participant['blocks_points_reached']) +
                                                      array_sum($course_participant['sheets_points_reached']);
        $course_participant['points_exercises_max'] = array_sum($course_participant['blocks_points_max']) +
                                                      array_sum($course_participant['sheets_points_max']);
        $course_participant['percentage_final']     = $percentage_final;
        $course_participant['grade_final']          = $grade_final;
        $course_participant['ects_grade']           = $ects_grade;

        $course_participants[] = $course_participant;
    } // endforeach participant


    ####################
    # sort if required #
    ####################

    function compare_grade ($a, $b) {
        return strcmp($a['grade_final'], $b['grade_final']);
    }

    function compare_percentage ($a, $b) {
        $diff = $b['percentage_final'] - $a['percentage_final'];

        // note: comparison must return integer value
        return $diff > 0 ? 1 : ($diff < 0 ? -1 : 0);
    }

    function compare_points($a, $b) {
        return $b['points_exercises'] - $a['points_exercises'];
    }

    $order_by = vips_get_option_param('order_by');

    if ($order_by == 'grade_final') {
        usort($course_participants, 'compare_grade');
    } else if ($order_by == 'percentage_final') {
        usort($course_participants, 'compare_percentage');
    } else if ($order_by == 'points_exercises') {
        usort($course_participants, 'compare_points');
    }

    $t_main->set_attribute('course_participants', $course_participants);
    $t_main->set_attribute('blocks', $blocks);
    $t_main->set_attribute('sheets', $sheets);
    $t_main->set_attribute('exams', $exams);
    $t_main->set_attribute('groups', $groups);
    $t_main->set_attribute('number_of_items', $number_of_items);
    $t_main->set_attribute('weighting', $weighting);
    $t_main->set_attribute('all_assignments_expired', $all_assignments_expired);
    $t_main->set_attribute('order_by', $order_by);
    setlocale(LC_NUMERIC, NULL);
    echo $t_main->render();
    setlocale(LC_NUMERIC, 'C');

    if ($format == 'csv')
    {
        // this hack is necessary to disable the standard Stud.IP layout
        ob_start('discard_buffer');
    }
}

?>
