<?php
/*
 * vips_wiki.inc.php - Vips plugin for Stud.IP
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */
 
// for StudipContentmoduleHelper::find_seminars_using_contentmodule()
require_once 'lib/webservices/api/studip_contentmodule.php';

function vips_wiki ($subaction)
{
    global
        $g_inAssignment_table,
        $g_assignment_table,
        $g_solution_table,
        $g_solution_points_table;

    if ($action != 'exams') {
        $g_assignment_table      = 'vips_uebungsblatt';
        $g_inAssignment_table    = 'vips_inUebungsblatt';
        $g_solution_table        = 'vips_T_Loesung';
        $g_solution_points_table = 'vips_T_Loesung_Punkte';
    } else {  // $action == 'exams'
        $g_assignment_table      = 'vips_klausur';
        $g_inAssignment_table    = 'vips_inKlausur';
        $g_solution_table        = 'vips_K_Loesung';
        $g_solution_points_table = 'vips_K_Loesung_Punkte';
    }

    if ($subaction == 'show') {
        vips_wiki_show();
    } else if ($subaction == 'submit') {
        vips_wiki_submit();
    } else if ($subaction == 'import_shortquiz') {
        vips_wiki_import_shortquiz();
    } else if ($subaction == 'exercise_id') {
        vips_wiki_exercise_id();
    } else {
        vips_wiki_show();
    }
}

/*
 * This currently is almost the same as formatReady().
 */
function vips_wiki_format ($string, $trim = true)
{
    return formatReady($string, $trim, true);
}

/*
 * Set current user id to an authid from the wiki.
 */
function vips_wiki_set_user ($authid)
{
    global $STUDIP_INSTALLATION_ID, $g_outlet;

    if (preg_match('/^[\\w@-]+$/', $authid)) {
        $g_outlet->userID = $authid;

        if (preg_match('/(.+)@(.+)/', $authid, $matches)) {
            if ($matches[2] == $STUDIP_INSTALLATION_ID) {
                $g_outlet->userID = get_userid($matches[1]);
            }
        }
    }

    return $g_outlet->userID;
}

function vips_wiki_check_host_access ($system_id)
{
    $wiki_config = $GLOBALS['ELEARNING_INTERFACE_MODULES'][$system_id];
    $wiki_addrs = array();

    if (isset($wiki_config)) {
        $wiki_url = parse_url($wiki_config['ABSOLUTE_PATH_ELEARNINGMODULES']);
        $wiki_hosts = $wiki_config['host_aliases'];
        $wiki_hosts[] = $wiki_url['host'];
        $wiki_addrs = array_map('gethostbyname', $wiki_hosts);
    }

    return in_array($_SERVER['REMOTE_ADDR'], $wiki_addrs);
}

function vips_wiki_check_access ($system_id, $module_id, $exercise_id)
{
    global $vips_db, $g_inAssignment_table, $g_assignment_table;

    if (empty($system_id) || empty($module_id))
        return false;

    if (!vips_wiki_check_host_access($system_id))
        return false;

    $courses = StudipContentmoduleHelper::find_seminars_using_contentmodule($system_id, $module_id);

    $sql = "SELECT vips_aufgabe
                FROM $g_inAssignment_table, $g_assignment_table
            WHERE
                $g_inAssignment_table.$g_assignment_table = $g_assignment_table.ID AND
                $g_inAssignment_table.vips_aufgabe = '$exercise_id' AND
                $g_assignment_table.Kurs IN ('".join("','", $courses)."')";

    $result = $vips_db->executeQuery($sql);

    return $result->getRecordCount() != 0;
}

function vips_wiki_show ()
{
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table,
        $g_solution_table,
        $g_solution_points_table,
        $vipsTemplateFactory;

    $system_id     = vips_get_string_param('system_id');
    $module_id     = vips_get_string_param('module_id');

    $assignment_id     = vips_get_int_param('assignment_id');
    $exercise_position = vips_get_int_param('exercise_position');
    $exercise_id       = vips_get_int_param('exercise_id');

    $solver_id      = vips_get_string_param("solver_id");
    $solver_user_id = vips_wiki_set_user($solver_id);
    $single_solver  = true;

    $is_solved     = false;

    #####################################
    #       get exercise info           #
    #####################################

    if (isset($exercise_id)) {
        // fetch exercise info, type and points, determined by exercise_id
        $query = "SELECT DISTINCT
                vips_aufgabe.*,
                vips_aufgaben_typen.Typenname,
                $g_inAssignment_table.Position,
                $g_inAssignment_table.Punkte,
                $g_inAssignment_table.$g_assignment_table
            FROM
                vips_aufgabe,
                vips_aufgaben_typen,
                $g_inAssignment_table
            WHERE
                vips_aufgabe.ID = $exercise_id
                AND $g_inAssignment_table.vips_aufgabe    = $exercise_id
                AND vips_aufgaben_typen.URI               = vips_aufgabe.URI
            ";
    } else {
        // fetch exercise info, type and points, determined by assignment and position
        $query = "SELECT DISTINCT
                vips_aufgabe.*,
                vips_aufgaben_typen.Typenname,
                $g_inAssignment_table.Position,
                $g_inAssignment_table.Punkte,
                $g_inAssignment_table.$g_assignment_table
            FROM
                vips_aufgabe,
                vips_aufgaben_typen,
                $g_inAssignment_table
            WHERE
                $g_inAssignment_table.$g_assignment_table = '$assignment_id'
                AND $g_inAssignment_table.vips_aufgabe    = vips_aufgabe.ID
                AND $g_inAssignment_table.Position        = '$exercise_position'
                AND vips_aufgaben_typen.URI               = vips_aufgabe.URI
            ";
    }

    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
        $exercise_XML        = $result->get('Aufgabe');
        $exercise_name       = $result->get('Name');
        $exercise_id         = $result->get('ID');
        $exercise_type       = $result->get('URI');
        $assignment_id       = $result->getInt($g_assignment_table);
        $exercise_position   = $result->getInt('Position');
        $exercise_title      = $result->get('Typenname');
        // $exercise_max_points = $result->getFloat('Punkte');
    } else {
        // TODO: improve error message here
        echo '<p><b>Keine Aufgabe gefunden!</b></p>';
        return;
    }

    if (!vips_wiki_check_access($system_id, $module_id, $exercise_id)) {
        // TODO: improve error message here
        echo '<p><b>Zugriff verweigert!</b></p>';
        return;
    }

    /******************************************/
    require_once "exercises/$exercise_type.php";

    $o_exercise = new $exercise_type($exercise_XML);
    /******************************************/

    ###################################
    # get user solution if applicable #
    ###################################

    // check if there is already a user's solution OR a group solution

    $solver_field = 'Userid';

    $sql = "SELECT * FROM $g_solution_table
                    WHERE Aufgabenid      = '$exercise_id'
                     AND  Assignmentid    = '$assignment_id'
                     AND  $solver_field   = '$solver_user_id'
                     AND  isLastSolution  = 't'";
    $result2 = $vips_db->executeQuery($sql);

    if ($result2->next()) {
        $is_solved          = true;
        $solution           = $result2->getRow();
        $solution_XML       = $solution['Loesung'];
        //vv important: student_array with complete user solution vv
        $student_array      = $o_exercise->getTagsFromXML($solution_XML, "answer");

        $student_comment    = $solution['Anmerkung'];
        $solution_id        = $solution['Loesungsid'];
        $lecturer_comment   = $solution['Korrektur'];
        $commented_solution = $solution['bearbeiteteLoesung'];
        $is_corrected       = $solution['isAlreadyCorrected'] == 1;
    }

    // selftest
    $sql = "SELECT Selbsttest FROM $g_assignment_table WHERE ID = '$assignment_id'";
    $result = $vips_db->executeQuery($sql);

    if ($result->next() && $result->getInt('Selbsttest') == 1) {
        $is_selftest = true;
    }

    // determine the absolute Stud.IP assets URL
    $picturepath = $GLOBALS['ASSETS_URL'];

    if (!preg_match('/\w*:/', $picturepath)) {
        if (isset($GLOBALS['ABSOLUTE_URI_STUDIP'])) {
            $picturepath = $GLOBALS['ABSOLUTE_URI_STUDIP'].$picturepath;
        } else {
            $picturepath = 'http://'.$GLOBALS['EXTERN_SERVER_NAME'].$picturepath;
        }
    }

    $picturepath .= 'images/';

    ################################
    #     construct templates      #
    ################################

    $template_header = $vipsTemplateFactory->open('exercises/wiki_exercise_header');
    if ($is_corrected) {
        $template = $o_exercise->getCorrectedWikiTemplate($solution_XML);
    } else {
        $template = $o_exercise->getWikiTemplate($solution_XML);
    }

    $template_footer = $vipsTemplateFactory->open('exercises/wiki_exercise_footer');

    $template_header->set_attributes(array(
        'assignment_id'     => $assignment_id,
        'exercise_position' => $exercise_position,
        'exercise_id'       => $exercise_id,
        'exercise_name'     => $exercise_name,
        'exercise_type'     => $exercise_title,
        'is_solved'         => $is_solved
        ));

    $template->set_attributes(array(
        'exercise_id'       => $exercise_id,
        'picturepath'       => $picturepath
        ));

    $template_footer->set_attributes(array(
        'is_corrected'     => $is_corrected,
        'is_selftest'      => $is_selftest,
        'lecturer_comment' => $lecturer_comment
        ));

    echo $template_header->render();
    echo $template->render();
    echo $template_footer->render();
}

/**
 * Fetches an exercise solution from the REQUEST environment and stores it in
 * the database.
 *
 * @returns true if the database operations were successful
 */
function vips_wiki_submit()
{
    global $vips_db,
        $g_solution_table,
        $g_solution_points_table,
        $g_assignment_table,
        $g_inAssignment_table;

    $system_id     = vips_get_string_param('system_id');
    $module_id     = vips_get_string_param('module_id');

    $assignment_id     = vips_get_int_param('assignment_id');
    $exercise_position = vips_get_int_param('exercise_position');
    $exercise_id       = vips_get_int_param('exercise_id');
    $wiki_solver_id    = vips_get_string_param('solver_id');
    $solver_id         = vips_wiki_set_user($wiki_solver_id);
    // $student_comment   = vips_get_string_param('student_comment');

    if (!vips_wiki_check_access($system_id, $module_id, $exercise_id)) {
        // TODO: improve error message here
        echo '<p><b>Zugriff verweigert!</b></p>';
        return;
    }

    // fetch exercise XML from database
    $sql = "SELECT *
            FROM vips_aufgabe, $g_inAssignment_table
            WHERE vips_aufgabe.ID = '$exercise_id' AND
                $g_inAssignment_table.vips_aufgabe = '$exercise_id'";
    $result = $vips_db->executeQuery($sql);

    if ($result->next()) {
        // note: this check can be removed when the exercise_id is always used in the wiki
        if ((empty($assignment_id) || $assignment_id == $result->getInt($g_assignment_table)) &&
            (empty($exercise_position) || $exercise_position == $result->getInt('Position'))) {
            $exercise_xml = $result->getString('Aufgabe');
            $exercise_type = $result->getString('URI');
            $assignment_id = $result->getInt($g_assignment_table);
        }
    } else {
        // TODO: improve error message here
        echo '<p><b>Keine Aufgabe gefunden!</b></p>';
        return;
    }

    if (vips_get_submit_param('reset')) {
        $sql = "UPDATE $g_solution_table SET isLastSolution = 'f'
                WHERE Aufgabenid = '$exercise_id' AND Userid = '$solver_id'";
        $vips_db->executeUpdate($sql);
        return;
    }

    require_once 'exercises/'.$exercise_type.'.php';
    $o_exercise = new $exercise_type($exercise_xml);
    $solution_xml = $o_exercise->genSolution($exercise_id);

    // set isLastSolution to 'f' on every older solution
    $sql = "UPDATE $g_solution_table
        SET isLastSolution = 'f'
        WHERE Aufgabenid     = '$exercise_id'
            AND Assignmentid = '$assignment_id'
            AND Userid       = '$solver_id'
        ";
    $vips_db->executeUpdate($sql);

    // insert new solution
    $sql = "INSERT INTO $g_solution_table
        SET Loesung        = '".addslashes($solution_xml)."',
            Aufgabenid     = '$exercise_id',
            Assignmentid   = '$assignment_id',
            Userid         = '$solver_id',
            Zeitpunkt      = NOW(),
            isLastSolution = 't'
        ";
    $vips_db->executeUpdate($sql);
    $id_gen = $vips_db->getIdGenerator();
    $solution_id = $id_gen->getId();  // stores the recently inserted id

    // selftest
    $sql = "SELECT Selbsttest FROM $g_assignment_table WHERE ID = '$assignment_id'";
    $result = $vips_db->executeQuery($sql);
    if ($result->next() && $result->getInt('Selbsttest') == 1) {  // if selftest: correct exercise
        $evaluation      = $o_exercise->evaluate($exercise_xml, $solution_xml);
        $mistake_comment = $o_exercise->getQuirk('mistake_comment');

        // compute reached points
        $sql = "SELECT Punkte
            FROM $g_inAssignment_table
            WHERE vips_aufgabe = '$exercise_id'";
        $result = $vips_db->executeQuery($sql);
        if ($result->next()) {
            $max_points = $result->getInt('Punkte');
        }
        $reached_points = round_to_half_point(($evaluation['percent'] * $max_points) / 100);
        $safe = $evaluation['safe'] == 1 ? 't' : 'f';

        // insert points
        $sql = "INSERT INTO $g_solution_points_table
            (idUser, idLoesung, Punkte, Safe)
            VALUES
            ('$solver_id', '$solution_id', '$reached_points', '$safe')
            ";
        $vips_db->executeQuery($sql);

        // if $mistake_comment is not empty and field 'Korrektur' is not already set, mark it for update
        if (!empty($mistake_comment)) {
            $sql = "SELECT Korrektur
                FROM $g_solution_table
                WHERE Loesungsid = '$solution_id'";
            $result = $vips_db->executeQuery($sql);
            if ($result->next() && $result->getString('Korrektur') == '') {
                $update_comment = true;
            }
        }

        // set isAlreadyCorrected in solution, optionally update Korrektur
        $sql = "UPDATE $g_solution_table
            SET isAlreadyCorrected = 1
                ".($update_comment ? ", Korrektur = '$mistake_comment'" : "")."
            WHERE Aufgabenid       = '$exercise_id'
                AND Assignmentid   = '$assignment_id'
                AND Userid         = '$solver_id'
                AND isLastSolution = 't'
            ";
        $vips_db->executeQuery($sql);
    }
}



/**
 * Imports a shortquiz exercise from wiki, converts it into a vips single choice
 * exercise, stores it in the database, links it to the assignment and returns
 * its position in the sheet.
 *
 * @returns The exercise position in the assignment sheet
 */
function vips_wiki_import_shortquiz()
{
    global $vips_db,
        $g_assignment_table,
        $g_inAssignment_table;

    $assignment_id = vips_get_int_param('assignment_id');
    $exercise_name = vips_get_string_param('exercise_name', true);
    $wiki_user_id  = vips_get_string_param('user_id');
    $user_id       = vips_wiki_set_user($wiki_user_id);

    require_once 'exercises/sc_exercise.php';
    $o_exercise      = new sc_exercise();
    $exercise_xml    = $o_exercise->getExerciseConstruct();
    $exercise_points = $o_exercise->getMaxPoints();

    // detect maximum position in the assignment
    $sql = "
        SELECT MAX(Position) AS pos
        FROM $g_inAssignment_table
        WHERE $g_assignment_table = '$assignment_id'
        ";
    $result = $vips_db->executeQuery($sql);
    if ($result->next()) {
        $exercise_position = $result->getInt('pos') + 1;
    }

    // store exercise in db
    $sql = "
        INSERT INTO vips_aufgabe
            (Name, Aufgabe, URI, Userid)
        VALUES
            ('".addslashes($exercise_name)."',
             '".addslashes($exercise_xml)."',
             'sc_exercise', '$user_id')
        ";
    $vips_db->executeQuery($sql);
    $vips_idgen = $vips_db->getIdGenerator();
    $exercise_id = $vips_idgen->getId();

    // link exercise to assignment
    $sql = "
        INSERT INTO $g_inAssignment_table
        (vips_aufgabe, $g_assignment_table, Punkte, Position)
        VALUES (
            '$exercise_id',
            '$assignment_id',
            '$exercise_points',
            '$exercise_position')
        ";
    $vips_db->executeQuery($sql);

    header('Content-Type: text/plain; charset=ISO-8859-1');
    echo $exercise_position;
}

/**
 * Fetch the id of an exercise specified by assignment id and position.
 *
 * @returns id of the exercise
 */
function vips_wiki_exercise_id()
{
    global $vips_db,
        $g_assignment_table,
        $g_inAssignment_table;

    $assignment_id     = vips_get_int_param('assignment_id');
    $exercise_position = vips_get_int_param('exercise_position');

    // fetch exercise info, type and points, determined by assignment and position
    $query = "SELECT vips_aufgabe FROM $g_inAssignment_table
              WHERE $g_assignment_table = '$assignment_id' AND
                    Position = '$exercise_position'";
    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
        $exercise_id = $result->get('vips_aufgabe');
    } else {
        $exercise_id = 0;
    }

    header('Content-Type: text/plain; charset=ISO-8859-1');
    echo $exercise_id;
}

?>
